# -*- coding: utf-8 -*-
# Copyright 2011 Thamini S.à.R.L    This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

try:
    from cStringIO import StringIO
    _hush_pyflakes = [ StringIO ]
except ImportError:
    from StringIO import StringIO

import logging
from lxml import etree

from report.report_sxw import report_sxw, rml_parse
from report.render.rml import rml
from report.render.rml2pdf.trml2pdf import _rml_doc
from report.render.rml2pdf.trml2pdf import _rml_flowable
from report.render.rml2pdf import utils
from report.render.rml2pdf import trml2pdf as server_trml2pdf

import trml2pdf_interface

# _rml_flowable:
#
# add support from extra flowable comming from _rml_doc.
# 'extra_flowables' will have priority over system flowables
class customized_rml_flowable(_rml_flowable):
    def __init__(self, doc, localcontext, images=None, path='.', title=None):
        super(customized_rml_flowable, self).__init__(doc, localcontext, images=images, path=path, title=title)
        if hasattr(doc, '_extra_flowables') and doc._extra_flowables:
            self._extra_flowables = doc._extra_flowables
        else:
            self._extra_flowables = {}

    def render(self, node_story):
        def process_story(node_story):
            sub_story = []
            for node in utils._child_get(node_story, self):
                tag = node.tag
                if tag == etree.Comment:
                    node.text = ''
                    continue
                if tag in self._extra_flowables:
                    flow = self._extra_flowables[tag](self, node)
                else:
                    flow = self._flowable(node)
                if flow:
                    if isinstance(flow,list):
                        sub_story = sub_story + flow
                    else:
                        sub_story.append(flow)
            return sub_story
        return process_story(node_story)

# monkey patch the server _rml_flowable class with our, support custom
# extra_flowables :)
server_trml2pdf._rml_flowable = customized_rml_flowable

def customized_parseNode(rml, localcontext=None,fout=None, images=None, path='.',title=None, extra_flowables=None):
    node = etree.XML(rml)
    if localcontext is None:
        localcontext = {}
    if images is None:
        images = {}
    r = _rml_doc(node, localcontext, images, path, title=title)
    r._extra_flowables = extra_flowables
    #try to override some font mappings
    try:
        from report.render.rml2pdf.customfonts import SetCustomFonts
        SetCustomFonts(r)
    except ImportError:
        # means there is no custom fonts mapping in this system.
        pass
    except Exception:
        logging.getLogger('report').warning('Cannot set font mapping', exc_info=True)
        pass
    fp = StringIO()
    r.render(fp)
    return fp.getvalue()

class customized_rml(rml):
    def __init__(self, rml, localcontext = None, datas=None, path='.', title=None,
                        extra_flowables=None):
        super(customized_rml, self).__init__(rml, localcontext, datas, path, title)
        self.extra_flowables = extra_flowables

    def _render(self):
        return customized_parseNode(self.rml, self.localcontext,
                                    images=self.bin_datas, path=self.path,
                                    title=self.title,
                                    extra_flowables=self.extra_flowables)

class customized_report_sxw(report_sxw):
    _extra_flowables = {} # contains raw parser: { 'tag': tag_parser}
    _use_extra_flowables = [] # list of extra flowable to automatically register for this report

    def __init__(self, name, table, rml=False, parser=rml_parse, header=True, store=False):
        super(customized_report_sxw, self).__init__(name, table, rml, parser, header, store)
        if self._use_extra_flowables:
            for tag_name in self._use_extra_flowables:
                try:
                    tag_parser = trml2pdf_interface.get_tag(tag_name)
                    self._extra_flowables[tag_name] = tag_parser
                except Exception:
                    pass

    def create_pdf(self, rml, localcontext = None, logo=None, title=None):
        if not localcontext:
           localcontext = {}
        localcontext.update({'internal_header':self.internal_header})
        if logo:
            self.bin_datas['logo'] = logo
        else:
            if 'logo' in self.bin_datas:
                del self.bin_datas['logo']
        obj = customized_rml(rml, localcontext, self.bin_datas, self._get_path(), title,
                             extra_flowables=self._extra_flowables)
        obj.render()
        return obj.get()

class stored_checkbox_position_report_sxw(customized_report_sxw):
    """
    Report SXW which store checkbox position and participation_id
    (currently only working in the training's exam_sheet report)
    """

    def create_single_pdf(self, cr, uid, ids, data, report_xml, context=None):
        if not context:
            context={}
        logo = None
        context = context.copy()
        title = report_xml.name
        rml = report_xml.report_rml_content
        # if no rml file is found
        if not rml:
            return False
        rml_parser = self.parser(cr, uid, self.name2, context=context)
        objs = self.getObjects(cr, uid, ids, context)
        rml_parser.set_context(objs, data, ids, report_xml.report_type)
        processed_rml = etree.XML(rml)
        if report_xml.header:
            rml_parser._add_header(processed_rml, self.header)
        processed_rml = self.preprocess_rml(processed_rml,report_xml.report_type)
        if rml_parser.logo:
            logo = base64.decodestring(rml_parser.logo)
        create_doc = self.generators[report_xml.report_type]
        localcontext = rml_parser.localcontext
        pdf = create_doc(etree.tostring(processed_rml), localcontext, logo, title.encode('utf8'))
        if context.has_key('checkboxes_position'):
            context['checkboxes_position'].extend(localcontext['checkboxes_position'])
        if context.has_key('checkboxes_context'):
            context['checkboxes_context'].extend(localcontext['checkboxes_context'])
        if context.has_key('participation_id'):
            context['participation_id'].extend(localcontext['participation_id'])
        return (pdf, report_xml.report_type)

