# -*- encoding: utf-8 -*-
############################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    Copyright (C) 2008-2009 AJM Technologies S.A. (<http://www.ajm.lu). All Rights Reserved
#    Copyright (C) 2010-2011 (<http://www.thamini.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################################

from osv import osv
from osv import fields
import tools
from tools.func import partial

FMT = '%Y-%m-%d'

class act_window(osv.osv):

    _inherit = 'ir.actions.act_window'
    _columns = {
        'domain': fields.char('Domain Value', size=512),    # bigger size...
    }

act_window()


class training_session(osv.osv):
    _inherit = 'training.session'

    def _has_confirmed_stakeholder_compute(self, cr, uid, ids, fields, args, context=None):
        result = dict.fromkeys(ids, 0)
        for obj in self.browse(cr, uid, ids, context=context):
            result[obj.id] = all(any(part.state == 'accepted' for part in seance.contact_ids)
                              for seance in obj.seance_ids)
        return result

    _columns = {
        'has_confirmed_stakeholder' : fields.function(_has_confirmed_stakeholder_compute, method=True, string='Stakeholder', type='boolean'),
    }

training_session()

def _generate_sql_working_days(working_days):
    positive = working_days > 0
    sign = ['-', '+'][positive]

    return """(SELECT date
                 FROM (SELECT current_date %s s.t as date
                         FROM generate_series(0, 365) as s(t)
                      ) AS days
                WHERE NOT EXISTS(SELECT 1
                                   FROM training_holiday_period
                                  WHERE days.date BETWEEN date_start AND date_stop
                                )
                LIMIT %d)
    """ % (sign, abs(working_days) + 1)


class training_session(osv.osv):
    _inherit = 'training.session'

    def search(self, cr, uid, domain, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}
        _super = partial(super(training_session, self).search, cr, uid, offset=offset, limit=limit, order=order, context=context, count=count)

        min_limit_confirmed = context.get('min_limit_confirmed', False)
        if min_limit_confirmed:
            ids = _super(order='date desc', count=False)
            ids = [session.id for session in self.browse(cr, uid, ids, context=context) if session.min_limit < (session.manual and session.participant_count_manual or session.participant_count)]
            if count:
                return len(ids)
            return ids

        working_days = context.get('working_days', False)

        if working_days is not False:   # 0 is for today
            cr.execute("select id from training_session where date::date in %s" % (_generate_sql_working_days(working_days),))

            ids = [x[0] for x in cr.fetchall()]
            return _super([('id', 'in', ids)]+domain)
        return _super(domain)

training_session()

class training_seance(osv.osv):
    _inherit = 'training.seance'

    def _valid_participant_count(self, cr, uid, ids, fn, args, context=None):
        result = dict.fromkeys(ids, False)
        for seance in self.browse(cr, uid, ids, context=context):
            count = ['participant_count_manual', 'participant_count'][seance.manual]
            result[seance.id] = seance[count] in range(seance.min_limit, seance.max_limit + 1)
        return result

    def _course_has_attachments(self, cr, uid, ids, fn, args, context=None):
        result = dict.fromkeys(ids, False)
        for seance in self.browse(cr, uid, ids, context=context):
            result[seance.id] = seance.kind == 'standard' and len(seance.course_id.attachment_ids) > 0 or False
        return result

    def _course_has_attachments_search(self, cr, uid, obj, name, domain, context=None):
        if not len(domain):
            return []

        val = bool(domain[0][2])
        if domain[0][1] != '=':
            val = not val

        _not = ['NOT', ''][val]
        query = """SELECT distinct(id)
                     FROM training_seance s
                    WHERE %s EXISTS(SELECT 1
                                      FROM ir_attachment a
                                     WHERE a.res_model = 'training.course'
                                       AND a.res_id = s.course_id
                                       AND a.type = 'course_material'
                                    )
                """ % ( _not,)
        cr.execute(query)
        ids = [i for (i,) in cr.fetchall()]
        if not ids:
            return [('id','=','0')]
        return [('id', 'in', ids)]


    _columns = {
        'valid_participant_count': fields.function(_valid_participant_count, method=True, string='Valid Participant Count', store=True),
        'course_has_attachments': fields.function(_course_has_attachments, method=True, fnct_search=_course_has_attachments_search, string='Couse Has Attachements?'),
        'course_forced_nosupport': fields.related('course_id', 'forced_nosupport', type='boolean', string='Course Forced No Support'),
    }

    def search(self, cr, uid, domain, offset=0, limit=None, order=None,context=None, count=False):
        if context is None:
            context = {}
        _super = partial(super(training_seance, self).search, cr=cr, uid=uid, domain=domain, offset=offset, limit=limit, order=order, context=context, count=count)

        working_days = context.get('working_days', False)

        if working_days is not False:   # 0 is for today
            cr.execute("select id from training_seance where date::date in %s" % (_generate_sql_working_days(working_days),))
            ids = [x[0] for x in cr.fetchall()]
            return _super(domain=[('id', 'in', ids)]+domain)

        return _super()

training_seance()

#- Zone de tableaux de board
#---- Par rapport à la première date d'une séance (donc début de la session)----
#- Lister les seances qui commencent dans moins de 21 jours et qui remplissent au minimum l'une des règles suivantes :
#- Statut > Open
#- Formateur non confirmé
#- Support de cours non présent (pour au min. une séance liée)
#- Pas de lieux (classroom)
#- Support de cours en cours de mise à jours
#- Seuil min. non atteint
#- Seuil max. dépassé



class training_participation_stakeholder(osv.osv):

    _inherit = 'training.participation.stakeholder'

    def search(self, cr, uid, domain, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}
        condition = context.get('condition', False)
        if condition and condition in ('yes', 'no'):
            # To check the condition for the payment of the lecturer
            # 1. Check the evaluation form
            # 2. Check the presence list
            pass

        return super(training_participation_stakeholder, self).search(cr, uid, domain, offset=offset, limit=limit, order=order, context=context, count=count)

training_participation_stakeholder()

class training_course_inactive_lecturer(osv.osv):
    _name = 'training.course.inactive.lecturer'
    _table = 'training_course_inactive_lecturer'

    #_auto = False
    _rec_name = 'contact_id'
    _columns = {
        'contact_id' : fields.many2one('res.partner.job', 'Contact', readonly=True),
        'course_id' : fields.many2one('training.course', 'Course', readonly=True),
    }

    #def init(self, cr):
    #    tools.drop_view_if_exists(cr, 'training_course_inactive_lecturer')
    #    cr.execute('CREATE OR REPLACE VIEW training_course_inactive_lecturer AS ( '
    #                    'SELECT 1 as id, rpc.id as contact_id, tccr.course_id as course_id '
    #                   'FROM res_partner_contact rpc, training_course_job_rel tccr, res_partner_job rpj, res_partner_function rpf '
    #                   'WHERE rpf.code = %s '
    #                   'AND rpj.function_id = rpf.id '
    #                   'AND rpj.state != %s '
    #                   'AND rpj.contact_id = rpc.id '
    #                   'AND rpc.id = tccr.contact_id )',
    #               ('TR', 'current',)
    #              )

training_course_inactive_lecturer()

class training_participation_stakeholder_doublon(osv.osv):
    _name = 'training.participation.stakeholder.doublon'

    _table = 'training_participation_stakeholder_doublon'

    _auto = False

    _columns = {
        'participation_id' : fields.many2one('training.participation.stakeholder', 'Participation', readonly=True, select=1),
        'contact_id' : fields.many2one('res.partner.contact', 'Contact', readonly=True, select=1),
        'seance_id' : fields.many2one('training.seance', 'Seance', readonly=True, select=1),
        'date' : fields.datetime('Date', readonly=True, select=1),
        'state' : fields.selection([('draft', 'Draft'),
                                    ('accepted', 'Accepted'),
                                    ('refused', 'Refused'),
                                    ('cancelled', 'Cancelled'),
                                    ('done', 'Done'),
                                   ], 'State', readonly=True, select=1),
    }

    def init(self, cr):
        tools.drop_view_if_exists(cr, 'training_participation_stakeholder_doublon')
        cr.execute(
            """CREATE OR REPLACE VIEW training_participation_stakeholder_doublon AS (
            SELECT tps.id as id, tps.id as participation_id, tps.contact_id as contact_id, tps.seance_id, tps.date as date, tps.state as state
            FROM training_participation_stakeholder tps, training_seance s
            WHERE tps.state IN ('requested', 'confirmed', 'draft', 'accepted')
            AND s.id = tps.seance_id
            AND EXISTS (
                SELECT 1
                FROM training_participation_stakeholder tps2, training_seance this_seance
                WHERE tps2.state IN ('requested', 'confirmed', 'draft', 'accepted')
                AND this_seance.id = tps2.seance_id
                AND tps.date >= CURRENT_TIMESTAMP
                AND tps.id != tps2.id
                AND tps.contact_id = tps2.contact_id
                AND (
                    (this_seance.date BETWEEN s.date AND s.date + (s.duration * interval '1 hour'))
                    OR (this_seance.date + (this_seance.duration * interval '1 hour') BETWEEN s.date AND s.date + (s.duration * interval '1 hour'))
                    OR (this_seance.date < s.date AND this_seance.date + (this_seance.duration * interval '1 hour') > s.date + (s.duration * interval '1 hour'))
                    )
                )
            AND tps.date >= CURRENT_TIMESTAMP
            ORDER BY tps.contact_id, tps.seance_id, tps.state, tps.date DESC
            )
            """)

training_participation_stakeholder_doublon()

class training_participation_doublon(osv.osv):
    _name = 'training.participation.doublon'

    _table = 'training_participation_doublon'

    _auto = False

    _columns = {
        'participation_id' : fields.many2one('training.participation', 'Participation', readonly=True, select=1),
        'contact_id' : fields.many2one('res.partner.contact', 'Contact', readonly=True, select=1),
        'seance_id' : fields.many2one('training.seance', 'Seance', readonly=True, select=1),
        'date' : fields.datetime('Date', readonly=True, select=1),
        'subscription_line_id' : fields.many2one('training.subscription.line', 'Subscription Line', readonly=True, select=1),
        'exam_id' : fields.many2one('training.course', 'Exam', readonly=True, select=1),
    }

    def init(self, cr):
        tools.drop_view_if_exists(cr, 'training_participation_doublon')
        cr.execute(
            """CREATE OR REPLACE VIEW training_participation_doublon AS (
            SELECT tp1.id as id, tp1.id as participation_id, tp1.contact_id, tp1.seance_id, tp1.date, tp1.subscription_line_id, tp1.course_questionnaire_id as exam_id
            FROM training_participation tp1, training_seance ts1
            WHERE EXISTS(
                    SELECT 1 FROM training_participation tp2, training_seance ts2
                    WHERE tp1.id != tp2.id
                    AND tp1.contact_id = tp2.contact_id
                    AND tp1.date = tp2.date
                    AND tp2.date >= CURRENT_TIMESTAMP
                    AND ts2.kind IN ('standard', 'intra')
                    AND ts2.id = tp2.seance_id
            )
            AND tp1.seance_id = ts1.id
            AND tp1.date >= CURRENT_TIMESTAMP
            AND ts1.kind IN ('standard', 'intra')
            ORDER BY tp1.contact_id ASC
            )
            """)

training_participation_doublon()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

