# -*- encoding: utf-8 -*-
############################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    Copyright (C) 2008-2009 AJM Technologies S.A. (<http://www.ajm.lu). All Rights Reserved
#    Copyright (C) 2010-2011 Thamini S.à.R.L (<http://www.thamini.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################################

import string
from random import *
import unicodedata
import time

from osv import osv
from osv import fields
import tools
from tools.translate import _

class ir_action_act_window_portal_training(osv.osv):
    _inherit = 'ir.actions.act_window'

    def for_xml_id(self, cr, uid, module, xml_id, context=None):
        if module == 'base' and xml_id == 'res_request-act':
            portal_obj = self.pool.get('res.portal')
            if portal_obj.search(cr, 1, [('users','in',[uid])]):
                return {} # return empty action - not authorized to use requests
        return super(ir_action_act_window_portal_training, self).for_xml_id(cr, uid, module, xml_id, context=context)

ir_action_act_window_portal_training()

class training_contact_course_portal(osv.osv):
    """Contact By Course"""
    _name = 'training.contact.course.portal'
    _auto = False

    _columns = {
        'course_id' : fields.many2one('training.course', 'Course', readonly=True),
        'category_id' : fields.related('course_id', 'category_id', type='many2one', relation='training.course_category', string='Product Line', readonly=True),
        'type_id' : fields.related('course_id', 'course_type_id', type='many2one', relation='training.course_type', string='Type', readonly=True),
        'lang_id' : fields.related('course_id', 'lang_id', type='many2one', relation='res.lang', string='Lang', readonly=True),
        'contact_id' : fields.many2one('res.partner.contact', 'Contact', readonly=True),
    }

    def init(self, cr):
        tools.drop_view_if_exists(cr, 'training_contact_course_portal')
        cr.execute("CREATE OR REPLACE VIEW training_contact_course_portal AS ( "
                   "SELECT course.id as id, course.id AS course_id, contact.id AS contact_id "
                   "FROM training_course course, training_course_job_rel rel, res_partner_job job, res_partner_contact contact "
                   "WHERE course.id = rel.course_id "
                   "AND job.id = rel.job_id "
                   "AND contact.id = job.contact_id )")

training_contact_course_portal()


class portal_training_lecturer(osv.osv):
    _inherit = 'res.users'

    _columns = {
        'partner_id': fields.related('address_id', 'partner_id', string="Partner", type='many2one', relation='res.partner', readonly=True),
        'contact_id': fields.many2one('res.partner.contact', 'Contact'),
    }

portal_training_lecturer()

def _partner_from_uid(obj, cr, uid, context=None):
    return False

class training_subscription(osv.osv):
    _inherit = 'training.subscription'

    def _partner_default(self, cr, uid, context=None):
        user = self.pool.get('res.users').browse(cr, uid, uid, context)
        if user and user.partner_id:
            return user.partner_id.id
        return _partner_from_uid(self, cr, uid, context)

    def _address_default(self, cr, uid, context=None):
        part_id  = _partner_from_uid(self, cr, uid, context)
        return self.on_change_partner(cr, uid, [], part_id).get('value', {}).get('address_id', False)

    _defaults = {
        'partner_id': _partner_default,
        'address_id': _address_default,
    }

training_subscription()

class portal_projected_purchase_lines(osv.osv):
    _name = 'portal.projected.purchase.lines'
    _auto = False

    def _notes_compute(self, cr, uid, ids, fieldnames, args, context=None):
        res = dict.fromkeys(ids, '')
        for obj in self.browse(cr, uid, ids, context=context):
            po_line = obj.purchase_line_id
            note = []
            if po_line.attachment_id:
                for component in po_line.attachment_id.component_ids:
                    note.append("%sx %s" % (component.product_qty, component.product_id.name,))

                if po_line.attachment_id.support_note:
                    note.append("==================================")
                    note.append(po_line.attachment_id.support_note)

            res[obj.id] = "\n".join(note)

        return res

    _columns = {
        'product_id' : fields.many2one('product.product', 'Product', readonly=True, select=1),
        'partner_id' : fields.many2one('res.partner', 'Partner', readonly=True, select=1),
        'seance_id' : fields.many2one('training.seance', 'Seance', readonly=True, select=1),
        'course_id': fields.many2one('training.course', 'Course', readonly=True, select=1),
        'duration' : fields.float('Duration', readonly=True),
        'date' : fields.datetime('Date', readonly=True, select=1),
        'purchase_line_id' : fields.many2one('training.seance.purchase_line', 'Purchase Line', readonly=True, select=1),
        'purchase_line_qty' : fields.related('purchase_line_id', 'proc_qty', string="Estimated Quantity", type='integer', readonly=True, select=1),
        'notes' : fields.function(_notes_compute, method=True, type="text", string="Notes", readonly=True),
    }

    def init(self, cr):
        tools.drop_view_if_exists(cr, 'portal_projected_purchase_lines')

        cr.execute(
            "CREATE OR REPLACE VIEW portal_projected_purchase_lines AS ( "
            "SELECT tspl.id as id, pr.id as product_id, partner.id as partner_id, ts.id as seance_id, tspl.course_id, ts.date, ts.duration, tspl.id as purchase_line_id "
            "FROM "
            "training_seance_purchase_line tspl, "
            "training_seance ts, "
            "product_product pr, "
            "product_supplierinfo ps, "
            "res_partner partner "
            "WHERE tspl.seance_id = ts.id "
            "AND tspl.procurement_id is null "
            "AND ts.state = 'opened' "
            "AND pr.id = tspl.product_id "
            "AND ts.date between current_date AND current_date + 20 "
            "AND ps.product_id = tspl.product_id "
            "AND ps.name = partner.id "
            ")"
        )

portal_projected_purchase_lines()

class training_seance(osv.osv):
    _inherit = 'training.seance'

    def _attachment_compute(self, cr, uid, ids, fieldnames, args, context=None):
        res = dict.fromkeys(ids, [])
        proxy = self.pool.get('ir.attachment')

        for obj in self.browse(cr, uid, ids, context=context):
            if obj.course_id:
                res[obj.id] = proxy.search(cr, uid,
                                           [('res_model', '=', 'training.course'),
                                            ('res_id', '=', obj.course_id.id),
                                            ('type', 'in', ['lecturer_material','course_material']),
                                           ],
                                           context=context
                                          )

        return res

    def _user_is_stakeholder(self, cr, uid, ids, fieldname, arg, context=None):
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        result = dict.fromkeys(ids, False)
        for this in self.browse(cr, uid, ids, context=context):
            result[this.id] = any(sh.contact_id == user.contact_id and sh.state == 'confirmed' for sh in this.contact_ids)
        return result

    def _user_is_stakeholder_search(self, cr, uid, obj, name, domain, context=None):
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        if not len(domain):
            return []

        val = bool(domain[0][2])
        if domain[0][1] != '=':
            val = not val

        _not = ['NOT', ''][val]
        query = """SELECT distinct(id)
                     FROM training_seance s
                    WHERE %s EXISTS(SELECT 1
                                      FROM training_participation_stakeholder sh
                                     WHERE sh.seance_id = s.id
                                       AND sh.state in ('done', 'accepted')
                                       AND sh.contact_id = %%s
                                    )
                """ % ( _not,)
        cr.execute(query, (user.contact_id.id or 0,))
        ids = [i for (i,) in cr.fetchall()]
        if not ids:
            return [('id','=','0')]
        return [('id', 'in', ids)]

    def search(self, cr, uid, domain, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}
        if context.get('portal_search_reverse_order', False):
            order = 'date DESC'
        return super(training_seance, self).search(cr, uid, domain, offset=offset,
                            limit=limit, order=order,context=context, count=count)

    def read(self, cr, user, ids, fields=None, context=None, load='_classic_read'):
        if context is None:
            context = {}
        if context.get('portal_search_reverse_order', False):
            res = super(training_seance, self).read(cr, user, ids, fields=fields, context=context,
                        load=load)
            # we juste reverse the list. we don't have problem with offset and limit here because
            # the search() method take care of that.
            res.reverse()
            return res
        return super(training_seance, self).read(cr, user, ids, fields=fields, context=context, load=load)

    #def _has_conf_pol_compute(self, cr, uid, ids, fn, args, context=None):
    #    res = dict.fromkeys(ids, False)
    #    for seance in self.browse(cr, uid, ids, context=context):
    #        res[seance.id] = any(pol.state == 'confirmed' for pol in seance.purchase_order_line_ids)
    #        #res[seance.id] = any(pl.procurement_id.state == 'running' for pl in seance.purchase_line_ids)
    #    return res

    #def _has_conf_pol_srch(self, cr, uid, obj, fn, domain, context=None):
    #    if not len(domain):
    #        return []
    #
    #    val = bool(domain[0][2])
    #    if domain[0][1] != '=':
    #        val = not val
    #
    #    _not = ['NOT', ''][val]
    #    query = """SELECT distinct(id)
    #                 FROM training_seance s
    #                WHERE %s EXISTS(SELECT 1
    #                                  FROM purchase_order po
    #                            INNER JOIN purchase_order_line pol ON pol.order_id = po.id
    #                                 WHERE po.seance_id = s.id
    #                                   AND pol.state = 'confirmed'
    #                                )
    #            """ % ( _not,)
    #    cr.execute(query)
    #    ids = [i for (i,) in cr.fetchall()]
    #    if not ids:
    #        return [('id','=','0')]
    #    return [('id', 'in', ids)]


    _columns = {
        'attachment_ids' : fields.function(_attachment_compute, method=True, string='Material', type='one2many', relation='ir.attachment'),

        'user_is_stakeholder': fields.function(_user_is_stakeholder, method=True, type="boolean", readonly=True, fnct_search=_user_is_stakeholder_search),
        #'has_confirmed_pol': fields.function(_has_conf_pol_compute,
        #                                    method=True,
        #                                    fnct_search=_has_conf_pol_srch,
        #                                    type='boolean',
        #                                    string='Has Confirmed Purchase Order Lines',
        #                                    readonly=True),

        #'purchase_order_line_ids': fields.one2many('purchase.order.line', 'seance_id', 'Purchase Order Lines', readonly=True),
    }

training_seance()


class participation(osv.osv):
    _inherit = 'training.participation'

    _columns = {
        'seance_state': fields.related('seance_id', 'state', readonly=True, type='selection',
                                selection=[
                                    ('opened', 'Opened'),
                                    ('confirmed', 'Confirmed'),
                                    ('inprogress', 'In Progress'),
                                    ('closed', 'Closed'),
                                    ('cancelled', 'Cancelled'),
                                    ('done', 'Done')]),
    }

    def switch_presence(self, cr, uid, ids, context=None):
        for this in self.browse(cr, uid, ids, context=context):
            this.write({'present': not this.present}, context=context)
        return True

participation()

class training_session(osv.osv):
    _inherit = 'training.session'

    def search(self, cr, uid, domain, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}
        from_subscription = ('from_subscription' in context) and context.pop('from_subscription') or False

        result = super(training_session, self).search(cr, uid, domain, offset=offset, limit=limit, order=order, context=context, count=count)
        if isinstance(result, (int,long)):
            result = [ result ]

        if from_subscription:
            imd_proxy = self.pool.get('ir.model.data')
            imd_ids = imd_proxy.search(cr, uid, [('module', '=', 'portal_training'),('name', '=', 'res_group_portal_partner'),('model', '=', 'res.groups')], context=context)
            if imd_ids:
                imd = imd_proxy.browse(cr, uid, imd_ids[0], context=context)
                if imd:
                    group = self.pool.get(imd.model).browse(cr, uid, imd.res_id, context=context)
                    user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
                    if any(group_id.id == group.id for group_id in user.groups_id):
                        result2 = super(training_session, self).search(cr, uid, [('kind', '!=', 'intra'),
                                                                                 ('date', '>', time.strftime('%Y-%m-%d')),
                                                                                 ('state', 'in', ('opened', 'opened_confirmed')),
                                                                                 ('id', 'in', result),
                                                                                ])

                        return result2

        return result

training_session()

class training_create_portal_user(osv.osv_memory):
    _name = 'training.create.portal.user.wizard'

    _columns = {
        #'partner_id' : fields.many2one('res.partner', 'Partner'),
        'job_id' : fields.many2one('res.partner.job', 'Contact'),

        'email' : fields.char('Email', size=128),
        'notify_by_email' : fields.boolean('Notify By Email'),

        'username' : fields.char('Username', size=16, required=True),
        'password' : fields.char('Password', size=16, required=True),

        'portal_id' : fields.many2one('res.portal', 'Portal', required=True),
    }

    _defaults = {
        'notify_by_email' : lambda *a: True,
        'portal' : lambda *a: 'lecturer',
    }

    def _strip_accents(self, s):
        if isinstance(s, str):
            s = unicode(s, 'utf-8')
        return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn'))

    def _compute_username(self, cr, uid, contact_brec, context=None):
        first_name = ''
        if contact_brec.first_name:
            first_name = self._strip_accents(contact_brec.first_name[0])
        return (first_name + self._strip_accents(contact_brec.name)).lower()

    def _generate_random_password(self):
        """ Create a password of random length between 8 and 16
            characters long, made up of numbers and letters.
        """
        # The characters to make up the random password
        chars = string.ascii_letters + string.digits + "_!-[]{}#"
        return "".join(choice(chars) for x in range(8))

    def onchange_job_id(self, cr, uid, ids, job_id, context=None):
        if not job_id:
            ocv = {
                'value': {
                    'username': '',
                    'password': '',
                    'email': '',
                }
            }
        else:
            job = self.pool.get('res.partner.job').browse(cr, uid, job_id, context=context)
            ocv = {
                'value': {
                    'username': self._compute_username(cr, uid, job.contact_id, context=context),
                    'password': self._generate_random_password(),
                    'email': job.email or job.contact_id.email or '',
                },
            }
        return ocv

#    def onchange_partner_id(self, cr, uid, ids, partner_id, context=None):
#        val = partner_id and [ partner_id ] or []
#        return {
#            'domain': {
#                'job_id': [('address_id','!=',False),('address_id.partner_id','in',val)],
#            }
#        }

    def button_generate_random_password(self, cr, uid, ids, context=None):
        self.write(cr, uid, [ ids[0] ], {'password': self._generate_random_password()}, context=context)
        return True

    def close_cb(self, cr, uid, ids, context=None):
        return {'type' : 'ir.actions.act_window_close'}

    def create_cb(self, cr, uid, ids, context=None):
        if not ids:
            return False
        this = self.browse(cr, uid, ids[0], context=context)
        if this.notify_by_email and len(this.email.strip()) == 0:
            raise osv.except_osv(_('Error'),
                                 _("""You have specifed an empty email !"""))

        proxy = self.pool.get('res.users')
        user_ids = proxy.search(cr, uid, [('login', '=', this.username)], context=context)
        if not user_ids:
            values = {
                'login' : this.username,
                'password' : this.password,
                'name' : this.job_id and this.job_id.contact_id and this.job_id.contact_id.name_get()[0][1] or False,
                'groups_id' : [(6, 0, [this.portal_id.group_id.id])],
                'address_id': this.job_id and this.job_id.address_id and this.job_id.address_id.id or False,
                'contact_id': this.job_id and this.job_id.contact_id and this.job_id.contact_id.id or False,
            }

            user_id = proxy.create(cr, uid, values, context=context)

            if this.notify_by_email:
                email_proxy = self.pool.get('training.email')
                user_account = proxy.browse(cr, uid, user_id, context=context)
                res = email_proxy.send_email(cr, uid, 'portal_create_user', 'po', to=this.email, context=context, user_account=user_account, user_login=values['login'], user_password=values['password'])

            proxy_data = self.pool.get('ir.model.data')
            view_ids = proxy_data.search(cr, uid, [
                ('name', '=', 'view_users_form'),
                ('module', '=', 'base'),
                ('model', '=', 'ir.ui.view')
            ], context=context)

            view_id = proxy_data.browse(cr, uid, view_ids[0], context=context).res_id

            return {
                'type' : 'ir.actions.act_window',
                'res_model' : 'res.users',
                'res_id' : user_id,
                'view_id' : [view_id],
                'name' : 'Portal User',
                'view_type' : 'form',
                'view_mode' : 'form',
            }
        else:
            raise osv.except_osv(_('Warning'),
                                 _("""This login already exists in the system. Please Choose an other username"""))

training_create_portal_user()

class purchase_order_line(osv.osv):
    _inherit = 'purchase.order.line'

    _columns = {
        'supplier_note' : fields.text('Supplier Note'),
    }

purchase_order_line()

class training_participation_stakeholder_request(osv.osv):
    _inherit = 'training.participation.stakeholder.request'

    def search(self, cr, uid, domain, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}
        if context.get('portal_search_reverse_order', False):
            order = 'date DESC'
        return super(training_participation_stakeholder_request, self).search(cr, uid, domain, offset=offset,
                            limit=limit, order=order,context=context, count=count)

    def read(self, cr, user, ids, fields=None, context=None, load='_classic_read'):
        if context is None:
            context = {}
        if context.get('portal_search_reverse_order', False):
            res = super(training_participation_stakeholder_request, self).read(cr, user, ids, fields=fields, context=context,
                        load=load)
            res_dict = dict([(x['id'],x) for x in res])
            res = [ res_dict[id] for id in ids ]
            return res
        return super(training_participation_stakeholder_request, self).read(cr, user, ids, fields=fields, context=context, load=load)

training_participation_stakeholder_request()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
