from osv import osv
from osv import fields

from tools.translate import _

class training_subscription_basket(osv.osv):
    _name = 'training.subscription.basket'

    def name_get(self, cr, uid, ids, context=None):
        res = []
        for obj in self.browse(cr, uid, ids, context=context):
            value = "%s - %s %s" % (obj.partner_id.name,
                                    obj.contact_id.first_name,
                                    obj.contact_id.name)
            res.append((obj.id, value))
        return res

    _columns = {
        'partner_id' : fields.many2one('res.partner', 'Partner', required=True),
        'job_id' : fields.many2one('res.partner.job', 'Job', domain="[('name', '=', partner_id)]", required=True),
        'contact_id' : fields.related('job_id', 'contact_id', type='many2one', relation='res.partner.contact', string='Contact', store=True),

        #'period_start' : fields.date('Period Start'),
        #'period_stop' : fields.date('Period Stop'),

        #'type' : fields.selection([('limited', 'Limited Subscriptions'),
        #                           ('unlimited', 'Unlimited Subscriptions')],
        #                          'Type', required=True),

        #'number_of_sub' : fields.integer('Number of Available Subscriptions'),

        'active' : fields.boolean('Active'),

        'subscription_line_ids' : fields.one2many('training.subscription.line', 'basket_id', 'Subscriptions', readonly=True),
    }

    _sql_constraints = [
        ('uniq_basket', 'unique(partner_id, contact_id)', """The partner can fill a basket only for this contact"""),
    ]

    _defaults = {
        #'type' : lambda *a: 'unlimited',
        'active' : lambda *a: 1,
        #'number_of_sub' : lambda *a: 1,
    }

training_subscription_basket()

class training_subscription_line(osv.osv):
    _inherit = 'training.subscription.line'

    _columns = {
        'basket_id' : fields.many2one('training.subscription.basket', 'Basket'),
    }

training_subscription_line()

class training_subscription_participant_wizard(osv.osv_memory):
    _name = 'training.subscription.participant.wizard'

    _columns = {
        'basket_id' : fields.many2one('training.subscription.basket', 'Basket', required=True),

        'session_id' : fields.many2one('training.session', 'Session',
                                       domain="[('kind', '!=', 'intra'),('state', 'in', ('opened', 'opened_confirmed')),('date', '>', current_date)]",
                                       required=True),

        'state' : fields.selection([('init', 'Init'),
                                    ('subscribed', 'Subscribed')], 'State', readonly=True, required=True),
    }

    def default_get(self, cr, uid, fields, context=None):
        basket_id = False

        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)

        if user.contact_id:
            proxy = self.pool.get('training.subscription.basket')
            ids = proxy.search(cr, uid,
                         [
                             ('contact_id', '=', user.contact_id.id),
                             ('active', '=', 1)
                         ], context=context
                        )

            if len(ids) > 0:
                basket_id = ids[0]
            else:
                raise osv.except_osv(_('Warning'),
                                     _('You can not use this assistant because you do not have any basket'))

        defaults = {
            'state' : 'init',
            'basket_id' : basket_id,
        }
        return defaults

    def subscribe_cb(self, cr, uid, ids, context=None):
        # We check if the participant can subscribe to a session
        this = self.browse(cr, uid, ids[0], context=context)

        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)

        values = {
            'name' : '/',
            'partner_id' : this.basket_id.partner_id.id,
            'origin' : 'User Portal: %s "%s"' % (user.login, user.name)
        }

        proxy = self.pool.get('training.subscription')

        vals = proxy.on_change_partner(cr, uid, [], this.basket_id.partner_id.id)

        values.update(vals['value'])

        subscription_id = proxy.create(cr, uid, values, context=context)

        values = {
            'session_id' : this.session_id.id,
            'job_id' : this.basket_id.job_id.id,
            'subscription_id' : subscription_id,
            'basket_id' : this.basket_id.id,
        }
        self.pool.get('training.subscription.line').create(cr, uid, values, context=context)

        return self.write(cr, uid, ids, {'state' : 'subscribed'}, context=context)

training_subscription_participant_wizard()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
