# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import orm
from osv import fields

import tools
import time
import os
import re
import xlwt

from zipfile import PyZipFile, ZIP_DEFLATED
import base64
try:
    import cStringIO as StringIO
except:
    import StringIO

sort_position = lambda a, b: cmp(a[0], b[0])

class training_course(osv.osv):
    _inherit = 'training.course'

    def search(self, cr, uid, domain, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}

        has_validated_questionnaire = context.get('has_validated_questionnaire', False)
        has_lang_id = context.get('has_lang_id', False)

        if has_validated_questionnaire and has_lang_id:
            cr.execute("SELECT course.id "

                       "FROM training_course course "
                       "WHERE EXISTS ( SELECT * FROM training_exam_questionnaire WHERE main_course_id = course.id) "
                       "  AND course.lang_id = %s", (has_lang_id,))
            course_ids = [x[0] for x in cr.fetchall()]

            second_course_ids = super(training_course, self).search(cr, uid, domain, offset=offset,
                                                                    limit=limit,
                                                                    order=order,
                                                                    context=context,
                                                                    count=count)

            return list(set(course_ids) & set(second_course_ids))


        return super(training_course, self).search(cr, uid, domain, offset=offset,
                                                   limit=limit,
                                                   order=order,
                                                   context=context,
                                                   count=count)

training_course()

class training_degree(osv.osv):
    _name = 'training.degree'

    _columns = {
        'name' : fields.char('Name', size=64, required=True),
        'course_ids' : fields.many2many('training.course',
                                        'training_course_degree_rel',
                                        'degree_id',
                                        'course_id',
                                        'Courses',
                                       ),
        'state' : fields.selection([('draft', 'Draft'),
                                    ('validated', 'Validated'),
                                    ('deprecated', 'Deprecated')],
                                   'State',
                                   required=True, readonly=True),
        'lang_id' : fields.many2one('res.lang', 'Language', required=True),
        'active': fields.boolean('Active', select=True),
    }

    def action_deprecate(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state' : 'deprecated'}, context=context)

    def action_validate(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state' : 'validated'}, context=context)

    def action_draft(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state' : 'draft'}, context=context)

    def _lang_default(self, cr, uid, context=None):
        lang_proxy = self.pool.get('res.lang')
        user_proxy = self.pool.get('res.users')
        lang = user_proxy.browse(cr, uid, uid, context=context).context_lang
        lang_ids = lang_proxy.search(cr, uid, [('code', '=', lang)], context=context)
        return lang_ids and lang_ids[0] or False

    _defaults = {
        #'lang_id' : _lang_default,
        'state' : lambda *a: 'draft',
        'active': True,
    }

training_degree()

class training_degree_contact(osv.osv):
    _name = 'training.degree.contact'
    _rec_name = 'degree_id'

    def _get_courses_status(self, cr, uid, ids, fieldnames, args, context=None):
        degree_courses_cache = {} # degree_id: [ course_id, course_id, ..]
        participation_cache = {} # contact_id: [ part_id, part_id, ...]
        res = {}

        part_proxy = self.pool.get('training.participation')

        for degree in self.browse(cr, uid, ids, context=context):
            dg = degree.degree_id
            dc = degree.contact_id
            dc_status = {}
            dc_history = set()

            # Populate cache if it's not there
            if dg.id not in degree_courses_cache:
                degree_courses_cache[dg.id] = [ c.id for c in dg.course_ids ]
            if dc.id not in participation_cache:
                cr.execute("""SELECT * FROM training_exam_contact_results_stat WHERE contact_id = %s""", (dc.id,))
                d = cr.dictfetchall()
                participation_cache[dc.id] = d


            dg_cache = degree_courses_cache[dg.id]
            dc_status = dict((x,False) for x in dg_cache)
            # browse each participation
            for p in participation_cache[dc.id]:
                pc = p['course_id']
                if pc in dg_cache:
                    dc_history.add(p['participation_id'])
                    if p['succeeded'] == True:
                        dc_status[pc] = True


            res[degree.id] = {
                'courses_todo_ids': [ c for c, s in dc_status.iteritems() if not s ],
                'courses_done_ids': [ c for c, s in dc_status.iteritems() if s],
                'courses_history': list(dc_history),
            }
        return res


    def _check_particip_degree(self, cr, uid, ids, context=None):
        """ store function with return degree contact to be updated
        """
        contact_set = set()
        for p in self.browse(cr, uid, ids, context=context):
            if p and p.job_id and p.contact_id.id:
                contact_set.add(p.job_id.contact_id.id)
        dc_proxy = self.pool.get('training.degree.contact')
        degree_ids = dc_proxy.search(cr, uid, [('contact_id','in',list(contact_set))])
        return degree_ids

    def _check_self_degree(self, cr, uid, ids, context=None):
        return ids

    def _is_eligible_compute(self, cr, uid, ids, fieldnames, args, context=None):
        res = dict.fromkeys(ids, 0)
        for obj in self.browse(cr, uid, ids, context=context):
            res[obj.id] = len(obj.courses_todo_ids) and 'no' or 'yes'
        return res

    def _state_summary_compute(self, cr, uid, ids, fieldnames, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context=context):
            if obj.state == 'acquired':
                # degree already acquired, nothing more to check
                res[obj.id] = 'acquired'
            else:
                # degree is at least desired
                res[obj.id] = 'desired'
                # but check if it is eligible
                if not len(obj.courses_todo_ids):
                    res[obj.id] = 'eligible'
        return res

    def _get_participations(self, cr, uid, ids, context=None):
        contact_ids = [obj.contact_id.id for obj in self.browse(cr, uid, ids, context=context)]
        return self.pool.get('training.degree.contact').search(cr, uid, [('contact_id', 'in', contact_ids),('state', '=', 'desired')], context=context)

    _columns = {
        'degree_id' : fields.many2one('training.degree', 'Degree',
                                      domain="[('state', '=', 'validated')]",
                                      required=True,
                                      select=1),
        'courses_todo_ids': fields.function(_get_courses_status,
                                           method=True,
                                           string='Courses Todo',
                                           type='one2many',
                                           relation='training.course',
                                           multi='courses_status'),
        'courses_done_ids': fields.function(_get_courses_status,
                                            method=True,
                                            string='Courses Done',
                                            type='one2many',
                                            relation='training.course',
                                            multi='courses_status'),
        'courses_history': fields.function(_get_courses_status,
                                           method=True,
                                           string='Exam History',
                                           type='one2many',
                                           relation='training.participation',
                                           multi='courses_status'),

        'contact_id' : fields.many2one('res.partner.contact', 'Contact', required=True, select=1),
        'contact_lastname': fields.related('contact_id', 'name', type='char', string='Contact Name', readonly=True, select=1),
        'contact_firstname': fields.related('contact_id', 'first_name', type='char', string='Contact First Name', readonly=True, select=1),
        'date' : fields.date('Date Obtained', select=1),
        'state' : fields.selection([('desired', 'Desired'),
                                    ('acquired', 'Acquired'),
                                   ],
                                   'Internal State',
                                   required=True,
                                   readonly=True),
        'is_eligible' : fields.function(_is_eligible_compute, method=True,
                                        string='Eligible',
                                        type='selection',
                                        selection=[('yes', 'Yes'),
                                                   ('no', 'No')],
                                       ),
        'state_summary': fields.function(_state_summary_compute,
                                    method=True,
                                    string='State',
                                    type='selection',
                                    selection=[
                                        ('desired', 'Desired'),
                                        ('eligible', 'Eligible'),
                                        ('acquired', 'Acquired')
                                    ],
                                    select=1,
                                    store={
                                        'training.participation': (
                                            _check_particip_degree,
                                            None,
                                            10,
                                        ),
                                        'training.degree.contact': (
                                            _check_self_degree,
                                            None,
                                            10,
                                        )
                                    },
                        ),
    }

    _sql_constraints = [
        ('uniq_contact_degree', 'unique(contact_id, degree_id)', """There is already a relation between this degree for this contact"""),
    ]

    _defaults = {
        'state' : lambda *a: 'desired',
        'is_eligible' : lambda *a: 'no',
    }

    def action_acquire(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state' : 'acquired', 'date' : time.strftime("%Y-%m-%d")}, context=context)

training_degree_contact()

class res_partner_contact(osv.osv):
    _inherit = 'res.partner.contact'

    def search(self, cr, uid, domain, offset = 0, limit = None, order = None, context = None, count = False):
        if context is None:
            context = {}

        eligible_degree = context.get('eligible_degree', False)

        if eligible_degree:
            proxy = self.pool.get('training.degree.contact')
            ids = proxy.search(cr, uid, [('degree_id', '=', eligible_degree), ('state', '=', 'desired')], context=context)
            return [dc.contact_id.id for dc in proxy.browse(cr, uid, ids, context=context) if dc.is_eligible == 'yes']

        return super(res_partner_contact, self).search(cr, uid, domain, offset=offset, limit=limit, order=order, context=context, count=count)

res_partner_contact()

class training_degree_contact_view(osv.osv):
    _name = 'training.degree.contact.view'

    _auto = False

    _columns = {
        'participation_id' : fields.many2one('training.participation', 'Participation', readonly=True),
        'contact_id' : fields.many2one('res.partner.contact', 'Contact', readonly=True),
        'course_id' : fields.many2one('training.course', 'Course', readonly=True),
        'degree_id' : fields.many2one('training.degree', 'Degree', readonly=True),
        'degree_contact_id' : fields.many2one('training.degree.contact', 'Degree Contact', readonly=True),
        'succeeded' : fields.boolean('Succeeded'),
    }

    def init(self, cr):
        tools.drop_view_if_exists(cr, 'training_degree_contact_view')
        cr.execute("""
                   CREATE OR REPLACE VIEW training_degree_contact_view as (
                       SELECT tp.id as id,
                           tp.id as participation_id,
                           tp.result,
                           tp.passing_score,
                           tp.seance_id,
                           tp.seance_date,
                           tp.RESULT >= tp.passing_score as succeeded,
                           tp.contact_id,
                           tp.course_questionnaire_id as course_id,
                           td.id AS degree_id,
                           (SELECT id FROM training_degree_contact WHERE contact_id = tp.contact_id AND degree_id = td.id) as degree_contact_id
                       FROM training_participation tp, training_degree td
                       WHERE tp.course_questionnaire_id IN (SELECT tcdr.course_id FROM training_course_degree_rel tcdr WHERE tcdr.degree_id = td.id)
                       AND td.state = 'validated'
                       ORDER BY tp.contact_id, tp.course_questionnaire_id
                  )
                   """)

training_degree_contact_view()

class training_degree_contact_wiz_line(osv.osv_memory):
    _name = 'training.degree.contact.wiz.line'
    _max_count = 10000

    _columns = {
        'wizard_id' : fields.many2one('training.degree.contact.wizard', 'Wizard', required=True),
        'degree_id' : fields.many2one('training.degree', 'Degree', readonly=True),
        'contact_id' : fields.many2one('res.partner.contact', 'Contact', readonly=True),
        'subscribe' : fields.boolean('Subscribe'),
        'state' : fields.selection([('desired', 'Desired'),
                                    ('eligible', 'Eligible'),
                                    ('acquired', 'Acquired')],
                                   'State',
                                   readonly=True,
                                   required=True),
    }
    _defaults = {
        'subscribe' : lambda *a: 0,
        'state' : lambda *a: 'desired',
    }

training_degree_contact_wiz_line()

class training_degree_contact_wizard(osv.osv_memory):
    _name = 'training.degree.contact.wizard'

    _columns = {
        'kind_search' : fields.selection([('contact_id', 'Contact'),
                                          ('degree_id', 'Degree')],
                                         'Search', required=True),
        'state' : fields.selection([('init', 'Init'),
                                    ('show_degrees', 'Show Degrees')],
                                   'State',
                                   readonly=True,
                                   required=True),
        'contact_id' : fields.many2one('res.partner.contact', 'Contact'),
        'degree_id' : fields.many2one('training.degree', 'Degree', domain="[('state', '=', 'validated')]"),

        'line_ids' : fields.one2many('training.degree.contact.wiz.line', 'wizard_id', 'Degrees'),
    }

    _defaults = {
        'state' : lambda *a: 'init',
        'kind_search' : lambda *a: 'degree_id',
    }

    def on_change_kind_search(self, cr, uid, ids, kind_search, context=None):
        key = {'contact_id' : 'degree_id', 'degree_id' : 'contact_id'}.get(kind_search, 'degree_id')
        return { 'value' : { key : 0 }}

    def show_degrees_cb(self, cr, uid, ids, context=None):
        dc_proxy = self.pool.get('training.degree.contact.view')
        wl_proxy = self.pool.get('training.degree.contact.wiz.line')
        dg_proxy = self.pool.get('training.degree')

        for obj in self.browse(cr, uid, ids, context=context):
            wl_proxy.unlink(cr, uid,
                            wl_proxy.search(cr, uid, [('wizard_id', '=', obj.id)], context=context),
                            context=context)

            domain = [(obj.kind_search, '=', obj[obj.kind_search].id),('degree_contact_id', '=', False)]
            dc_ids = dc_proxy.search(cr, uid, domain, context=context)

            # First pass
            degree_contacts = {}
            dg_cache = {}
            if obj.kind_search in ('degree_id','contact_id'):
                for dc in dc_proxy.browse(cr, uid, dc_ids, context=context):
                    dg_id = dc.degree_id.id
                    dc_id = dc.contact_id.id

                    if dg_id not in dg_cache:
                        dg_cache[dg_id] = [ c.id for c in dg_proxy.browse(cr, uid, dg_id, context=context).course_ids ]
                    if dc[obj.kind_search].id == obj[obj.kind_search].id:
                        if (dc_id, dg_id) not in degree_contacts:
                            degree_contacts[(dc_id, dg_id)] = dict((c,True) for c in dg_cache[dg_id])
                        if dc.participation_id.succeeded == 'yes':
                            degree_contacts[(dc_id, dg_id)][dc.course_id.id] = True

            # Second pass
            for (contact_id, degree_id), vals_dict in degree_contacts.iteritems():
                vals = vals_dict.values()
                if not any(vals):
                    continue
                c_id = wl_proxy.create(cr, uid,
                                {
                                    'wizard_id' : obj.id,
                                    'degree_id' : degree_id,
                                    'contact_id' : contact_id,
                                    'state' : all(vals) and 'eligible' or 'desired',
                                },
                                context=context)

        return self.write(cr, uid, ids, {'state' : 'show_degrees'}, context=context)

    def select_all_cb(self, cr, uid, ids, context=None):
        for obj in self.browse(cr, uid, ids, context=context):
            for line in obj.line_ids:
                line.write({'subscribe' : 1})
        return True

    def deselect_all_cb(self, cr, uid, ids, context=None):
        for obj in self.browse(cr, uid, ids, context=context):
            for line in obj.line_ids:
                line.write({'subscribe' : 0})
        return True

    def subscribe_cb(self, cr, uid, ids, context=None):
        proxy = self.pool.get('training.degree.contact')
        dc_ids = []
        for obj in self.browse(cr, uid, ids, context=context):
            for line in obj.line_ids:
                if line.state == 'acquired':
                    continue
                elif line.subscribe:
                    new_id = proxy.create(cr, uid, { 'degree_id' : line.degree_id.id, 'contact_id' : line.contact_id.id, 'state' : 'desired' }, context=context)
                    dc_ids.append(new_id)
        if dc_ids:
            return {
                'domain' : "[('id', 'in', [%s])]" % ",".join(map(str, dc_ids)),
                'name' : 'Eligible Degrees',
                'view_type' : 'form',
                'view_mode' : 'tree,form',
                'res_model' : 'training.degree.contact',
                'view_id' : False,
                'type' : 'ir.actions.act_window',
            }
        else:
            return self.close_cb(cr, uid, ids, context=context)

    def close_cb(self, cr, uid, ids, context=None):
        return {'type' : 'ir.actions.act_window_close'}

training_degree_contact_wizard()

def get_zip_from_directory(directory, b64enc=True):
    RE_exclude = re.compile('(?:^\..+\.swp$)|(?:\.py[oc]$)|(?:\.bak$)|(?:\.~.~$)', re.I)

    def _zippy(archive, path):
        path = os.path.abspath(path)
        base = os.path.basename(path)
        for f in tools.osutil.listdir(path, True):
            bf = os.path.basename(f)
            if not RE_exclude.search(bf):
                archive.write(os.path.join(path, f), os.path.join(base, f))

    archname = StringIO.StringIO()
    archive = PyZipFile(archname, "w", ZIP_DEFLATED)
    archive.writepy(directory)
    _zippy(archive, directory)
    archive.close()
    val = archname.getvalue()
    archname.close()

    if b64enc:
        val = base64.encodestring(val)

    return val

class training_degree_contact_report(osv.osv_memory):
    _name = 'training.degree.contact.report'

    _columns = {
        'state' : fields.selection([('init', 'Init'), ('generate_reports', 'Generate Reports')],
                                   'State',
                                   readonly=True,
                                   required=True),
        'degree_id': fields.many2one('training.degree', 'Degree'),
        'file_data' : fields.binary('File', readonly=True),
        'file_name' : fields.char('Filename', readonly=True, size=64),
    }

    _defaults = {
        'state' : lambda *a: 'init',
        'degree_id': lambda *a: False,
    }

    def get_contact_email_address(self, cr, uid, contact_id, context=None):
        contact = self.pool.get('res.partner.contact').browse(cr, uid, contact_id, context=context)
        jobs = self.pool.get('res.partner.job')
        if contact.email:
            return contact.email
        contact_jobs = jobs.search(cr, uid, [('contact_id','=',contact_id)], context=context)
        if not contact_jobs:
            return '' # no jobs found for this contact

        candidate_job_ids = set()
        emails = {}
        for job in jobs.browse(cr, uid, contact_jobs, context=context):
            if job.state == 'past':
                # ignore past job
                continue
            if job.email:
                candidate_job_ids.add(job.id)

        if not len(candidate_job_ids):
            return '' # no current jobs have valid mail

        cr.execute("SELECT create_date, write_date, email FROM res_partner_job WHERE id IN (%s)" % (','.join(map(str, candidate_job_ids))))
        for job in cr.dictfetchall():
            last_date = job['write_date'] or job['create_date']
            emails[last_date] = job['email']

        if not len(emails):
            return '' # we didn't find any email

#        last = reversed(sorted(emails.iterkeys()))[0]
        date_list = emails.keys()
        date_list.sort()
        last = date_list[-1]

        return emails[last]

    def get_contact_priv_address(self, cr, uid, contact_id, context=None):
        address = contact_id.partner_id.address and contact_id.partner_id.address[0].name_get()
        if address:
            return address[0][1]
        return ''

    def generate_reports_cb(self, cr, uid, ids, context=None):
        proxy = self.pool.get('training.degree.contact')
        jobs_proxy = self.pool.get('res.partner.job')
        participation_proxy = self.pool.get('training.participation')
        dc_ids = []

        Headers = {
            'first_name' : 'First Name',
            'name' : 'Name',
            'email' : 'Email',
            'address': 'Private Address',
            'partner_id' : 'Partner',
            'partner_email': 'HR Email',
            'nb_exam' : 'Exams Todo',
            'nb_exam_success' : 'Exams Success',
            'state' : 'State',
            'acquisition_date' : 'Acquisition Date',
        }
        BASIC_HEADERS = ('first_name', 'name', 'email', 'address', 'partner_id', 'partner_email', 'nb_exam', 'nb_exam_success', 'state', 'acquisition_date')
        size_basic_headers = len(BASIC_HEADERS)
        BASIC_HEADERS_TYPE = [(name == 'acquisition_date' and 'date' or 'text') for name in BASIC_HEADERS]
        BASIC_HEADERS = dict((name, idx) for idx, name in enumerate(BASIC_HEADERS))

        for obj in self.browse(cr, uid, ids, context=context):
            if not obj.degree_id:
                cr.execute("""SELECT DISTINCT(degree_id) FROM training_degree_contact""")
                degrees_to_report = [ degree_id for (degree_id,) in cr.fetchall() ]
            else:
                degrees_to_report = [ obj.degree_id.id ]

            degrees = self.pool.get('training.degree').browse(cr, uid, degrees_to_report, context=context)

            try:
                import tempfile
                parent_directory = tempfile.mkdtemp(prefix='openerp_', suffix='_degree_reports')
                directory = os.path.join(parent_directory, 'Degrees')
                os.mkdir(directory)

                for degree in degrees:
                    number_of_courses = len(degree.course_ids)
                    ids_of_courses = tuple([ x.id for x in degree.course_ids ])
                    dc_ids = proxy.search(cr, uid, [('degree_id', '=', degree.id)], context=context)

                    dc_full_stats = {}
                    if ids_of_courses:
                        cr.execute("""
                            SELECT
                                contact_id,
                                participation_id,
                                course_id,
                                succeeded
                            FROM
                                training_exam_contact_results_stat
                            WHERE
                                course_id IN %s
                            AND
                                contact_id IN (
                                    SELECT contact_id
                                    FROM training_degree_contact
                                    WHERE degree_id = %s
                                )
                            """, (ids_of_courses, degree.id,))
                    dc_results = cr.dictfetchall()
                    for result in dc_results:
                        c = result['contact_id']
                        if c not in dc_full_stats:
                            dc_full_stats[c] = {
                                'courses_history': set(),
                                'courses_success': set(),
                            }
                        dc_full_stats[c]['courses_history'].add(result['participation_id'])
                        if result['succeeded']:
                            dc_full_stats[c]['courses_success'].add(result['course_id'])

                    try:
                        strIO = StringIO.StringIO()
                        wb = xlwt.Workbook(encoding='utf-8')
                        stream = wb.add_sheet('Degree Contact Report')
                        stream.portrait = 0 # Landscape
                        stream_row = 0
                        basic_headers = BASIC_HEADERS.copy()
                        basic_headers_type = BASIC_HEADERS_TYPE[:]

                        for idx, course in enumerate(degree.course_ids):
                            basic_headers[str(course.id)] = idx + size_basic_headers
                            basic_headers_type.append('text')
                            Headers[str(course.id)] = tools.ustr(course.name).encode('utf8')

                        def stream_write_row(row, values, cols, col_types):
                            r = stream.row(row)
                            for idx, val in enumerate(values):
                                if cols and col_types[idx] == 'date' and val:
                                    val = time.strftime('%d/%m/%Y', time.strptime(val, '%Y-%m-%d'))
                                xlwt.Row.set_cell_text(r, idx, val)
                            return row + 1

                        sorted([(pos, name) for name, pos in basic_headers.iteritems()], sort_position)
                        stream_row = stream_write_row(stream_row, [Headers[name] for pos, name in sorted([(pos, name) for name, pos in basic_headers.iteritems()], sort_position)], False, basic_headers_type)
                        for dc_idx, dc in enumerate(proxy.browse(cr, uid, dc_ids, context=context)):
                            dc_contact = dc.contact_id
                            if dc_contact.id not in dc_full_stats:
                                dc_full_stats[dc_contact.id] = {
                                    'courses_history': set(),
                                    'courses_success': set(),
                                }
                            values = {
                                'first_name' : dc_contact.first_name,
                                'name' : dc_contact.name,
                                'address': self.get_contact_priv_address(cr, uid, dc_contact, context=None),
                                'email' : self.get_contact_email_address(cr, uid, dc_contact.id, context=None),
                                'partner_id' : dc_contact.partner_id.name,
                                'partner_email': dc_contact.partner_id.notif_contact_id and (dc_contact.partner_id.notif_contact_id.email or ''),
                                'nb_exam' : "%d/%d" % (number_of_courses - len(dc_full_stats[dc_contact.id]['courses_success']), number_of_courses),
                                'nb_exam_success' : "%d/%d" % (len(dc_full_stats[dc_contact.id]['courses_success']), number_of_courses),
                                'state' : dc.state_summary,
                                'acquisition_date' : dc.date or '',
                            }

                            if not values['partner_id']:
                                # try to find first job with partner + state == 'current'
                                dc_contact_jobs = orm.orm.search(jobs_proxy, cr, uid, [('contact_id','=',dc.contact_id.id)])
                                if dc.contact_id and dc_contact_jobs:
                                    for j in jobs_proxy.browse(cr, uid, dc_contact_jobs, context=context):
                                        if j.address_id and j.state == 'current':
                                            if j.address_id.partner_id.name:
                                                values['partner_id'] = j.address_id.partner_id.name
                                                break

                            # If multiple participation, show each participation sorted by date
                            today = time.time()
                            for participation in participation_proxy.browse(cr, uid, list(dc_full_stats[dc_contact.id]['courses_history']), context=context):
                                part_date = time.mktime(time.strptime(participation.date, '%Y-%m-%d %H:%M:%S'))
                                s = 'N/A'
                                if part_date <= today:
                                    # past and present
                                    if participation.present and participation.result_received:
                                        s = '%04.2f%%' % (participation.result)
                                    elif participation.present:
                                        s = 'Present'
                                    elif (not participation.present and not participation.result_received):
                                        s = 'Absent'
                                else:
                                    s = time.strftime('%d/%m/%Y', time.strptime(participation.date, '%Y-%m-%d %H:%M:%S'))

                                k = str(participation.course_questionnaire_id.id)
                                if k not in values:
                                    values[k] = s
                                else:
                                    values[k] += ' '+s


                            for c_id in ids_of_courses:
                                if str(c_id) not in values:
                                    values[str(c_id)] = 'N/A'

                            columns = sorted([(basic_headers[key], value) for key, value in values.iteritems() if key in basic_headers], sort_position)

                            columns_value = [tools.ustr(value).encode('utf8') for _, value in columns]
                            stream_row = stream_write_row(stream_row, columns_value, columns, basic_headers_type)
                        wb.save(strIO)
                    finally:
                        file_path = os.path.join(directory, "Degree %s.xls" % tools.ustr(degree.name).encode('utf8'))
                        fp = file(file_path, "w")
                        fp.write(strIO.getvalue())
                        fp.close()
                        strIO.close()

                result = get_zip_from_directory(directory, True)
                fp = file(os.path.join(parent_directory, 'output.zip'), 'w')
                fp.write(result)
                fp.close()

                values = {
                    'state' : 'generate_reports',
                    'file_data' : result,
                    'file_name' : 'Degree_Reports.zip',
                }

            finally:
                import shutil
                shutil.rmtree(parent_directory)

        return self.write(cr, uid, ids, values, context=context)

    def close_cb(self, cr, uid, ids, context=None):
        return {'type' : 'ir.actions.act_window_close'}

training_degree_contact_report()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

