# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
import tools
from tools.translate import _

class wizard_eligible_contacts(wizard.interface):
    first_screen_fields = {
        'create_degrees' : {
            'string' : 'Assign degrees',
            'type' : 'boolean',
        },
        'show' : {
            'string' : 'Show',
            'type' : 'selection',
            'selection' : [('contacts', 'Contacts'),('degrees', 'Degrees')],
            'default' : 'contacts',
        }
    }

    first_screen_form = '''<?xml version="1.0"?>
    <form string="Eligible Contacts">
        <field name="show" />
        <group colspan="4" col="2" attrs="{'invisible' : [('show', '=', 'contacts')]}">
            <separator string="" colspan="4"/>
            <field name="create_degrees" />
        </group>
    </form>'''

    def init(self, cr, uid, data, context=None):
        if data['id']:
            degree = pooler.get_pool(cr.dbname).get(data['model']).browse(cr, uid, data['id'], context=context)
            if degree.state != 'validated':
                raise wizard.except_wizard(_('Warning'),_("Please, you can only select a validated degree !"))
        return {}

    def compute_eligible_contacts(self, cr, uid, data, context=None):
        pool = pooler.get_pool(cr.dbname)

        degree_id = data['id']
        ids = []
        contact_ids = pool.get('res.partner.contact').search(cr, uid, [], context=context)

        for contact_id in contact_ids:
            cr.execute("""
                       SELECT tp.contact_id, tp.result, tp.passing_score, tp.result >= tp.passing_score
                       FROM training_course_degree_rel tcdr
                       LEFT OUTER JOIN training_participation tp
                       ON tcdr.course_id = tp.course_questionnaire_id
                       AND tcdr.degree_id = %s
                       AND tp.contact_id = %s
                       WHERE tcdr.degree_id = %s
                       """, (degree_id, contact_id, degree_id))

            if all(x[3] == True for x in cr.fetchall()):
                ids.append(contact_id)

        if not ids:
            return {'type':'ir.actions.act_window_close'}

        degree_ids = []
        if data['form']['create_degrees']:
            for contact_id in ids:
                values = {
                    'degree_id' : degree_id,
                    'contact_id' : contact_id,
                }

                new_degree_id = pool.get('training.degree.contact').create(cr, uid, values, context=context)
                degree_ids.append(new_degree_id)

        if data['form']['show'] == 'contacts':
            return {
                'domain' : "[('id', 'in', [%s])]" % ','.join(map(str,ids)),
                'name' : 'Eligible Contacts',
                'view_type' : 'form',
                'view_mode' : 'tree,form',
                'res_model' : 'res.partner.contact',
                'view_id' : False,
                'type' : 'ir.actions.act_window',
            }
        else:
            return {
                'domain' : "[('id', 'in', [%s])]" % ','.join(map(str,degree_ids)),
                'name' : 'Eligible Degrees',
                'view_type' : 'form',
                'view_mode' : 'tree,form',
                'res_model' : 'training.degree.contact',
                'view_id' : False,
                'type' : 'ir.actions.act_window',
            }



    states = {
        'init': {
            'actions': [init],
            'result': {
                'type': 'form',
                'arch': first_screen_form,
                'fields': first_screen_fields,
                'state':[('end','Cancel', 'gtk-cancel'),('compute', 'Compute Eligible Contacts', 'gtk-execute')],
            }
        },
        'compute' : {
            'result' : {
                'type' : 'action',
                'action' : compute_eligible_contacts,
                'state' : 'end',
            }
        }
    }

wizard_eligible_contacts('training.degree.eligible.contacts')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
