
from osv import osv
from osv import fields
from tools.translate import _

WRITABLE_ONLY_IN_DRAFT = dict(readonly=True, states={'draft': [('readonly',False)]})

class training_config_contact_function(osv.osv):
    _inherit = 'training.config.contact.function'

    def _get_values_of_kind(self):
        vals = super(training_config_contact_function, self)._get_values_of_kind()
        if not vals:
            vals = []
        vals.append(('diagnostic', 'Exam Diagnostic'))
        return vals

training_config_contact_function()

class training_participation_remarks_template(osv.osv):
    _name = 'training.participation.remarks.template'
    _order = 'sequence'
    _columns = {
        'name': fields.char('Remark Text', size=255, select=1, required=True),
        'sequence': fields.integer('Order', required=True),
    }

    _defaults = {
        'sequence': lambda *a: 0,
    }

training_participation_remarks_template()

class training_participation_remarks(osv.osv):
    _name = 'training.participation.remarks'

    def get_remark_levels(self, cr, uid, context=None):
        return [
            ('convincing', 'Convaincant'),
            ('unconvincing', 'Peu Convaincant'),
        ]

    _columns = {
        'participation_id': fields.many2one('training.participation', 'Participation', required=True),
        'name': fields.many2one('training.participation.remarks.template', 'Remarks', required=True, select=1),
        'level': fields.selection(get_remark_levels, 'Level', required=True, select=1),
    }

training_participation_remarks()

class training_exam_question_answer(osv.osv):
    _inherit = 'training.exam.question.answer'

    _columns = {
        'competency_ids': fields.many2many('training.competency',
                                           'training_exam_question_answer_competency_rel',
                                           'answer_id',
                                           'competency_id',
                                           string='Competencies', **WRITABLE_ONLY_IN_DRAFT),
    }
training_exam_question_answer()

class training_exam_question_level(osv.osv):
    _name = 'training.exam.question.level'
    _columns = {
        'name': fields.char('Name', size=64, required=True),
        'code': fields.char('Code',  size=16, required=True),
        'level': fields.integer('Level', required=True),
        'color': fields.char('Color', size=16, help='HTML Color notation for color used to display this level in reporting'),
    }

training_exam_question_level()

class training_exam_question(osv.osv):
    _inherit = 'training.exam.question'

    _columns = {
        'competency_ids': fields.many2many('training.competency',
                                          'training_exam_question_competency_rel',
                                          'question_id',
                                          'competency_id',
                                          string='Competency', **WRITABLE_ONLY_IN_DRAFT),
        'level_id': fields.many2one('training.exam.question.level', 'Level', **WRITABLE_ONLY_IN_DRAFT),
    }

    def _get_extended_question_infos_elements(self, cr, uid, question, context=None, report_context=None):
        res = super(training_exam_question, self)._get_extended_question_infos_elements(cr, uid, question, context=context, report_context=report_context)
        if question.level_id:
            res.append(('level', _('LEVEL: %s') % (question.level_id.name)))
        if question.competency_ids:
            domain_list = set()
            for comp in question.competency_ids:
                for dom in comp.domain_ids:
                    domain_list.add(dom.id)
            if domain_list:
                domains = self.pool.get('training.domain').browse(cr, uid, list(domain_list), context=context)
                res.append(('domains', _('DOMAIN: %s') % (', '.join([ d.name for d in domains]))))
        return res

training_exam_question()

class training_exam_questionnaire_domain(osv.osv):
    _name = 'training.exam.questionnaire.domain'
    _rec_name = 'domain_id'

    def f_get_total_point(self, cr, uid, ids, fieldnames, args, context=None):
        res = dict.fromkeys(ids, 0.0)

        domain_per_quizz = {} # 'training.exame.questionnaire:id' => 'training.domain:id' => value
        quizz_domain_per_domain = {} # 'training.exam.questionnaire.domain:id' => training.domain:id

        for teqd in self.browse(cr, uid, ids, context=context):
            quizz_domain_per_domain[teqd.domain_id.id] = teqd.id
            # test we already have compute domain for the questionnaire
            # relate to this domain
            quizz_id = teqd.questionnaire_id.id
            if quizz_id not in domain_per_quizz:
                domain_per_quizz[quizz_id] = {}
            else:
                continue

            dpq = domain_per_quizz[quizz_id]
            for quizz_question in teqd.questionnaire_id.question_ids:
                for comp in quizz_question.question_id.competency_ids:
                    for domain in comp.domain_ids:
                        if domain.id not in dpq:
                            dpq[domain.id] = 0.0
                        dpq[domain.id] += quizz_question.question_id.point

        # we only one quizz for integrity
        if not len(domain_per_quizz):
            return res
        domains = domain_per_quizz.popitem()[1]

        # update existing dict
        res2 = ((quizz_domain_per_domain[k], v) for k, v in domains.iteritems() if k in quizz_domain_per_domain)
        res.update(dict(res2))
        return res

    def _get_states_selection(self, cr, uid, context=None):
        return self.pool.get('training.exam.questionnaire')._states_selection

    _columns = {
        'domain_id': fields.many2one('training.domain', 'Domain', required=True),
        'questionnaire_id': fields.many2one('training.exam.questionnaire', 'Questionnaire', required=True, ondelete='cascade'),
        'ideal_score': fields.float('Ideal Profile', required=True),
        'total_points': fields.function(f_get_total_point, method=True, type='float', string="Total Points", readonly=True),
        'state': fields.related('questionnaire_id', 'state', type='selection', selection=_get_states_selection, string='State', readonly=True),
    }

    _defaults = {
        'ideal_score': lambda *a: 0.0,
    }

training_exam_questionnaire_domain()

class training_exam_questionnaire_domain_ideal_score_wizard(osv.osv_memory):
    _name = 'training.exam.questionnaire.domain.idealscore.wizard'
    _columns = {
        'ideal_score': fields.float('Ideal Score'),
    }

    def _default_get_ideal_score(self, cr, uid, context=None):
        if context is None:
            context = {}
        active_id = context.get('active_id')
        if active_id:
            questionnaire_domain_obj = self.pool.get('training.exam.questionnaire.domain')
            return questionnaire_domain_obj.browse(cr, uid, active_id, context=context).ideal_score
        else:
            return 0.0

    _defaults = {
        'ideal_score': _default_get_ideal_score,
    }

    def button_update_ideal_score(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        active_id = context.get('active_id')
        if not active_id or not ids:
            return {}
        wizard = self.browse(cr, uid, ids[0], context=context)
        value = {'ideal_score': wizard.ideal_score}
        questionnaire_domain_obj = self.pool.get('training.exam.questionnaire.domain')
        questionnaire_domain_obj.write(cr, uid, [active_id], value, context=context)
        return {}
training_exam_questionnaire_domain_ideal_score_wizard()

class training_exam_questionnaire(osv.osv):
    _inherit = 'training.exam.questionnaire'

    def _trigger_categorization_update(self, cr, uid, ids, context=None, passing_score_update=True):
        """add/remove categorizations field update questionnaire
           (this imply course_ids [training_exam] and domain_ids [training_diagnostic])"""
        quizz_domain = self.pool.get('training.exam.questionnaire.domain')

        super(training_exam_questionnaire, self)._trigger_categorization_update(cr, uid, ids, context=context, passing_score_update=passing_score_update)
        if context is None:
            context = {}
        if context.get('new_revision'):
            return True
        for quizz in self.browse(cr, uid, ids, context=context):
            if quizz.state != 'draft':
                continue # only update draft questionnaires!

            quizz_type = quizz.type
            # collect existing domains
            existing_domain_dict = dict(( (d.domain_id.id, d) for d in quizz.domain_ids ))
            existing_domain_ids = set(existing_domain_dict.iterkeys())
            # search current domains based on questions
            current_domain_ids = set()
            for q in quizz.question_ids:
                #TODO: make this work when we have multiple domain per competency
                for c in q.question_id.competency_ids:
                    for d in c.domain_ids:
                        current_domain_ids.add(d.id)

            # domain compute differences
            domain_ids_to_remove = existing_domain_ids - current_domain_ids
            domain_ids_to_add = current_domain_ids - existing_domain_ids

            # remove un-necessary domains
            dids = [ existing_domain_dict[x].id for x in domain_ids_to_remove ]
            quizz_domain.unlink(cr, uid, dids, context=context)

            quizz_id = quizz.id
            # add new domains
            for did in domain_ids_to_add:
                q = {
                    'domain_id': did,
                    'questionnaire_id': quizz_id,
                }
                #XXX: other value of training.exam.questionnaire.domain should
                #     have default values otherwise next call will fail!
                quizz_domain.create(cr, uid, q)

            # update passing score on course (default to 50%)
            if passing_score_update:
                new_all_quizz_domain = quizz_domain.search(cr, uid, [('questionnaire_id','=',quizz_id)], context=context)
                for qc in quizz_domain.browse(cr, uid, new_all_quizz_domain, context=context):
                    quizz_domain.write(cr, uid, [qc.id], {'ideal_score': qc.total_points / 2.0}, context=context)
        return True

    def _f_update_from_questionnaire_questions1(self, cr, uid, ids, context=None):
        """return questionnaire ids for which question were added/removed/modified
           (note: we only take into account questionnaire in draft state)"""
        r = [ x.questionnaire_id.id for x in self.browse(cr, uid, ids, context=context) if x.questionnaire_id.state == 'draft' ]
        return r

    def _f_update_from_questions1(self, cr, uid, ids, context=None):
        """return questionnaire ids which contain question which were modified
           (note: we only take into account questionnaire and question in draft state)"""
        r = []
        for q in self.browse(cr, uid, ids, context=context):
            if q.state != 'draft':
                continue
            r.extend([ x.id for x in q.questionnaire_ids if x.state == 'draft' ])
        return r

    def _f_update_from_question_answers1(self, cr, uid, ids, context=None):
        r = []
        for a in self.browse(cr, uid, ids, context=context):
            q = a.question_id
            if q.state != 'draft':
                continue
            r.extend([ x.id for x in q.questionnaire_ids if x.state == 'draft' ])
        return r

    def _f_update_wo_passingscore(self, cr, uid, ids, fieldname, args, context=None):
        self._trigger_categorization_update(cr, uid, ids, context=context, passing_score_update=False)
        return dict.fromkeys(ids, False)

    _columns = {
        'type': fields.selection([('examen','Examen'),
                                  ('diagnostic', 'Diagnostic')],
                                  'Type', required=True, select=2),
        'domain_ids': fields.one2many('training.exam.questionnaire.domain', 'questionnaire_id', 'Domains'),
        'update1': fields.function(_f_update_wo_passingscore, type='boolean', string='Internal Store Update',
                                        help='This stored field help updating categorization (courses) on questionnaire',
                                        method=True,
                                        store={
                                            'training.exam.questionnaire.question': (_f_update_from_questionnaire_questions1, [], 30),
                                            'training.exam.question': (_f_update_from_questions1, ['competency_ids'], 20),
                                            'training.exam.question.answer': (_f_update_from_question_answers1, ['competency_ids'], 10),
                                        }),
    }

training_exam_questionnaire()

class training_participation_domain_line(osv.osv):
    _name = 'training.participation.domain.line'
    _rec_name = 'domain_id'

    def f_get_score(self, cr, uid, ids, fieldnames, args, context=None):
        res = dict.fromkeys(ids, 0.0)
        domain_per_part = {} # domain per participation

        for dl in self.browse(cr, uid, ids, context=context):
            part = dl.participation_id
            if part.id not in domain_per_part:
                domain_per_part[part.id] = {}
            else:
                # this participation have already been computed,
                # get value from cache
                if dl.domain_id.id in domain_per_part[part.id]:
                    res[dl.id] = domain_per_part[part.id][dl.domain_id.id]
                continue

            # compute all domain line for this participation
            ddict = domain_per_part[part.id]
            for respline in part.participation_line_ids:
                resp_point = respline.point
                if -0.00001 <= resp_point <= 0.00001:
                    # point is null, no score to add to domain
                    continue
                domains = set()
                for comp in respline.question_id.competency_ids:
                    for dom in comp.domain_ids:
                        domains.add(dom.id)
                for dom in domains:
                    if dom not in ddict:
                        ddict[dom] = 0.0
                    ddict[dom] += resp_point
            if dl.domain_id.id in ddict:
                res[dl.id] = ddict[dl.domain_id.id]
        return res

    _columns = {
        'participation_id': fields.many2one('training.participation', 'Participation', required=True, ondelete='cascade'),
        'domain_id': fields.many2one('training.domain', 'Domain', required=True, readionly=True),
        'ideal_score': fields.float('Ideal Score', required=True),
        'max_score': fields.float('Max Score', required=True),
        'score': fields.function(f_get_score, method=True, type='float', required=True, string='Score'),
    }

training_participation_domain_line()

class training_participation(osv.osv):
    _inherit = 'training.participation'

    _columns = {
        'domain_line_ids': fields.one2many('training.participation.domain.line', 'participation_id', 'Domain Lines'),
        'type_questionnaire_id': fields.related('questionnaire_id', 'type', type='selection', selection=[('examen', 'Examen'),('diagnostic','Diagnostic')],
                                                string='Type', readonly=True),
        'kind': fields.related('seance_id', 'kind', type='selection', selection=[('standard', 'Course'),('exam', 'Exam'),('diagnostic','Diagnostic')],
                                                string='Kind', select='1'),
        'diagnostic_extra_summary': fields.text('Diagnostic Extra Summary'),
        'psychogram_passed': fields.boolean('Psychogram Passed'),
        'remark_ids': fields.one2many('training.participation.remarks', 'participation_id', 'Remarks'),
        'interview_date': fields.date('Interview Date'),
    }

    def copy_data(self, cr, uid, object_id, values=None, context=None):
        # domain line should not be copied !
        if not values:
            values = {}
        if 'domain_line_ids' not in values:
            values['domain_line_ids'] = []
        return super(training_participation, self).copy_data(cr, uid, object_id, values, context)

    def random_questionnaire_get_domain(self, cr, uid, participation):
        seance = participation.seance_id
        if not seance:
            return super(training_participation, self).random_questionnaire_get_domain(cr, uid, participation)
        if seance.kind == 'diagnostic':
            return [('type','=','diagnostic'),('state','in',['validated']),('main_course_id','=',participation.course_questionnaire_id.id),('printable','=',True)]
        # otherwise keep default behaviour
        return super(training_participation, self).random_questionnaire_get_domain(cr, uid, participation)

    def after_exam_sheet_generation(self, cr, uid, participation_id, context=None):
        #XXX: beware, self is comming from reporting !!!! its not a ORM object
        super(training_participation, self).after_exam_sheet_generation(cr, uid, participation_id, context=context)

        tpdl = self.pool.get('training.participation.domain.line')
        tpcl = self.pool.get('training.participation.course.line')

        tpdl_current_ids = tpdl.search(cr, uid, [('participation_id', '=', participation_id)], context=context)
        if len(tpdl_current_ids):
            raise osv.except_osv(_('Error'),
                                _('Can\'t generate exam sheet on participation which already have domain line'))

        p = self.browse(cr, uid, participation_id, context=context)
        if p.questionnaire_id:
            for qdomain in p.questionnaire_id.domain_ids:
                tpdl.create(cr, uid, {
                    'participation_id': participation_id,
                    'domain_id': qdomain.domain_id.id,
                    'ideal_score': qdomain.ideal_score,
                    'max_score': qdomain.total_points,
                }, context=context)

            for qcourse in p.questionnaire_id.course_ids:
                tpcl.create(cr, uid, {
                    'participation_id': participation_id,
                    'course_id': qcourse.course_id.id,
                    'passing_score': qcourse.passing_score,
                    'max_score': qcourse.total_point,
                }, context=context)
        return True


    def on_change_questionnaire(self, cr, uid, ids, questionnaire_id, context=None):
        res = super(training_participation, self).on_change_questionnaire(cr, uid, ids, questionnaire_id, context=context)
        if not questionnaire_id:
            return res
        questionnaire = self.pool.get('training.exam.questionnaire').browse(cr, uid, questionnaire_id, context=context)
        if questionnaire.type == 'diagnostic':
            # force passing score to 0 for diagnostic, because real passing score
            # if now per courses
            res.get('value',{}).update({'passing_score' : 0.0,
                                'total_points' : questionnaire.total_point,
                                })
        res.get('value',{}).update({'type_questionnaire_id' : questionnaire.type})
        return res

training_participation()

class training_seance(osv.osv):
    _inherit = 'training.seance'

    def _create_participation(self, cr, uid, seance, subscription_line, context=None):
        participation = super(training_seance, self)._create_participation(cr, uid, seance, subscription_line, context=context)
        t_part_proxy = self.pool.get('training.participation')

        if seance.kind == 'diagnostic':
            # subscription is not on a diagnostic session, but this seance is an exam
            # so the exam participation with be taken on course from this seance
            t_part_proxy.write(cr, uid, [participation], {'course_questionnaire_id': seance.course_id.id})
        return participation

training_seance()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
