

from osv import osv
from osv import fields

# advance declaration, to avoid loops
class training_competency_advance(osv.osv):

    _name = 'training.competency'

training_competency_advance()

# advance declaration, to avoid loops
class training_domain_advance(osv.osv):

    _name = 'training.domain'

training_domain_advance()

class training_profession(osv.osv):
    _name = 'training.profession'
    _columns = {
        'name': fields.char('Name', size=64, required=True, select=1),
        'description': fields.text('Description'),
        'domain_ids': fields.many2many('training.domain',
                                       'training_profession_domain_rel',
                                       'profession_id',
                                       'domain_id',
                                       string='Domains'),
    }
training_profession()

class training_domain(osv.osv):
    _name = 'training.domain'
    _columns = {
        'name': fields.char('Name', size=64, required=True, select=1),
        'description': fields.text('Description'),
        'competency_ids': fields.many2many('training.competency',
                                           'training_domain_competency_rel',
                                           'domain_id',
                                           'competency_id',
                                           string='Competencies'),
        'profession_ids': fields.many2many('training.profession',
                                           'training_profession_domain_rel',
                                           'domain_id',
                                           'profession_id',
                                           string='Professions'),
    }


    def unlink(self, cr, uid, ids, context=None):
        # search for related competency to unlink
        unlink_comp_ids = set()
        for i in self.browse(cr, uid, ids, context=context):
            comp_rel_found = False
            for c in i.competency_ids:
                if i.name == c.name:
                    comp_rel_found = True
                    unlink_comp_ids.add(c.id)
                    break
            if not comp_rel_found and len(i.competency_ids):
                unlink_comp_ids.add(i.competency_ids[0].id)
        # domain unlink
        result = super(training_domain, self).unlink(cr, uid, ids, context=context)
        # related competency unlink
        if unlink_comp_ids:
            tcomp_proxy = self.pool.get('training.competency')
            tcomp_proxy.unlink(cr, uid, list(unlink_comp_ids))
        return result

    def write(self, cr, uid, ids, vals, context=None):
        # passthough to competency
        tcomp_proxy = self.pool.get('training.competency')
        # get update vals
        comp_vals = {}
        for n in ('name','description'):
            if n in vals:
                comp_vals[n] = vals[n]
        # get update ids
        comp_ids = set()
        for i in self.browse(cr, uid, ids, context=context):
            # find whose competency have the same name as us
            curr_comp_found = False
            for c in i.competency_ids:
                if c.name == i.name:
                    curr_comp_found = True
                    comp_ids.add(c.id)
                    break
            if not curr_comp_found and len(i.competency_ids):
                comp_ids.add(i.competency_ids[0].id)

        result = super(training_domain, self).write(cr, uid, ids, vals, context=context)
        # write competency update
        if comp_vals and comp_ids:
            tcomp_proxy.write(cr, uid, list(comp_ids), comp_vals)
        # return original super result
        return result

    def create(self, cr, uid, vals, context=None):
        new_id = super(training_domain, self).create(cr, uid, vals, context=context)

        # create passthrough competency
        tcomp_proxy = self.pool.get('training.competency')
        comp_vals = vals.copy()
        if 'competency_ids' in comp_vals:
            del comp_vals['competency_ids']
        comp_vals['domain_ids'] = [(6, 0, [new_id])]
        comp_new_id = tcomp_proxy.create(cr, uid, comp_vals, context=context)

        return new_id

training_domain()

class training_competency(osv.osv):
    _name = 'training.competency'
    _columns = {
        'name': fields.char('Name', size=64, required=True, select=1),
        'description': fields.text('Description'),
        'domain_ids': fields.many2many('training.domain',
                                       'training_domain_competency_rel',
                                       'competency_id',
                                       'domain_id',
                                       string='Domains'),
    }
training_competency()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
