# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
from tools.translate import _
import os
import time
import netsvc

class evaluation_questionnaire(osv.osv):
    _name = 'training.evaluation.questionnaire'
evaluation_questionnaire()

class evaluation_category(osv.osv):
    _name = 'training.evaluation.category'
    _description = 'Training Evaluation Category'

    def _complete_name_compute(self, cr, uid, ids, fields, args, context=None):
        res = self.name_get(cr, uid, ids, context=context)
        return dict(res)

    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['name','parent_id'], context=context)
        res = []
        for record in reads:
            name = record['name']
            if record['parent_id']:
                name = record['parent_id'][1]+' / '+name
            res.append((record['id'], name))
        return res

    _columns = {
        'name' : fields.char('Name', size=64, required=True, select=1, translate=True),
        'parent_id' : fields.many2one('training.evaluation.category', 'Parent', select=1),
        'child_ids' : fields.one2many('training.evaluation.category', 'parent_id', 'Children'),
        'complete_name': fields.function(_complete_name_compute,
                                         method=True,
                                         type='string',
                                         string='Complete Name'),
        'type' : fields.selection([('lecturer', 'Lecturer'),
                                   ('course', 'Course'),
                                   ('org', 'Organization'),
                                   ('other', 'Other')],
                                  'Type',
                                  required=True, select=1),
        'partner_id': fields.many2one('res.partner', 'Partner'),
    }

    _defaults = {
        'type' : lambda *a: 'lecturer',
    }

    def on_change_parent(self, cr, uid, ids, parent_id, context=None):
        if not parent_id:
            return False

        category = self.browse(cr, uid, parent_id, context=context)
        return {
            'value' : {
                'type' : category.type,
                'partner_id': category.partner_id and category.partner_id.id or False,
            }
        }

evaluation_category()

class evaluation_question(osv.osv):
    _name = 'training.evaluation.question'

evaluation_question()

class training_evaluation_questionnaire_question_rel(osv.osv):
    _name = 'training.evaluation.questionnaire.question.rel'

    _columns = {
        'sequence' : fields.integer('Sequence'),
        'questionnaire_id' : fields.many2one('training.evaluation.questionnaire', 'Questionnaire', select=1, required=True),
        'question_id' : fields.many2one('training.evaluation.question', 'Question', select=1, required=True),
        'category_id' : fields.related('question_id', 'category_id',
                                       type='many2one',
                                       relation='training.evaluation.category',
                                       string='Category'),
        'question' : fields.related('question_id', 'question', type='text', string='Question'),
        'type' : fields.related('question_id', 'type',
                                type='selection',
                                selection= [('qcu', 'Question 1-5'), ('plain', 'Plain')],
                                string='Type',
                                readonly=True),
    }

    _sql_constraints = [
        ('uniq_course_offer', 'unique(questionnaire_id, question_id)', "You cannot have twice question in this questionnaire"),
    ]

    def on_change_question(self, cr, uid, ids, question_id, context=None):
        if not question_id:
            return {}

        question = self.pool.get('training.evaluation.question').browse(cr, uid, question_id, context=context)

        res = {
            'value' : {
                'category_id' : question.category_id.id,
                'question' : question.question,
                'type' : question.type,
            }
        }

        return res

    _order = 'sequence asc'

training_evaluation_questionnaire_question_rel()

class evaluation_question(osv.osv):
    _name = 'training.evaluation.question'
    _description = 'Training Evaluation Question'

    _columns = {
        'category_id' : fields.many2one('training.evaluation.category', 'Category', select=1, required=True),
        'name' : fields.char('Name', size=64, readonly=True, required=True, select=1,help='Name of Question'),
        'question' : fields.text('Exposition', required=True, select=1, translate=True),
        'note' : fields.text('Note'),
        'type' : fields.selection( [('qcu', 'Question 1-5'), ('plain', 'Plain')], 'Type', required=True, select=1 ,help='Question type'),
    }

    _defaults = {
        'type' : lambda *a: 'qcu',
        'name' : lambda *a: '/',
    }

    def create(self, cr, uid, values, context=None):
        values['name'] = self.pool.get('ir.sequence').get(cr, uid, 'tr.eval.question')
        return super(evaluation_question, self).create(cr, uid, values, context=context)

evaluation_question()

class evaluation_questionnaire(osv.osv):
    _name = 'training.evaluation.questionnaire'
    _description = 'Training Evaluation Questionnaire'

    _columns = {
        'name' : fields.char('Name', size=64, readonly=True, required=True, select=1, help='Name of questionnaire'),
        'state' : fields.selection([('draft', 'Draft'),
                                    ('validated', 'Validated'),
                                    ('pending', 'Pending'),
                                    ('inprogress', 'In Progress'),
                                    ('deprecated', 'Deprecated')
                                   ],
                                   'State',
                                   required=True,
                                   readonly=True,
                                   select=1,
                                   help='The state of the Questionnaire'),
        'kind' : fields.selection([('automatic', 'Automatic'),
                                   ('manual', 'Manual')],
                                  'Kind',
                                  required=True, select=1),
        'question_ids' : fields.one2many('training.evaluation.questionnaire.question.rel',
                                         'questionnaire_id',
                                         'Questions'),
    }

    _defaults = {
        'state' : lambda *a: 'draft',
        'kind' : lambda *a: 'automatic',
        'name' : lambda *a: '/',
    }

    def _check_questionnaire(self, cr, uid, ids, context=None):
        for obj in self.browse(cr, uid, ids, context=context):
            if obj.state == 'validated':
                cnt = self.search_count(cr, uid, [('state', '=', 'validated'),
                                                  ('id', '!=', obj.id)],
                                        context=context)
                if cnt != 0:
                    return False
        return True

    _constraints = [
        (_check_questionnaire, "You can not have two validated questionnaires.", []),
    ]

    def create(self, cr, uid, values, context=None):
        values['name'] = self.pool.get('ir.sequence').get(cr, uid, 'tr.eval.questionnaire')
        return super(evaluation_questionnaire, self).create(cr, uid, values, context=context)

    def get_validated_questionnaire(self, cr, uid, context=None):
        ids = self.search(cr, uid, [('state', '=', 'validated')], context=context)
        if not ids:
            raise osv.except_osv(_('Warning'),
                                 _('No validated questionnaire for the evaluation'))
        return ids[0]

    def action_signal_draft(self, cr, uid, ids, context=None):
        workflow = netsvc.LocalService('workflow')
        for oid in ids:
            workflow.trg_create(uid, self._name, oid, cr)
        return self.write(cr, uid, ids, {'state' : 'draft'}, context=context)


evaluation_questionnaire()


class evaluation_global(osv.osv):
    _name = 'training.evaluation.global'
    _description = 'Global Evaluation'

    def _participation_ids_compute(self, cr, uid, ids, fields, args, context=None):
        res = dict.fromkeys(ids, 0)
        for obj in self.browse(cr, uid, ids, context=context):
            res[obj.id] = [psh.id for psh in obj.seance_id.contact_ids]
        return res

    def search_evaluation_ids_with_contacts(self, cr, uid, contact_ids, context=None):
        res = dict.fromkeys(contact_ids, [])
        if contact_ids:
            return res

        cr.execute("""
               SELECT teg.id, tps.contact_id
               FROM training_evaluation_global teg,
                    training_participation_stakeholder tps
               WHERE teg.seance_id = tps.seance_id
                 AND tps.contact_id IN (%s)""", contact_ids)

        for evaluation_id, contact_id in cr.fetchall():
            res[contact_id].append(evaluation_id)

        return res

    def do_evaluation_scan(self, cr, uid, res_id, page_num, scanner, context=None):
        position_storage_pool = self.pool.get('position.storage')
        evaluation_pool = self.pool.get('training.evaluation')

        checkboxes_position = position_storage_pool.load_checkboxes_position(cr, uid, self._name, res_id, page_num, context)
        checkboxes_context = position_storage_pool.load_checkboxes_context(cr, uid, self._name, res_id, page_num, context)
        if not checkboxes_position:
            msg = "No checkboxes position found for global evaluation %s on page %s" % (res_id, page_num)
            return

        values = scanner.detect_checkboxes_values(checkboxes_position)
        questions = {}

        for idx, value in enumerate(values):
            q_id, r_num = map(int, checkboxes_context[idx].split('-'))
            if q_id not in questions:
                questions[q_id] = 0
            if value:
                if questions[q_id] != 0:
                    # TODO: Handle case where we cannot have more than 1 response
                    pass
                questions[q_id] = r_num

        # create a new evaluation, and load defaut questions
        new_eval_id = evaluation_pool.create(cr, uid, {'evaluation_global_id': res_id, }, context=context)
        evaluation_pool.load_questions(cr, uid, [new_eval_id], context=context)
        # update results
        new_eval_record = evaluation_pool.browse(cr, uid, new_eval_id, context=context)
        for eval_line in new_eval_record.line_ids:
            value = questions.get(eval_line.question_id.id, None)
            if value is not None:
                eval_line.write({'response_value': value})
        return True

    _columns = {
        'name' : fields.char('Reference', size=64, required=True, select=1, readonly=True),
        'evaluation_ids' : fields.one2many('training.evaluation', 'evaluation_global_id', 'Evaluations'),
        'seance_id' : fields.many2one('training.seance', 'Seance', domain="[('kind', '=', 'standard')]", select=1, required=True),
        'course_id' : fields.related('seance_id', 'course_id',
                                     string='Course',
                                     type='many2one',
                                     relation='training.course',
                                     readonly=True,
                                     select=1),
        'date' : fields.related('seance_id', 'date', string='Date', type='datetime', readonly=True, select=1),
        'participation_ids' : fields.function(_participation_ids_compute, method=True,
                                              type='one2many', relation='training.participation.stakeholder',
                                              readonly=True, string='Lecturers'),
        'questionnaire_id' : fields.many2one('training.evaluation.questionnaire', 'Questionnaire', required=True),
        'summary': fields.text('Summary'),
    }

    _defaults = {
        'name' : lambda *a: '/',
    }

    _sql_constraints = [
        ('unique_seance_id', 'UNIQUE(seance_id)', _('Only one global Evaluation instance per Seance should exist')),
    ]

    def create(self, cr, uid, values, context=None):
        values['name'] = self.pool.get('ir.sequence').get(cr, uid, 'tr.eval.global')
        return super(evaluation_global, self).create(cr, uid, values, context=context)

    def search(self, cr, uid, domain, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}

        evaluation_contact_id = ('evaluation_contact_id' in context and context.pop('evaluation_contact_id')) or False
        if evaluation_contact_id:
            cr.execute("""
                       SELECT teg.id
                       FROM training_evaluation_global teg, training_participation_stakeholder tps
                       WHERE teg.seance_id = tps.seance_id
                       AND tps.contact_id = %s
                       """, (evaluation_contact_id,))

            return [x[0] for x in cr.fetchall()]

        evaluation_course_id = ('evaluation_course_id' in context and context.pop('evaluation_course_id')) or False
        if evaluation_course_id:
            cr.execute("""
                       SELECT teg.id
                       FROM training_evaluation_global teg, training_seance ts
                       WHERE teg.seance_id = ts.id
                         AND ts.course_id = %s
                       """, (evaluation_course_id,))

            return [x[0] for x in cr.fetchall()]

        return super(evaluation_global, self).search(cr, uid, domain, offset=offset, limit=limit, order=order, context=context, count=count)

evaluation_global()

class evaluation(osv.osv):
    _name = 'training.evaluation'

    def load_questions(self, cr, uid, ids, context=None):
        ev_line = self.pool.get('training.evaluation.line')
        for obj in self.browse(cr, uid, ids, context=context):
            cr.execute('select question_id from training_evaluation_line where evaluation_id=%s',(obj.id,))
            question_ids = map(lambda x: x[0], cr.fetchall())
            for question_rel in obj.questionnaire_id.question_ids:
                if question_rel.question_id.id not in question_ids:
                    values = {
                        'evaluation_id' : obj.id,
                        'question_id' : question_rel.question_id.id,
                    }
                    ev_line.create(cr, uid, values, context=context)

        return True

    _columns = {
        'name' : fields.char('Reference', size=64, required=True, readonly=True, select=1),
        'evaluation_global_id' : fields.many2one('training.evaluation.global', 'Global Evaluation', required=True, select=1, ondelete='restrict'),
        'seance_id' : fields.related('evaluation_global_id', 'seance_id', type='many2one', relation='training.seance', string='Seance', readonly=True, store=True, select=1),
        'course_id' : fields.related('evaluation_global_id', 'seance_id', 'course_id',
                                     string='Course',
                                     type='many2one',
                                     relation='training.course',
                                     readonly=True,
                                     select=1,
                                    ),

        'date' : fields.related('evaluation_global_id', 'seance_id', 'date', string='Date', type='datetime', readonly=True, select=1),
        'questionnaire_id' : fields.related('evaluation_global_id', 'questionnaire_id', type='many2one', relation='training.evaluation.questionnaire', readonly=True, string="Questionnaire"),
        'line_ids' : fields.one2many('training.evaluation.line', 'evaluation_id', 'Lines'),
    }

    _defaults = {
        'name' : lambda *a: '/',
    }

    def create(self, cr, uid, values, context=None):
        values['name'] = self.pool.get('ir.sequence').get(cr, uid, 'tr.eval')
        return super(evaluation, self).create(cr, uid, values, context=context)

    def on_change_seance(self, cr, uid, ids, seance_id, context=None):
        if not seance_id:
            return {}
        seance = self.pool.get('training.seance').browse(cr, uid, seance_id, context=context)
        res = {
            'value' : {
                'course_id' : seance.course_id.id,
                'date' : seance.date,
            },
        }
        return res

    def on_change_evaluation_global(self, cr, uid, ids, evg_id, context=None):
        if not evg_id:
            return {
                'value' : {
                    'seance_id' : False,
                    'course_id' : False,
                    'date' : False,
                    'questionnaire_id' : False,
                }
            }

        evg = self.pool.get('training.evaluation.global').browse(cr, uid, evg_id, context=context)
        return {
            'value' : {
                'seance_id' : evg.seance_id.id,
                'course_id' : evg.seance_id.course_id.id,
                'date' : evg.seance_id.date,
                'questionnaire_id' : evg.questionnaire_id.id,
            }
        }

evaluation()

class evaluation_line(osv.osv):
    _name = 'training.evaluation.line'
    _rec_name = 'question_id'

    def on_change_question(self, cr, uid, ids, question_id, context=None):
        if not question_id:
            return False

        question = self.pool.get('training.evaluation.question').browse(cr, uid, question_id, context=context)

        return {
            'value': {
                'type_question_id' : question.type,
            }
        }

    _columns = {
        'evaluation_id' : fields.many2one('training.evaluation', 'Evaluation', ondelete='cascade'),
        'question_id' : fields.many2one('training.evaluation.question', 'Question'),

        'type_question_id' : fields.related('question_id', 'type',
                                            type='selection',
                                            selection = [('qcu', 'Question 1-5'),
                                                         ('plain', 'Plain')],
                                            string='Type',
                                            store=True,
                                            readonly=True),

        # Response from the user
        'response_plain' : fields.text('Response'),
        'response_value' : fields.integer('Response'),

        'category_question_id' : fields.related('question_id', 'category_id',
                                                type='many2one',
                                                relation='training.evaluation.category',
                                                string='Category',
                                                store=True,
                                                readonly=True),
        'category_question_partner_id': fields.related('question_id', 'category_id', 'partner_id',
                                                type='many2one',
                                                relation='res.partner',
                                                string='Partner',
                                                store=True,
                                                readonly=True),
        'type_category_question_id' : fields.related('question_id', 'category_id', 'type',
                                            type='selection',
                                            selection = [('lecturer', 'Lecturer'),
                                                         ('course', 'Course'),
                                                         ('other', 'Other')],
                                            string='Type Category',
                                            store=True,
                                            readonly=True),
        'exposition_question_id' : fields.related('question_id', 'question', type='text', string='Exposition', readonly=True),
    }

evaluation_line()

class training_seance_generate_pdf_wizard(osv.osv_memory):
    _inherit = 'training.seance.generate.zip.wizard'
    _columns = {
        'evaluations_report' : fields.boolean('Evaluations',
                                             help="If you select this option, you will print the evaluations report. The file name format is Evaluation_DATE_INDEX.pdf"),
        'evaluations_number' : fields.integer('Number',
                                             help="The number of reports to print. The default value comes from the Confirmed Seats field"),
    }

    def _default_evaluations_number(self, cr, uid, context=None):
        if context is None:
            context = {}

        active_id = context.get('active_id')
        if active_id:
            seance = self.pool.get('training.seance').browse(cr, uid, active_id, context=context)
            return seance[['participant_count', 'participant_count_manual'][seance.manual]]

        return 0

    _defaults = {
        'evaluations_report' : lambda *a: False,
        'evaluations_number' : _default_evaluations_number,
    }

    def add_selections(self, cr, uid, ids, directory, log, context=None):
        if context is None:
            context = {}

        active_id = context.get('active_id')
        evg_pool = self.pool.get('training.evaluation.global')
        ev_quizz_pool = self.pool.get('training.evaluation.questionnaire')

        seance = self.pool.get('training.seance').browse(cr, uid, active_id, context=context)
        ts = time.strptime(seance.date, '%Y-%m-%d %H:%M:%S')
        date = time.strftime('%Y%m%d', ts)

        evaluation_directory = os.path.join(directory, 'Evaluations')
        os.mkdir(evaluation_directory)

        for obj in self.browse(cr, uid, ids, context=context):
            if obj.evaluations_report and obj.evaluations_number > 0:
                evg_ids = evg_pool.search(cr, uid, [('seance_id','=',seance.id)], context=context)
                if evg_ids:
                    evg_id = evg_ids[0]
                else:
                    evg_id = evg_pool.create(cr, uid, {
                        'seance_id' : seance.id,
                        'questionnaire_id' : ev_quizz_pool.get_validated_questionnaire(cr, uid, context=context),
                    }, context=context)
                res = self._get_report(cr, uid, evg_id, 'report.training.evaluation.empty.report', context=context)

                for idx in range(obj.evaluations_number):
                    filename = os.path.join(evaluation_directory, 'Evaluation_%s_%03d.pdf' % (date, idx+1))
                    fp = file(filename, 'w')
                    fp.write(res)
                    fp.close()

        return super(training_seance_generate_pdf_wizard, self).add_selections(cr, uid, ids, directory, log, context=context)

training_seance_generate_pdf_wizard()

class ir_attachment_eval_unlink(osv.osv):
    _inherit = 'ir.attachment'

    def unlink(self, cr, uid, ids, context=None):
        positions_by_model = {}
        positions_to_delete = []
        posstor_proxy = self.pool.get('position.storage')

        for attach in self.browse(cr, uid, ids, context=context):
            if attach.res_model == 'training.evaluation.global' and attach.datas_fname.startswith('EVAL') and attach.datas_fname.endswith('.pdf'):
                positions_by_model.setdefault(attach.res_model, []).append(attach.res_id)
        for model, model_ids in positions_by_model.iteritems():
            sids = posstor_proxy.search(cr, uid,
                    [('res_model','=',model), ('res_id','in',model_ids)])
            if sids:
                positions_to_delete.extend(sids)
        result = super(ir_attachment_eval_unlink, self).unlink(cr, uid, ids, context=context)
        if result:
            result_position = posstor_proxy.unlink(cr, uid, positions_to_delete, context=context)
        return result

ir_attachment_eval_unlink()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
