# -*- encoding: utf-8 -*-
############################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    Copyright (C) 2008-2009 AJM Technologies S.A. (<http://www.ajm.lu). All Rights Reserved
#    Copyright (C) 2010-2011 Thamini S.à.R.L (<http://www.thamini.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################################

from osv import osv
from osv import fields

import tools
import time
from tools.translate import _
import netsvc
from datetime import datetime
from dateutil.relativedelta import relativedelta

WRITABLE_ONLY_IN_DRAFT = dict(readonly=True, states={'draft': [('readonly', False)]})

class res_partner_job(osv.osv):
    _inherit = 'res.partner.job'
#
    def search(self, cr, uid, domain, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}
        participation_proxy = self.pool.get('training.participation')
        correct = context.get('wizard_create_correction_request', False)
        if correct:
            job_set = set()
            part_ids = context.get('active_ids', [])
            for p in participation_proxy.browse(cr, uid, context.get('active_ids', []), context=context):
                if p.seance_id and p.seance_id.kind == 'exam':
                    if p.course_questionnaire_id:
                        lecturers = p.course_questionnaire_id.lecturer_ids
                        if lecturers:
                            job_set.update([ x.id for x in lecturers])
            if job_set:
                domain.extend([('id', 'in', list(job_set))])
        return super(res_partner_job, self).search(cr, uid, domain, offset=0, limit=limit, order=order, context=context, count=count)

res_partner_job()

class training_config_contact_function(osv.osv):
    _inherit = 'training.config.contact.function'

    def _get_values_of_kind(self):
        vals = super(training_config_contact_function, self)._get_values_of_kind()
        if not vals:
            vals = []
        vals.append(('corrector', 'Corrector'))
        return vals

training_config_contact_function()

class exam_correction_request(osv.osv):
    _name = 'training.exam.correction.request'
    _description = 'Correction Request'
    _order = 'create_date ASC'

    def _amount_untaxed(self, cr, uid, ids, name, args, context=None):
        result = dict.fromkeys(ids, 0.0)
        if not ids:
            return result
        for request in self.browse(cr, uid, ids, context=context):
            correction_count = len([ c.id for c in request.correction_ids if c.state != 'moved' ])
            result[request.id] = request.product_id.standard_price * correction_count
        return result

    def _get_invoices_and_paid(self, cr, uid, ids, fieldname, args, context=None):
        STATES_TO_SKIP = ('cancel')
        result = {}
        for id in ids:
            result.setdefault(id, {'paid': False, 'invoice_ids': [],
                                    'invoiced_rate': 0.0, 'invoiced': False})

        for request in self.browse(cr, uid, ids, context=context):
            invoices = {}
            order = request.po_id
            if not order:
                continue

            invoiced_amount = 0.0
            for invoice in order.invoice_ids:
                invoices[invoice.id] = invoice.reconciled
                if invoice.state not in STATES_TO_SKIP:
                    invoiced_amount += invoice.amount_untaxed

            for line in order.order_line:
                if not line.invoiced:
                    continue
                for invoice_line in line.invoice_lines:
                    invoice = invoice_line.invoice_id
                    if invoice:
                        invoices[invoice.id] = invoice.reconciled
                        if invoice.state not in STATES_TO_SKIP:
                            invoiced_amount += invoice_line.price_subtotal

            invoiced_rate = min(100.0, invoiced_amount * 100.0 / (order.amount_untaxed or 1.0))
            result[request.id]['invoice_ids'] = invoices.keys()
            result[request.id]['invoiced_rate'] = invoiced_rate
            result[request.id]['invoiced'] = invoiced_rate >= 100.0 and True or False
            if invoices and all(invoices.values()):
                result[request.id]['paid'] = True
        return result

    def _search_fully_invoiced_state(self, cr, uid, obj, fieldname, args, context=None):
        search_invoiced = False
        for a in args:
            if isinstance(a, (tuple,list)) and len(a) == 3:
                if a[0] == fieldname:
                    if a[2]:
                        search_invoiced = True
                    else:
                        search_invoiced = False
                    break
        request_ids = self.search(cr, uid, [], context=context)
        result_ids = []
        for request in self.browse(cr, uid, request_ids, context=context):
            if request.invoiced == search_invoiced:
                result_ids.append(request.id)
        if result_ids:
            return [('id','in',tuple(result_ids))]
        return []

    _columns = {
        'name' : fields.char('Reference', size=32, required=True, readonly=True, select=1),
        'job_id' : fields.many2one('res.partner.job', 'Job', required=True, select=1,
                                   **WRITABLE_ONLY_IN_DRAFT),
        'contact_id' : fields.related('job_id', 'contact_id', type='many2one', relation='res.partner.contact',
                                      select=1, string='Contact', store=True, readonly=True),
        'partner_id' : fields.related('job_id', 'name', string='Partner', type='many2one', relation='res.partner',
                                      readonly=True, select=1, store=True),
        'create_date' : fields.datetime('Create Date', readonly=True),
        'validate_date' : fields.datetime('Validate Date', select=1, readonly=True),
        'acceptation_date': fields.datetime('Acceptation Date', select=2, readonly=True),
        'forward_looking_date' : fields.datetime('Forward Looking Date', select=1, required=True, write=['training.group_training']),
        'correction_ids' : fields.one2many('training.exam.correction', 'request_id', 'Corrections', **WRITABLE_ONLY_IN_DRAFT),
        'po_id' : fields.many2one('purchase.order', 'Purchase Order', select=1, **WRITABLE_ONLY_IN_DRAFT),
        'po_id_state' : fields.related('po_id', 'state', type='selection',
                                       selection=[('draft', 'Request for Quotation'),
                                                  ('wait', 'Waiting'),
                                                  ('confirmed', 'Confirmed'),
                                                  ('approved', 'Approved'),
                                                  ('except_picking', 'Shipping Exception'),
                                                  ('except_invoice', 'Invoice Exception'),
                                                  ('done', 'Done'),
                                                  ('cancel', 'Cancelled')
                                                 ], string='Order Status', readonly=True),
        'po_id_amount' : fields.related('po_id', 'amount_total', type='float', string='Total Amount', readonly=True),
        'product_id' : fields.many2one('product.product', 'Product', select=2, required=True, **WRITABLE_ONLY_IN_DRAFT),
        'state': fields.selection([('draft', 'Draft'),
                                   ('validated', 'Validated'),
                                   ('requested', 'Requested'),
                                   ('accepted', 'Accepted'),
                                   ('refused', 'Refused'),
                                   ('cancelled', 'Cancelled'),
                                   ('done', 'Done'),
                                  ], 'State', readonly=True, required=True, select=1),
        'payment_mode_id': fields.many2one('training.request.payment.mode', 'Payment Mode', **WRITABLE_ONLY_IN_DRAFT),
        'email' : fields.char('Email', size=128, select=1, **WRITABLE_ONLY_IN_DRAFT),
        'amount_untaxed': fields.function(_amount_untaxed, string='Amount untaxed', type='float', method=True,
                                            help='Request total untaxed amount'),
        'invoiced': fields.function(_get_invoices_and_paid, type='boolean', string='Invoiced', method=True, multi='invoices',
                                    fnct_search=_search_fully_invoiced_state,
                                    help='Indicate if the request is fully invoiced'),
        'invoiced_rate': fields.function(_get_invoices_and_paid, type='float', string='Invoiced Rate', method=True, multi='invoices'),
        'invoice_ids': fields.function(_get_invoices_and_paid, string='Invoices', type='many2many', relation='account.invoice', method=True, multi='invoices'),
        'paid': fields.function(_get_invoices_and_paid, string='Paid', type='boolean', method=True, multi='invoices',
                                help='This request is fully paid / reconciled'),
    }

    def create(self, cr, uid, vals, context=None):
        vals['name'] = self.pool.get('ir.sequence').get(cr, uid, 'training.excr')
        return super(exam_correction_request, self).create(cr, uid, vals, context)

    def _default_forward_looking_date(self, cr, uid, context=None):
        return (datetime.now() + relativedelta(days=15)).strftime('%Y-%m-%d 08:30:00')

    _defaults = {
        'state' : lambda *a: 'draft',
        'name' : lambda *a: '/',
        'forward_looking_date' : _default_forward_looking_date,
    }

    @tools.cache(skiparg=4)
    def _get_stock_location(self, cr, uid, context=None):
        purchase_order_pool = self.pool.get('purchase.order')
        default_warehouse_id = purchase_order_pool.default_get(cr, uid, ['warehouse_id'], context=context).get('warehouse_id')
        if not default_warehouse_id:
            return False
        location_id = self.pool.get('stock.warehouse').browse(cr, uid, default_warehouse_id, context=context).lot_input_id.id
        return location_id

    def _prepare_purchase_order(self, cr, uid, request, context=None):
        values = {
            'name': "Correction Request : %s" % (request.name),
            'partner_id': request.job_id.name.id,
            'contact_id': request.job_id.contact_id.id,
            'partner_address_id': request.job_id.address_id.id,
            'pricelist_id': request.job_id.name.property_product_pricelist_purchase.id,
            'location_id': self._get_stock_location(cr, uid, context),
            'invoice_method' : request.payment_mode_id.invoice_method,
            'fiscal_position': request.payment_mode_id.fiscal_position_id.id,
        }
        return values

    def action_create_purchase_order(self, cr, uid, ids, context=None):
        purchase_obj = self.pool.get('purchase.order')
        purchase_line_obj = self.pool.get('purchase.order.line')
        exam_correction_obj = self.pool.get('training.exam.correction')

        for request in self.browse(cr, uid, ids, context=context):
            if not request.payment_mode_id:
                raise osv.except_osv(_('Error'),
                                     _('The Payment Mode is not specified'))
            request_values = self._prepare_purchase_order(cr, uid, request, context=context)
            new_order_id = purchase_obj.create(cr, uid, request_values, context=context)
            request.write({'po_id' : new_order_id})
            order = purchase_obj.browse(cr, uid, new_order_id, context=context)
            for corr_participation in request.correction_ids:
                values = exam_correction_obj._prepare_purchase_order_line(cr, uid, corr_participation, order, context=context)
                new_line_id = purchase_line_obj.create(cr, uid, values, context=context)
                corr_participation.write({'pol_id' : new_line_id})
        return True

    def _trigger_purchase_order_signal(self, cr, uid, ids, signal_name, context=None):
        wf_service = netsvc.LocalService('workflow')
        for request in self.browse(cr, uid, ids, context=context):
            if request.po_id:
                wf_service.trg_validate(uid, 'purchase.order', request.po_id.id, signal_name, cr)
        return True

    def _send_request_email(self, cr, uid, ids, email_type, context=None, **kwargs):
        email_proxy = self.pool.get('training.email')
        for request in self.browse(cr, uid, ids, context=context):
            email_proxy.send_email(cr, uid, email_type, 'e', request.email, context=context, request=request)
        return True

    def action_wkf_draft(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state' : 'draft'}, context=context)

    def test_wkf_cancel(self, cr, uid, ids, context=None):
        has_corrected = any(correction.state == 'corrected' for request in self.browse(cr, uid, ids, context=context) for correction in request.correction_ids)
        return not has_corrected

    def _do_cancel(self, cr, uid, ids, context=None):
        for request in self.browse(cr, uid, ids, context=context):
            for correction in request.correction_ids:
                correction.write({'state' : 'moved'})
                correction.participation_id.write({'correction_id': False})

            if request.po_id:
                if request.po_id.state in ('approved','except_picking','execpt_invoice','wait'):
                    request.po_id.action_cancel(context=context)
                else:
                    self._trigger_purchase_order_signal(cr, uid, ids, 'purchase_cancel', context=context)
        return True

    def action_wkf_cancel(self, cr, uid, ids, context=None):
        self._do_cancel(cr, uid, ids, context=context)
        return self.write(cr, uid, ids, {'state' : 'cancelled'}, context=context)

    def test_wkf_done(self, cr, uid, ids, context=None):
        return all(corr.state in ('moved','corrected') for corr_rq in self.browse(cr, uid, ids, context=context) for corr in corr_rq.correction_ids)

    def action_wkf_done(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state' : 'done'}, context=context)

    def test_wkf_validate(self, cr, uid, ids, context=None):
        for request in self.browse(cr, uid, ids, context=context):
            if not len(request.correction_ids) > 0:
                raise osv.except_osv(_('Error'),
                                     _('There is not any correction specify'))

        return all(bool(request.po_id) != False and len(request.correction_ids) > 0 for request in self.browse(cr, uid, ids, context=context))

    def action_wkf_validate(self, cr, uid, ids, context=None):
        self._trigger_purchase_order_signal(cr, uid, ids, 'purchase_confirm', context=context)
        return self.write(cr, uid, ids,
                          {'state' : 'validated',
                           'validate_date' : time.strftime('%Y-%m-%d %H:%M:%S'),
                           'forward_looking_date': self._default_forward_looking_date(cr, uid, context=context),
                          },
                          context=context)

    def test_wkf_request(self, cr, uid, ids, context=None):
        return True

    def action_wkf_request(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state' : 'requested'}, context=context)

    def action_wkf_send_request_email(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        for request in self.browse(cr, uid, ids, context=context):
            request_context = context.copy()
            if request.partner_id:
                request_context['lang'] = request.partner_id.lang or 'en_US'
            self._send_request_email(cr, uid, [request.id], 'correction_request', context=request_context)
        return True

    def action_wkf_refuse(self, cr, uid, ids, context=None):
        for request in self.browse(cr, uid, ids, context=context):
            request._do_cancel(context=context)
            self._send_request_email(cr, uid, [request.id], 'correction_refuse', context=context)
        return self.write(cr, uid, ids, {'state' : 'refused'}, context=context)

    def action_wkf_accept(self, cr, uid, ids, context=None):
        self._trigger_purchase_order_signal(cr, uid, ids, 'purchase_approve', context=context)
        self._send_request_email(cr, uid, ids, 'correction_accept', context=context)
        vals = {
            'state': 'accepted',
            'acceptation_date': time.strftime('%Y-%m-%d %H:%M:%S'),
        }
        return self.write(cr, uid, ids, vals, context=context)

    def action_reset_to_draft(self, cr, uid, ids, context=None):
        if not ids:
            return False
        workflow = netsvc.LocalService('workflow')
        workflow.trg_create(uid, self._name, ids[0], cr)
        #FIXME: also propagate this to the purchase order!
        return True

    def action_wkf_send_cancellation_email(self, cr, uid, ids, context=None):
        self._send_request_email(cr, uid, ids, 'correction_cancel', context=context)
        return True

    def action_create_invoice(self, cr, uid, ids, context=None):
        for inv in self.browse(cr, uid, ids, context=context):
            if len(inv.invoice_ids) > 0:
                raise osv.except_osv(_('Error'),
                                     _('Invoice already created'))
        return self.pool.get('purchase.order').action_invoice_create(cr, uid,
                                                                     [obj.po_id.id for obj in self.browse(cr, uid, ids, context=context) if obj.po_id]
                                                                     )
    def on_change_job(self, cr, uid, ids, job_id, context=None):
        if not job_id:
            return False

        job = self.pool.get('res.partner.job').browse(cr, uid, job_id, context=context)
        return {
            'value' : {
                'email': job.email,
                'partner_id' : job.name.id,
            }
        }

exam_correction_request()

class exam_correction(osv.osv):
    _name = 'training.exam.correction'

    def name_get(self, cr, uid, ids, context=None):
        return [(obj.id, obj.request_id.name) for obj in self.browse(cr, uid, ids, context=context)]

    def f_get_courses(self, cr, uid, ids, fname, args, context=None):
        res = dict.fromkeys(ids, '')
        for tec in self.browse(cr, uid, ids, context=context):
            quizz = tec.participation_id.questionnaire_id
            if quizz:
                courses = []
                for quizz_course in quizz.course_ids:
                    courses.append(quizz_course.course_id.name)
                res[tec.id] = '\n'.join(courses)
        return res

    _columns = {
        'name' : fields.char('Reference', size=16, required=True, select=1),
        'request_id' : fields.many2one('training.exam.correction.request', 'Request', required=True, ondelete='cascade'),
        'participation_id' : fields.many2one('training.participation', 'Participation', required=True,
                                             domain="[('kind', '=', 'exam'), ('result_received', '=', 0)]"),
        'course_ids': fields.function(f_get_courses, method=True, type='text', string='Course', readonly=1),
        'date' : fields.related('participation_id', 'date', type='datetime', readonly=True, string='Date'),
        'result' : fields.related('participation_id', 'result', type='float', readonly=True, string='Result'),
        'present' : fields.related('participation_id', 'present', type='boolean', string='Present', readonly=True),
        'passing_score' : fields.related('participation_id', 'passing_score', type='float', readonly=True, string="Passing Score"),
        'succeeded': fields.related('participation_id', 'succeeded', type='selection',
                                    selection=[('n/a', 'N/A'),
                                               ('no', 'No'),
                                               ('yes', 'Yes')], readonly=True, string='Succeeded'),
        'pol_id' : fields.many2one('purchase.order.line', 'Purchase Order Line', readonly=True),
        'po_id' : fields.related('pol_id', 'order_id', string='Purchase Order', type='many2one', relation='purchase.order', readonly=True),
        'state' : fields.selection([('to_correct', 'To Correct'),
                                    ('corrected', 'Corrected'),
                                    ('moved', 'Moved')],
                                   'State', readonly=True, required=True),
    }

    _defaults = {
        'name' : lambda *a: '/',
        'state' : lambda *a: 'to_correct',
    }

    def create(self, cr, uid, values, context=None):
        values['name'] = self.pool.get('ir.sequence').get(cr, uid, 'training.exc')
        correction_id = super(exam_correction, self).create(cr, uid, values, context=context)
        self.pool.get('training.participation').write(cr, uid, values['participation_id'], {'correction_id' : correction_id}, context=context)
        return correction_id

    def unlink(self, cr, uid, ids, context=None):
        if any(obj.state in ('corrected', 'moved') for obj in self.browse(cr, uid, ids, context=context)):
            raise osv.except_osv(_('Warning'),
                                 _("""You can not remove a corrected/moved correction"""))

        for obj in self.browse(cr, uid, ids, context=context):
            obj.participation_id.write({'correction_id' : False})

        return super(exam_correction, self).unlink(cr, uid, ids, context=context)

    def move_cb(self, cr, uid, ids, context=None):
        wkf = netsvc.LocalService('workflow')

        for obj in self.browse(cr, uid, ids, context=context):
            obj.participation_id.write({'correction_id' : 0})
            obj.write({'state' : 'moved'})

            wkf.trg_validate(uid, 'training.exam.correction.request', obj.request_id.id, 'excr_done', cr)

        return True

    def done_cb(self, cr, uid, ids, context=None):
        wkf = netsvc.LocalService('workflow')
        for obj in self.browse(cr, uid, ids, context=context):
            obj.write({'state': 'corrected'})
            wkf.trg_validate(uid, 'training.exam.correction.request', obj.request_id.id, 'excr_done', cr)
        return True

    def _prepare_purchase_order_line(self, cr, uid, corr_participation, order, context=None):
        fiscal_position_obj = self.pool.get('account.fiscal.position')
        obj = corr_participation

        if not obj.participation_id.course_questionnaire_id:
            raise osv.except_osv(_('Error'),
                                 _('Could not create Purchase Order Line for participation which does not have any Exam set'))
        price = 0 #obj.price
        qty = 1.0 #obj.seance_id.duration * 1.0
        product = obj.request_id.product_id

        # analytic distribution
        analytics_id = False
        account_analytic_id = obj.participation_id.course_questionnaire_id.analytic_account_id.id
        if account_analytic_id:
            prsh_analytic_journal_ids = self.pool.get('account.analytic.journal').search(cr, uid, [('type','=','purchase')])
            if not prsh_analytic_journal_ids:
                raise osv.except_osv(_('Error'), _('No purchase analytic journal defined'))
            analytics_id = self.pool.get('account.analytic.plan.instance').create(cr, uid, {
                'journal_id': prsh_analytic_journal_ids[0],
                'account_ids': [
                    (0, 0, {
                        'analytic_account_id': account_analytic_id,
                        'rate': 100.0,
                    }),
                ],
            }, context=context)

        # update purchase order line taxes
        supplier_tax_ids = fiscal_position_obj.map_tax(cr, uid, order.fiscal_position, product.supplier_taxes_id, context=context)

        values = {
            'name' : 'Correction %s' % obj.name,
            'date_planned' : obj.request_id.forward_looking_date,
            'order_id' : order.id,
            'price_unit' : product.standard_price,
            'product_qty' : 1.0,
            'product_id' : product.id,
            'product_uom' : product.uom_id.id,
            'taxes_id': [(6, 0, supplier_tax_ids)],
            'analytics_id': analytics_id,
            #'account_analytic_id': obj.seance_id.course_id.analytic_account_id and obj.seance_id.course_id.analytic_account_id.id or '',
        }

        return values

exam_correction()

class training_participation(osv.osv):
    _inherit = 'training.participation'

    _columns = {
        'correction_id' : fields.many2one('training.exam.correction', 'Correction', readonly=True),
        'correction_state_id' : fields.related('correction_id', 'state', type='selection', size=32, string="State", readonly=True, selection=[('to_correct', 'To Correct'),('corrected', 'Corrected'), ('moved', 'Moved')]),
        'correction_contact_id': fields.related('correction_id', 'request_id', 'contact_id', type='many2one', relation='res.partner.contact', string='Corrector Contact', readonly=True),
    }

    def do_correction_cb(self, cr, uid, ids, context=None):
        for obj in self.browse(cr, uid, ids, context=context):
            if not all([ x.graded for x in obj.participation_line_ids ]):
                raise osv.except_osv(_('Error'), _('Can\'t mark this participation as done because some candidate responses are note marked as graded'))
            obj.write({'result_received' : 1})
            obj.correction_id.done_cb()
        return True

    def dump_correction_info(self, q, page_num, r_list, r_user):
        title_data = []
        user_data = []
        for i, r in enumerate(r_list):
            title_data.append("(%08d) [%s] %s" % (r[0], r[2] and 'x' or ' ', r[1]))
            user_data.append("(%08d) [%s] %s" % (r_user[i][0], r_user[i][2] and 'x' or ' ', r_user[i][1]))
        linesep = "----- ----- -----"
        result = "To question %s (id:%d) on page %d\n%s\n%s\nGood responses:\n%s\n%s\nUser respones:\n%s\n%s\n" % (
            q['name'], q['id'], page_num, q['question'], linesep, '\n'.join(title_data), linesep, '\n'.join(user_data), linesep
        )
        if q['count_points_from'] == 'answers' and q['type'] == 'yesno':
            result += 'Good Answer: %.2f, Wrong Answer: %.2f, No Answer: %.2f\n' % (
                        q['point_yesno_good_answer'], q['point_yesno_wrong_answer'], q['point_yesno_no_answer'])
        return result

    def do_correction_auto(self, cr, uid, res_id, page_num, scanner, context=None):
        res_obj = self.browse(cr, uid, res_id)
        # first check that que res_id actually exists
        try:
            res_obj and res_obj.questionnaire_id.id or 0
        except KeyError:
            return
        # do not do correction on non-automatic questionnaire
        if not (res_obj and res_obj.questionnaire_id and res_obj.questionnaire_id.kind == u'automatic'):
            return

        position_storage_proxy = self.pool.get('position.storage')
        checkboxes_position = position_storage_proxy.load_checkboxes_position(cr, uid, self._name, res_id, page_num, context)
        checkboxes_context = position_storage_proxy.load_checkboxes_context(cr, uid, self._name, res_id, page_num, context)
        if not checkboxes_position:
            msg = "No checkbox found for training participation %s on page %s." % (res_id, page_num)
            # TODO log.warning(msg)
            return

        values = scanner.detect_checkboxes_values(checkboxes_position)

        user_answer_pool = self.pool.get('training.participation.line')
        #def update_response(q_id,
        responses = {}  # { 'question_id': ['responses'...] }
        for i, a_data in enumerate(checkboxes_context):
            q_id, a_id = map(int, a_data.split('-'))

            if not q_id in responses:
                responses[q_id] = []
            # if checked add as a response
            if values[i]:
                responses[q_id].append(a_id)

        questions = {}
        question_qcm_ids = set()
        # read question datas
        q_question_pool = self.pool.get('training.exam.question')
        qd = q_question_pool.read(cr, uid, responses.keys(), ['id', 'type', 'response_yesno', 'name', 'question', 'point', 'count_points_from',
                                                              'point_yesno_good_answer', 'point_yesno_wrong_answer', 'point_yesno_no_answer'])
        for q in qd:
            questions[q['id']] = q
            if q['type'] in ('qcu','qcm'):
                question_qcm_ids.add(q['id'])
        # read answer datas
        if question_qcm_ids:
            q_answer_pool = self.pool.get('training.exam.question.answer')
            ad_ids = q_answer_pool.search(cr, uid, [('question_id','in',list(question_qcm_ids))])
            ad = q_answer_pool.read(cr, uid, ad_ids, ['id','name','is_solution','question_id', 'point_checked', 'point_not_checked'])
            for a in ad:
                q_id = a['question_id'][0]
                questions[q_id].setdefault('response_ids', []).append(a)
                if a['is_solution'] == 'yes':
                    questions[q_id].setdefault('response_good_ids', set()).add(a['id'])

        to_write = []
        for qid in responses.keys():
            user_answers = responses[qid]
            qd = questions[qid]

            r_list = [] # (r_id, r_name, r_is_solution)
            r_user = [] # (r_id, checked?)
            r_status = False # False = Failed, True = Succeded
            r_data = {} # data to write to training.participation.line
            r_point = 0.0

            type = qd['type']
            points_from = qd['count_points_from']

            if type == 'yesno':
                good_answer_ids = set([(qd['response_yesno'] == 'no' and 1 or 0)])

                r_list += [
                    (0, 'Yes', qd['response_yesno'] == 'yes' and True or False),
                    (1, 'No', qd['response_yesno'] == 'no' and True or False),
                ]
                r_user += [(r_id, r_name, r_id in user_answers) for r_id, r_name, r_issol in r_list]
                # compute results
                r_status = (good_answer_ids == set(user_answers))
                r_data = (len(user_answers) == 1 and {'response_yesno': user_answers[0] == 0 and 'yes' or 'no' } or {})

                # compute points on yes/no question
                if points_from == 'answers':
                    if not len(user_answers):
                        r_point = qd['point_yesno_no_answer']
                    elif r_status:
                        r_point = qd['point_yesno_good_answer']
                    else:
                        r_point = qd['point_yesno_wrong_answer']
                else: # points_from == 'question'
                    r_point = r_status and qd['point'] or 0.0

            elif type in ('qcu','qcm'):
                for a in qd['response_ids']:
                    if points_from == 'answers':
                        a_detail = u'%s ([x]: %.2f, [ ]: %.2f)' % (a['name'], a['point_checked'], a['point_not_checked'])
                        if a['id'] in user_answers:
                            u_detail = u'%s (= %.2f)' % (a['name'], a['point_checked'])
                        else:
                            u_detail = u'%s (= %.2f)' % (a['name'], a['point_not_checked'])
                    else:
                        a_detail = u'%s' % (a['name'])
                        u_detail = a['name']
                    r_list.append((a['id'], a_detail, a['is_solution'] == 'yes' and True or False))
                    r_user.append((a['id'], u_detail, a['id'] in user_answers))

                good_answer_ids = qd['response_good_ids']
                r_status = (good_answer_ids == set(user_answers))
                r_data = {'response_qcm_ids': [(6, 0, user_answers)]}

                # compute points on qcu/qcm question
                if points_from == 'answers':
                    r_point = 0.0
                    for a in qd['response_ids']:
                        checked = a['id'] in user_answers
                        if checked:
                            r_point += a['point_checked']
                        else:
                            r_point += a['point_not_checked']
                else: # points_from = 'question'
                    r_point = r_status and qd['point'] or 0.0

            elif type == 'plain':
                #TODO: we don't automatically correct plain question
                r_data = {}

            z = self.dump_correction_info(qd, page_num, r_list, r_user)

            ids = user_answer_pool.search(cr, uid, [('participation_id','=',res_id),('question_id','=',qd['id'])])
            particip_line_id = ids and ids[0] or user_answer_pool.create(cr, uid, {'participation_id': res_id, 'question_id': qd['id']})
            particip_line_data = {
                'point': r_point,
                'note': z,
                'graded': True,
            }
            particip_line_data.update(r_data)
            to_write.append((particip_line_id, particip_line_data))

        # write all results
        for d in to_write:
            user_answer_pool.write(cr, uid, d[0], d[1])

        # Check if quizz have been completed
        not_answered_ids = self.pool.get('training.participation.line').search(cr, uid, [('participation_id','=',res_id),('graded','=',False)])
        if not not_answered_ids:
            # all participation line have been 'graded'
            self.write(cr, uid, [res_id], {'result_received': True})
            #particip.write({'result_received': True})

        position_storage_proxy.save_checkboxes_value(cr, uid, self._name, res_id, page_num, values, context)
        return True

training_participation()

class training_exam_correction_request_create(osv.osv_memory):
    _name = 'training.exam.correction.request.create'
    _description = 'Create a correction request from choosen participations'

    _columns = {
        'job_id': fields.many2one('res.partner.job', 'Job', required=True,),
        'product_id': fields.many2one('product.product', 'Product', required=True,),
    }

    def _get_default_job_id(self, cr, uid, context=None):
        """
        This is a *false* defaul method
        This method check if all participation are all of kind manual questionnaires, otherwise raise an error
        """
        if context is None:
            context = {}
        participation_pool = self.pool.get('training.participation')
        participation_ids = context.get('active_ids', [])
        if not participation_ids:
            return False
        for part in participation_pool.browse(cr, uid, participation_ids, context=context):
            if str(part.kind_questionnaire_id) != 'manual':
                raise osv.except_osv(_('Error'), _("One or more selected participations have a questionnaire of type other than 'manual', impossible to create a correction request"))
        return False

    _defaults = {
        'job_id': _get_default_job_id,
    }

    def create_request(self, cr, uid, ids, context=None):
        if not ids:
            return {}
        if context is None:
            context = {}
        wizard = self.browse(cr, uid, ids[0], context=context)
        proxy = self.pool.get('training.exam.correction.request')

        participation_pool = self.pool.get('training.participation')
        participation_ids = context.get('active_ids', [])
        if not participation_ids:
            # TODO: raise error to client?
            return {}

        request_value = {
            'job_id': wizard.job_id.id,
            'product_id': wizard.product_id.id,
            'forward_looking_date': time.strftime('%Y-%m-%d'),
        }
        ocv = proxy.on_change_job(cr, uid, [], request_value['job_id'], context=context)
        if ocv:
            request_value.update(ocv.get('value',{}))

        request_id = proxy.create(cr, uid, request_value, context=context)

        proxy_correction = self.pool.get('training.exam.correction')
        for obj in participation_pool.browse(cr, uid, participation_ids, context=context):
            if obj.kind != 'exam' or obj.correction_id or obj.forced_noresult:
                continue

            proxy_correction.create(cr, uid,
                                    {
                                        'request_id' : request_id,
                                        'participation_id' : obj.id,
                                    }, context=context)


        return {
            'view_type': 'form',
            "view_mode": 'form',
            'res_model': 'training.exam.correction.request',
            'view_id': self.pool.get('ir.ui.view').search(cr,uid,[('name','=','training.exam.correction.request.form')]),
            'type': 'ir.actions.act_window',
            'target': 'current',
            'res_id' : int(request_id),
        }

training_exam_correction_request_create()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
