# -*- encoding: utf-8 -*-
############################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    Copyright (C) 2008-2009 AJM Technologies S.A. (<http://www.ajm.lu). All Rights Reserved
#    Copyright (C) 2010-2011 Thamini S.à.R.L (<http://www.thamini.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################################

from report import report_sxw
from diagnostic_utils import diagnostic_report_utils
from base_rml2pdf_extra_tags.trml2pdf import customized_report_sxw

class diagnostic_report_parser(report_sxw.rml_parse, diagnostic_report_utils):
    def __init__(self, cr, uid, name, context):
        super(diagnostic_report_parser, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'range': range,
            'get_ctx': self.get_ctx,
            'get_domain_lines_info': self.get_domain_lines_info,
            'get_part_address': self.get_part_address,
            'get_address': self.get_address,
            'get_max_points': self.get_max_points,
            'get_max_domain_points': self.get_max_domain_points,
        })

    def get_ctx(self):
        return 'employer'

    def get_max_points(self, p):
        return reduce(lambda acc, a: acc + a,
                      [ l.point_question_id for l in p.participation_line_ids ], 0.0)

    def get_max_domain_points(self, p):
        return reduce(lambda acc, a: acc + a,
                     [ l.ideal_score for l in p.domain_line_ids], 0.0)

    def get_domain_lines_info(self, participation):
        _courses = dict([(x.course_id.id,x) for x in participation.course_line_ids if x.course_id])
        _domains = dict([(x.domain_id.id,x) for x in participation.domain_line_ids if x.domain_id])
        _courses_per_domain = {}

        for respline in participation.participation_line_ids:
            if not respline.question_id:
                # ignore lines without question
                continue
            resp_courses = set()
            for c in respline.question_id.course_ids:
                if c.id in _courses:
                    resp_courses.add(c.id)
            for comp in respline.question_id.competency_ids:
                for d in comp.domain_ids:
                    _courses_per_domain.setdefault(d.id, set()).update(resp_courses)

        res = []
        res_empty = {
            'domain': '',
            'score': '',
            'score_perc': '',
            'ideal_score': '',
            'max_score': '',
            'course': '',
            'course_level': '',
            'course_state': '',
            'course_recommended': False,
        }
        for dl in participation.domain_line_ids:
            r = res_empty.copy()
            r.update({
                'domain': dl.domain_id.name,
                'score': dl.score,
                'score_perc': dl.score * 100 / (dl.max_score or 1.0),
                'ideal_score': dl.ideal_score,
                'max_score': dl.max_score,
            })
            first_course = True
            for c in _courses_per_domain.get(dl.domain_id.id, set()):
                cl = _courses[c]
                if first_course:
                    rc = r.copy()
                else:
                    rc = res_empty.copy()
                rc.update({
                    'course': cl.course_id.name,
                    'course_level': cl.course_id.course_type_id.name,
                    'course_state': cl.succeeded == 'yes' and 'Pass' or 'Failed',
                    'course_recommended': cl.succeeded != 'yes' and True or False
                })
                res.append(rc)
                first_course = False

        return res

class diagnostic_report_parser_candidate(diagnostic_report_parser):
    def get_ctx(self):
        return 'candidate'

class report_sxw_part_diagreport(customized_report_sxw):
    _use_extra_flowable = ['CheckBox']

# Diagnostic Employer Report
report_sxw_part_diagreport('report.training.participation.diagnostic.employer.report',
                      'training.participation',
                      'addons/training_extra_report/report/diagnostic.rml',
                      parser=diagnostic_report_parser,
                      header=False)

report_sxw_part_diagreport('report.training.participation.diagnostic.candidate.report',
                      'training.participation',
                      'addons/training_extra_report/report/diagnostic.rml',
                      parser=diagnostic_report_parser_candidate,
                      header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
