# -*- encoding: utf-8 -*-

from report import report_sxw
from tools.translate import _
from diagnostic_utils import diagnostic_report_utils
from base_rml2pdf_extra_tags.trml2pdf import customized_report_sxw

colors = ['green', 'blue', 'red', '#dedede', '#cecece']

class domain_barchart(report_sxw.rml_parse, diagnostic_report_utils):
    def __init__(self, cr, uid, name, context):
        super(domain_barchart, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
            'get_barchartdata': self.get_barchartdata,
            'get_barchartdata_summary': self.get_barchartdata_summary,
            'get_ctx': self.get_ctx,
            'get_part_address': self.get_part_address,
            'get_address': self.get_address,
        })

    def get_barchartdata(self, p):
        context = { 'lang': self.localcontext['lang'] }
        res = {}
        res['title'] = _('Average per Domain and Level')
        res['categories'] = [ dict([('name', domline.domain_id.name),('id',domline.domain_id.id)]) for domline in p.domain_line_ids ]
        res['values'] = []
        res['datas'] = []
        res['datas_label'] = []
        res['datas_internal'] = []


        g_domains = {}
        g_domains_max = {}
        g_levels = set()
        for line in p.participation_line_ids:
            qn = line.question_id

            qn_level = qn.level_id
            g_levels.add(qn_level.id)

            qn_domains = set()
            for comp in qn.competency_ids:
                for dom in comp.domain_ids:
                    qn_domains.add(dom.id)

            for d in qn_domains:
                g_domains_max.setdefault(d, {}).setdefault(qn_level.id, []).append(qn.point)
            if -0.00001 <= line.point <= 0.00001:
                # point is null, no score to add to domain
                continue

            for d in qn_domains:
                g_domains.setdefault(d, {}).setdefault(qn_level.id, []).append(qn.point)
        level_pool = self.pool.get('training.exam.question.level')
        level_ids = []

        def get_level_records(self, g_levels):
            lvls = []
            if False in g_levels:
                g_levels.remove(False)
                class fake_level(object):
                    def __init__(self):
                        self.id = False
                        self.name = _('No level')
                lvls.append(fake_level())
            return lvls + level_pool.browse(self.cr, self.uid, list(g_levels))

        for i, l in enumerate(get_level_records(self, g_levels)):
            level_ids.append(l.id)
            res['values'].append({'name': l.name, 'color': colors[i]})
            res['datas'].append([])
            res['datas_internal'].append([])
            res['datas_label'].append([])

        for domline in p.domain_line_ids:
            d = domline.domain_id
            res['categories'].append({'name': d.name})

            dl_score = domline.score or 0.0
            dl_max = domline.max_score

            dom_qn = g_domains.get(d.id, {})
            dom_max_qn = g_domains_max.get(d.id, {})
            for i, lid in enumerate(level_ids):
                l_sum = sum(dom_qn.get(lid, []))
                l_score_ratio = l_sum * 100 / (dl_score or 1.0)
                l_max_ratio = l_sum * 100 / (dl_max or 1.0)
                l_dom_max_ratio = l_sum * 100.0 / (sum(dom_max_qn.get(lid, [])) or 1.0)
                res['datas'][i].append(l_dom_max_ratio)
                res['datas_internal'][i].append({
                    'l_sum': l_sum,
                    'l_score_ratio': l_score_ratio,
                    'l_max_ratio': l_max_ratio,
                    'l_dom_max_ratio': l_dom_max_ratio,
                })
                if l_sum > 0.0:
                    res['datas_label'][i].append('%.2f%%' % (l_dom_max_ratio))
                else:
                    res['datas_label'][i].append('') # do not display score if sum is ZERO
        return res

    def get_barchartdata_summary(self, p):
        context = { 'lang': self.localcontext['lang'] }
        res = self.get_barchartdata(p)
        res['title'] = _('Global Average per Domain')
        res['values'] = [ { 'name': 'Total', 'color': colors[0] } ]
        datas_len = len(res['datas'][0])
        datas = []
        datas.append([])
        datas_label = []
        datas_label.append([])
        for i in range(datas_len):
            isum = sum([ k[i]['l_max_ratio'] for k in res['datas_internal'] ])
            datas[0].append(isum)
            datas_label[0].append('%.2f%%' % (isum))
        res['datas'] = datas
        res['datas_label'] = datas_label
        return res

class report_sxw_barchart(customized_report_sxw):
    _use_extra_flowables = ['barchart']

report_sxw_barchart('report.training.participation.domain.barchart',
                      'training.participation',
                      'addons/training_extra_report/report/domain_barchart.rml',
                      parser=domain_barchart)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

