# -*- encoding: utf-8 -*-
############################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    Copyright (C) 2008-2009 AJM Technologies S.A. (<http://www.ajm.lu). All Rights Reserved
#    Copyright (C) 2010-2011 Thamini S.à.R.L (<http://www.thamini.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################################

import math
from report import report_sxw
from tools.translate import _
from diagnostic_utils import diagnostic_report_utils
from base_rml2pdf_extra_tags.trml2pdf import customized_report_sxw

class training_participation_false_responses_parser(report_sxw.rml_parse, diagnostic_report_utils):
    def __init__(self, cr, uid, name, context):
        super(training_participation_false_responses_parser, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'get_false_responses_and_domain': self.get_false_responses_and_domain,
            'solution':self.solution,
            'get_image': self.get_image,
            'get_question_type_tips': self.get_question_type_tips,
            'get_question_point': self.get_question_point,
            'get_plain_lines': self.get_plain_lines,
            'get_ctx': self.get_ctx,
            'get_address': self.get_address,
        })

    def get_ctx(self):
        return 'candidate'

    def get_false_responses_and_domain(self, participation):
        _domains = dict([(x.domain_id.id,x) for x in participation.domain_line_ids if x.domain_id])
        _respline = dict([(x.id,x) for x in participation.participation_line_ids])
        _respline_per_domain = {}
        _respline_no_domain = set()

        for respline in participation.participation_line_ids:
            if not respline.question_id:
                # ignore lines without question
                continue

            if respline.point < respline.point_question_id:
                # question response is False
                respline_domains = set()
                for comp in respline.question_id.competency_ids:
                    for d in comp.domain_ids:
                        respline_domains.add(d.id)
                if len(respline_domains) == 0:
                    _respline_no_domain.add(respline.id)
                for d in respline_domains:
                    _respline_per_domain.setdefault(d, set()).add(respline.id)

        res = []
        for d in participation.domain_line_ids:
            _domain = d.domain_id
            if not _domain:
                continue
            if _domain.id not in _respline_per_domain:
                continue
            r = {
                'domain': _domain.name,
                'resplines': [ _respline[x] for x in _respline_per_domain[_domain.id] ],
            }
            res.append(r)
        if len(_respline_no_domain):
            # display response lineis without any domain at end of report
            context = {'lang': self.localcontext['lang']}
            res.append({
                'domain': _('Without any domain'),
                'resplines': [ _respline[x] for x in _respline_no_domain ],
            })
        return res

    def solution(self, question, question_type, respline):
        _lt = self.localcontext['translate']
        res = []
        if question_type not in['plain','yesno']:
            resp_ids = [ x.id for x in respline.response_qcm_ids ]

            self.cr.execute('select name,id,is_solution from training_exam_question_answer where question_id = %s',(question.id,))
            res = self.cr.dictfetchall()
            for l in res:
                l['is_user_response'] = l['id'] in resp_ids and 'yes' or 'no'
            return res
        elif question_type =='yesno':
            return  [
                {'name': _lt('Yes'),
                 'id': 0,
                 'is_solution': question.response_yesno == 'yes' and 'yes' or 'no',
                 'is_user_response': respline.response_yesno == 'yes' and 'yes' or 'no',
                },
                {'name': _lt('No'),
                 'id': 1,
                 'is_solution': question.response_yesno == 'no' and 'yes' or 'no',
                 'is_user_response': respline.response_yesno == 'no' and 'yes' or 'no',
                }
            ]
        else:
            # Plain responses are currently not implemented for this report !!!!
            raise NotImplementedError()

    def get_image(self, question, position):
        if position != question.question_id.image_position:
            return ''
        print(type(question.question_id.image))
        image_data = question.question_id.image
        if image_data:
            return image_data
        return ''

    def get_question_type_tips(self, question_id):
        _lt = self.localcontext['translate']
        type = question_id.type
        if type in ('qcm'):
            return _lt('multiple responses')
        if type in ('qcu'):
            return _lt('one response')
        return ''

    def get_question_point(self, question_id):
        _lt = self.localcontext['translate']
        points = question_id.point
        float_p, int_p = math.modf(points)
        if float_p >= -0.00001 and float_p <= 0.00001:
            str_points = '%.0f' % (points)
        else:
            str_points = '%.2f' % (points)

        if points > 1:
            return _lt(r'%s points') % (str_points)
        else:
            return _lt(r'%s point') % (str_points)

    def get_plain_lines(self, question_id):
        lines = []
        max_chars = 143
        if not question_id.response_plain:
            lines.append('')
            return lines

        for line in question_id.response_plain.split('\n'):
            while len(line) > max_chars:
                for i, c in enumerate(reversed(line[:max_chars])):
                    if c.isspace():
                        lines.append(line[:max_chars-i])
                        line = line[max_chars-i:]
                        break
            lines.append(line)
        return lines

class report_sxw_examfalseresp(customized_report_sxw):
    _use_extra_flowables = ['CheckBox', 'imageinline']

# False Response Report
report_sxw_examfalseresp('report.training.participation.false.result.report',
                      'training.participation',
                      'addons/training_extra_report/report/exam_false_responses.rml',
                      parser=training_participation_false_responses_parser,
                      header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

