# -*- encoding: utf-8 -*-
############################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    Copyright (C) 2008-2009 AJM Technologies S.A. (<http://www.ajm.lu). All Rights Reserved
#    Copyright (C) 2010-2011 Thamini S.à.R.L (<http://www.thamini.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################################

import time
import datetime
import math
import base64
from lxml import etree
from report import report_sxw
import pooler
import netsvc
from osv import osv, fields
from tools.translate import _
from base_rml2pdf_extra_tags.trml2pdf import customized_report_sxw

try:
    import cStringIO as StringIO
except:
    import StringIO as StringIO

class fake_obj(object):
    def __init__(self, d):
        self.d = {}
        self._context = d.pop('_context', {})
        self._cache = d.pop('_cache', {})
        for k,v  in d.iteritems():
            if isinstance(v, dict):
                v.setdefault('_context', self._context)
                v.setdefault('_cache', self._cache)
                o = fake_obj(v)
                self.d[k] = o
            else:
                self.d[k] = v

    def __getattribute__(self, name):
        if name in ('d', '_context'):
            return super(fake_obj, self).__getattribute__(name)
        try:
            return self.d[name]
        except KeyError:
            return super(fake_obj, self).__getattribute__(name)

    def name_get(self):
        return [ (self.d.get('id',0), self.d.get('name','')) ]

    def exists(self):
        return True

    def __contains__(self, name):
        return name in self.d

class report_sxw_exam_sheet(customized_report_sxw):
    _use_extra_flowables = ['CheckBox', 'imageinline', 'para-nowrap', 'qrBarCode']

class training_participation_exam_sheet(report_sxw.rml_parse):
    REPORT_CONTEXT_OPTIONS = [
        # OPTION, TYPE, DEFAULT_VALUE
        ('wizard_print_raw', bool, False), # needed for 'Print RAW Questionnaire' only!
        ('force_course_id', bool, False),
        ('show_solution', bool, False),
        ('show_page_header', bool, False),
        ('show_participation_firstpage', bool, False),
        ('show_summary_table', bool, False),
        ('show_response_note', bool, False),
        ('show_extended_question_infos', bool, False),
    ]
    REPORT_CONTEXT_FORCE = {} # force some context before filtering it
    REPORT_CONTEXT_FILTERS = [] # if not empty, remove every REPORT_CONTEXT_OPTIONS not inside this list from context
    REPORT_EXTRA_INFOS_FILTERS = [] # if not empty, keep only thoses extra infos types

    def __init__(self, cr, uid, name, context=None):
        if context is None:
            context = {}

        for k, v in self.REPORT_CONTEXT_FORCE.iteritems():
            context[k] = v

        context_filters = self.REPORT_CONTEXT_FILTERS
        for option, type_, default_value in self.REPORT_CONTEXT_OPTIONS:
            if option in context and option in context_filters:
                context[option] = type_(context[option]) # enforce type of current value
            else:
                context[option] = default_value

        self.force_course_id = context.get('force_course_id',False)
        super(training_participation_exam_sheet, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
            'solution':self.solution,
            'setLang': self.setLang,
            'get_image': self.get_image,
            'get_question_type_tips': self.get_question_type_tips,
            'get_question_point': self.get_question_point,
            'checkboxes_position':[], # needed in create_single_pdf
            'checkboxes_context':[], # needed in create_single_pdf
            'preformat': self.preformat,
            'get_plain_lines': self.get_plain_lines,
            'get_partner_name': self.get_partner_name,
            'get_contact_name': self.get_contact_name,
            'get_questionnaire': self.get_questionnaire,
            'get_participation_id': self.get_participation_id,
            'get_participation_date': self.get_participation_date,
            'get_question_corrector_note': self.get_question_corrector_note,
            'get_extended_question_infos': self.get_extended_question_infos,
            'get_courses': self.get_courses,
            'is_exam_kind_manual': self.is_exam_kind_manual,
        })
        self.qcache = {}
        self.fake_obj_cache = {}

    def preformat(self, string):
        lines = []
        if string:
            for l in string.split('\n'):
                new_l = l.lstrip()
                full_l = u'\xa0' * (len(l) - len(new_l)) + new_l
                lines.append(full_l)
        return '\n'.join(lines)

    def is_exam_kind_manual(self, questionnaire):
        return questionnaire.kind == 'manual' and True or False

    def get_partner_name(self, participation):
        return participation.partner_id.name

    def get_contact_name(self, participation):
        return participation.job_id.contact_id.name_get(context=self.localcontext)[0][1]

    def get_participation_date(self, participation):
        return participation.date

    def get_participation_id(self, participation):
        return participation.id

    def get_questionnaire(self, participation):
        return participation.questionnaire_id

    def get_question_corrector_note(self, participation, question):
        tpline = self.pool.get('training.participation.line')
        part_id = self.get_participation_id(participation)
        part_line_ids = tpline.search(self.cr, self.uid, [('participation_id','=',part_id),('question_id','=',question.id)])
        if part_line_ids and len(part_line_ids) == 1: # one and only one part line
            part_line = tpline.read(self.cr, self.uid, part_line_ids[0], ['note'])
            if part_line:
                return part_line['note']
        return None

    def get_extended_question_infos(self, question):
        qn_pool = question.pool.get('training.exam.question')
        context = { 'lang': self.localcontext['lang'] }
        qn_elems = qn_pool._get_extended_question_infos_elements(self.cr, self.uid, question, context=context, report_context=self.localcontext)
        if self.REPORT_EXTRA_INFOS_FILTERS:
            qn_elems_keys = self.REPORT_EXTRA_INFOS_FILTERS
        else:
            qn_elems_keys = [ x[0] for x in qn_elems ]
        qn_elems = dict(qn_elems)

        extra_infos_text = ', '.join([ qn_elems.get(f, '') for f in qn_elems_keys if qn_elems.get(f,'') ])
        return ' %s' % (extra_infos_text)

    def get_courses(self, questionnaire):
        if self.force_course_id:
            courses = [ questionnaire.main_course_id ]
        else:
            # build courses list based on questionnaire questions
            course_ids = set()
            for i, qzn in enumerate(questionnaire.question_ids):
                for c in qzn.question_id.course_ids:
                    course_ids.add(c.id)
            courses = self.pool.get('training.course').browse(self.cr, self.uid, list(course_ids))
        return courses

    def solution(self, question, question_type, participation=None):
        _lt = self.localcontext['translate']
        res = []
        if self.localcontext.get('show_response_note',False) and not participation:
            return []
        if self.localcontext.get('show_response_note',False):
            # we are displaying was the user checked
            part_line_pool = self.pool.get('training.participation.line')
            part_line_ids = part_line_pool.search(self.cr, self.uid, [('participation_id','=',participation.id),('question_id','=',question.id)])
            part_line = part_line_pool.browse(self.cr, self.uid, part_line_ids[0])

        if question_type not in['plain','yesno']:
            self.cr.execute('select name,id,is_solution from training_exam_question_answer where question_id = %s ORDER BY id',(question.id,))
            res = self.cr.dictfetchall()
            if self.localcontext.get('show_response_note',False):
                part_mcq_answers = [ x.id for x in part_line.response_qcm_ids ]
                for row in res:
                    row['is_solution'] = (row['id'] in part_mcq_answers) and 'yes' or 'no'
            return res
        elif question_type =='yesno':
            resp_yesno = question.response_yesno
            if self.localcontext.get('show_response_note',False):
                resp_yesno = part_line.response_yesno
            return  [{'name': _lt('Yes'), 'id': 0, 'is_solution': resp_yesno == 'yes' and 'yes' or 'no'},
                     {'name': _lt('No'), 'id': 1, 'is_solution': resp_yesno == 'no' and 'yes' or 'no'}]
        else: # plain
            return [ dict([('name', x+1),('id',-1)]) for x in range(question.free_lines_count or 10) ]

    def get_image(self, question, position):
        if position != question.question_id.image_position:
            return ''
        image_data = question.question_id.image
        if image_data:
            return image_data
        return ''

    def get_question_type_tips(self, question_id):
        _lt = self.localcontext['translate']
        type = question_id.type
        if type in ('qcm'):
            return _lt('multiple responses')
        if type in ('qcu'):
            return _lt('one response')
        return ''

    def get_question_point(self, question_id):
        _lt = self.localcontext['translate']
        points = question_id.point
        float_p, int_p = math.modf(points)
        if float_p >= -0.00001 and float_p <= 0.00001:
            str_points = '%.0f' % (points)
        else:
            str_points = '%.2f' % (points)

        if points > 1:
            return _lt(r'%s points') % (str_points)
        else:
            return _lt(r'%s point') % (str_points)

    def get_plain_lines(self, question_id, participation=None):
        lines = []
        max_chars = 143

        if self.localcontext.get('show_response_note',False) and not participation:
            return '' # TODO: Make this more

        response_plain = question_id.response_plain

        if self.localcontext.get('show_response_note',False):
            # we are displaying was the user checked
            part_line_pool = self.pool.get('training.participation.line')
            part_line_ids = part_line_pool.search(self.cr, self.uid, [('participation_id','=',participation.id),('question_id','=',question_id.id)])
            part_line = part_line_pool.browse(self.cr, self.uid, part_line_ids[0])
            response_plain = part_line.response_plain

        if not response_plain:
            lines.append('')
            return lines

        for line in response_plain.split('\n'):
            while len(line) > max_chars:
                for i, c in enumerate(reversed(line[:max_chars])):
                    if c.isspace():
                        lines.append(line[:max_chars-i])
                        line = line[max_chars-i:]
                        break
            lines.append(line)
        return lines

    def setLang(self, lang):
        """overload report_sxw.setLang to fix obj._context AttributeError if obj is type int"""
        if not lang or self.default_lang.has_key(lang):
            if not lang:
                key = 'en_US'
        self.localcontext['lang'] = lang
        self.lang_dict_called = False
        for obj in self.objects:
            if isinstance(obj, (int,long)):
                continue
            obj._context['lang'] = lang
            for table in obj._cache:
                for id in obj._cache[table]:
                    self._lang_cache.setdefault(obj._context['lang'], {}).setdefault(table,
                            {}).update(obj._cache[table][id])
                    if lang in self._lang_cache \
                            and table in self._lang_cache[lang] \
                            and id in self._lang_cache[lang][table]:
                        obj._cache[table][id] = self._lang_cache[lang][table][id]
                    else:
                        obj._cache[table][id] = {'id': id}

class report_attachment(object):
    def create_with_attachment(self, cr, uid, ids, data, original_doc, context=None):
        import base64
        from report.pyPdf import PdfFileReader, PdfFileWriter
        o = PdfFileWriter()

        pool = pooler.get_pool(cr.dbname)
        quizz_id = ids[0]

        if data.get('model', '') == 'training.participation':
            participation = pool.get('training.participation').browse(cr, uid, data['id'])
            quizz_id = participation.questionnaire_id.id
            if isinstance(quizz_id, bool) and quizz_id == False:
                raise Exception(_('Could not generate exam sheet report, no questionnaire is defined on the participation'))

        quizz_proxy = pool.get('training.exam.questionnaire')
        quizz = quizz_proxy.browse(cr, uid, quizz_id)

        def write_orig_document():
            r = PdfFileReader(StringIO.StringIO(original_doc[0]))
            for p in range(r.getNumPages()):
                o.addPage(r.getPage(p))

        def generate_watermark_pdf(docid, basenum, npages):
            watermark_report = netsvc.LocalService('report.training.participation.watermark.qrcode')
            report_ids = [ '%s-%s%03d' % (docid, basenum, n+1) for n in range(npages) ]
            (result, format) = watermark_report.create(cr, uid, report_ids, {}, context=context)
            return PdfFileReader(StringIO.StringIO(result))

        attach_count = 0
        orig_doc_written = False
        for attach in quizz.attachment_ids:
            if attach.position == 'after' and not orig_doc_written:
                write_orig_document()
                orig_doc_written = True

            # 1 = annex and store
            # 2 = detect qrcode but don't store in the DMS
            basenum = '%1s%02d' % (attach.store_in_dms and 1 or 2, attach_count)

            attach_pdf_data = base64.decodestring(attach.attachment_id.datas)
            attach_pdf = PdfFileReader(StringIO.StringIO(attach_pdf_data))
            waterm_pdf = generate_watermark_pdf(ids[0], basenum, attach_pdf.getNumPages())

            for attach_page in range(attach_pdf.getNumPages()):
                docp = attach_pdf.getPage(attach_page)
                docp_wm = waterm_pdf.getPage(attach_page)
                docp.mergePage(docp_wm)
                o.addPage(docp)

            attach_count += 1
        if not orig_doc_written:
            write_orig_document()

        s = StringIO.StringIO()
        o.write(s)
        return (s.getvalue(), original_doc[1])

class report_and_store_checkboxes_position(report_sxw_exam_sheet, report_attachment):
    def __init__(self, *args, **kwargs):
        super(report_and_store_checkboxes_position, self).__init__(*args, **kwargs)
        # FIXME cr is not defined here
        # self.training_scan_result = pooler.get_pool(cr.dbname).get('training.scan.result')

    def create(self, cr, uid, ids, data, context=None):
        return super(report_and_store_checkboxes_position, self).create(cr, uid, ids, data, context=context)

    def create_single_pdf(self, cr, uid, ids, data, report_xml, context=None):
        if not context:
            context={}
        logo = None
        context = context.copy()
        title = report_xml.name
        rml = report_xml.report_rml_content
        # if no rml file is found
        if not rml:
            return False

        # TODO regroup in context['report_misc_data']
        pool = pooler.get_pool(cr.dbname)

        # before generating document, dynamically assign questionnaire
        t_part_proxy = pool.get('training.participation')
        t_part_proxy.random_questionnaire_assign(cr, uid, ids)

        rml_parser = self.parser(cr, uid, self.name2, context=context)
        objs = self.getObjects(cr, uid, ids, context)
        rml_parser.set_context(objs, data, ids, report_xml.report_type)
        rml_context = rml_parser.localcontext
        rml_context.update({
            'checkboxes_position': [], # add a reference in context since it can't be modified during next call
            'checkboxes_context': [], # add a reference in context since it can't be modified during next call
            'participation_id': [], # add a reference in context since it can't be modified during next call
        })

        processed_rml = etree.XML(rml)
        if report_xml.header:
            rml_parser._add_header(processed_rml, self.header)
        processed_rml = self.preprocess_rml(processed_rml,report_xml.report_type)
        if rml_parser.logo:
            logo = base64.decodestring(rml_parser.logo)
        create_doc = self.generators[report_xml.report_type]
        pdf, report_type = create_doc(etree.tostring(processed_rml),rml_parser.localcontext,logo,title.encode('utf8')), report_xml.report_type

        participation_id = int(rml_context['participation_id'][0])
        training_scan_result = pool.get('position.storage')
        training_scan_result.save_checkboxes_position(cr, uid, 'training.participation', participation_id, rml_context['checkboxes_position'], rml_context['checkboxes_context'])

        # do other things afterward
        t_part_proxy.after_exam_sheet_generation(cr, uid, participation_id, context=rml_context)

        return self.create_with_attachment(cr, uid, ids, data, (pdf, report_type), context=context)

class report_preview(report_sxw_exam_sheet, report_attachment):
    def create(self, cr, uid, ids, data, context=None):
        z = super(report_preview, self).create(cr, uid, ids, data, context=context)
        return self.create_with_attachment(cr, uid, ids, data, z, context=context)
#
# Preview Original Exam Sheet
#
class training_participation_exam_sheet_preview_parser(training_participation_exam_sheet):
    REPORT_CONTEXT_FILTERS = ['show_page_header', 'show_participation_firstpage', 'preview']
    REPORT_CONTEXT_FORCE = {
        'show_page_header': True,
        'show_participation_firstpage': True,
        'preview': False,
    }

class report_preview_questionnaire(report_preview):
    def getObjects(self, cr, uid, ids, context):
        quizzes = super(report_preview_questionnaire, self).getObjects(cr, uid, ids, context)
        if not quizzes:
            return []
        res = []
        for qz in quizzes:
            lang = 'en_US'
            if qz.main_course_id:
                lang = qz.main_course_id.lang_id.code
            context = {'lang': lang} # make GettextAlias local context aware
            p = fake_obj({
                '_context': context,
                'id': qz.id,
                'date': time.strftime('%Y-%m-%d %H:%M:%S'),
                'partner_id': {'name': _('DEMO PARTNER')},
                'job_id': {
                    'contact_id': {'name': _('John Doe')},
                },
                'questionnaire_id': qz,
            })
            res.append(p)
        return res

report_preview_questionnaire('report.training.participation.report.preview',
               'training.exam.questionnaire',
               'addons/training_extra_report/report/exam_sheet.rml',
               parser=training_participation_exam_sheet_preview_parser,
               header=False)

# Original Exam Sheet w. Corrector Note
class training_participation_exam_sheet_w_notes(training_participation_exam_sheet):
    REPORT_CONTEXT_FILTERS = ['show_page_header', 'show_participation_firstpage', 'show_solution', 'show_response_note', 'show_extended_question_infos' ]
    REPORT_CONTEXT_FORCE = {
        'show_page_header': True,
        'show_participation_firstpage': True,
        'show_solution': True,
        'show_response_note': True,
        'show_extended_question_infos': True,
    }
    REPORT_EXTRA_INFOS_FILTERS = [ 'type', 'level', 'domains' ]

report_preview('report.training.participation.report.w.notes',
                          'training.participation',
                          'addons/training_extra_report/report/exam_sheet.rml',
                          parser=training_participation_exam_sheet_w_notes,
                          header=False)

# Exam Sheet Report
class training_participation_exam_sheet_parser(training_participation_exam_sheet):
    REPORT_CONTEXT_FILTERS = ['show_page_header', 'show_participation_firstpage']
    REPORT_CONTEXT_FORCE = {
        'show_page_header': True,
        'show_participation_firstpage': True,
    }

report_and_store_checkboxes_position('report.training.participation.report',
                                     'training.participation',
                                     'addons/training_extra_report/report/exam_sheet.rml',
                                     parser=training_participation_exam_sheet_parser,
                                     header=False)
#
# Preview Exam Sheet
# 1. with solutions (default)
# 2. without solutions
#
class training_exam_questionnaire_parser(training_participation_exam_sheet):
    REPORT_CONTEXT_FILTERS = [ 'show_solution', 'show_summary_table', 'show_extended_question_infos', 'wizard_print_raw' ]
    REPORT_CONTEXT_FORCE = {
        'show_solution': True,
        'show_summary_table': True,
        'show_extended_question_infos': True,
    }

class training_exam_questionnaire_wo_solution_parser(training_participation_exam_sheet):
    REPORT_CONTEXT_FILTERS = [ 'show_summary_table', 'show_extended_question_infos', 'wizard_print_raw']
    REPORT_CONTEXT_FORCE = {
        'show_summary_table': True,
        'show_extended_question_infos': True,
    }

class report_questionnaire(report_sxw_exam_sheet):
    def getObjects(self, cr, uid, ids, context):
        if context is None:
            context = {}
        if context.get('wizard_print_raw',False) and context.get('active_ids',[]):
            # we are printing raw questions: create a fake participation + fake questionnaire
            pool = pooler.get_pool(cr.dbname)
            qn_pool = pool.get('training.exam.question')
            qn_ids = context.get('active_ids',[])
            wizard = pool.get('training.exam.question.print.raw.wizard').browse(cr, uid, ids[0], context)
            context = context.copy()
            context['lang'] = wizard.lang_id.code
            raw_qz = fake_obj({
                'questionnaire_id': {
                    'main_course_id': {
                        'name': _('RAW Questionnaire'),
                        'lang_id': {'code': context['lang']},
                    },
                    'question_ids': [ fake_obj({'question_id': x}) for x in qn_pool.browse(cr, uid, qn_ids, context=context) ],
                },
            })
            return [ raw_qz ]
        else:
            # we are printing standard questionnaires
            quizzes = super(report_questionnaire, self).getObjects(cr, uid, ids, context)
            return [ fake_obj({'questionnaire_id': qz}) for qz in quizzes ]

report_questionnaire('report.training.exam.questionnaire.report',
                      'training.exam.questionnaire',
                      'addons/training_extra_report/report/exam_sheet.rml',
                      parser=training_exam_questionnaire_parser,
                      header=False)

report_questionnaire('report.training.exam.questionnaire.wo.solution.report',
                     'training.exam.questionnaire',
                     'addons/training_extra_report/report/exam_sheet.rml',
                     parser=training_exam_questionnaire_wo_solution_parser,
                     header=False)
#
# Questions Folder for Seance
#
class training_seance_questions_folder_parser(training_participation_exam_sheet):
    REPORT_CONTEXT_FILTERS = [ 'show_solution', 'show_summary_table', 'force_course_id', 'show_extended_question_infos' ]
    REPORT_CONTEXT_FORCE = {
        'show_solution': True,
        'show_summary_table': True,
        'force_course_id': True,
        'show_extended_question_infos': True,
    }

class report_seance_questions_folder(report_sxw_exam_sheet):
    """ generate the question folder report on all valid questions related
        to the seance's course
    """
    def getObjects(self, cr, uid, ids, context):
        seances = super(report_seance_questions_folder, self).getObjects(cr, uid, ids, context)
        if not seances:
            return []

        pool = pooler.get_pool(cr.dbname)
        seance = seances[0]
        if not seance.course_id:
            return []
        course = seance.course_id
        qn_pool = pool.get('training.exam.question')
        qn_ids = qn_pool.search(cr, uid, [('state','=','validated'),('course_ids','in',[course.id])])
        qn_records = qn_pool.browse(cr, uid, qn_ids, context=context)

        # create a unique fake participation
        p = fake_obj({
            'questionnaire_id': fake_obj({
                'main_course_id': course,
                'question_ids': [ fake_obj({'question_id': x}) for x in qn_records ],
            }),
        })
        return [ p ]
report_seance_questions_folder('report.training.seance.questions.folder.report',
                              'training.seance',
                              'addons/training_extra_report/report/exam_sheet.rml',
                              parser=training_seance_questions_folder_parser,
                              header=False)

class report_participation_watermark_parser(report_sxw.rml_parse):

    def set_context(self, objects, data, ids, report_type = None):
        # call parents without objects as we need to handle it locally for watermarks
        super(report_participation_watermark_parser, self).set_context([], data, ids, report_type=report_type)
        self.localcontext['objects'] = objects
        self.objects = objects

class report_participation_watermark_qrcode(customized_report_sxw):
    _use_extra_flowables = ['qrBarCode']

    def getObjects(self, cr, uid, ids, context):
        return ids

report_participation_watermark_qrcode('report.training.participation.watermark.qrcode',
                                      'training.participation',
                                      'addons/training_extra_report/report/watermark_qrcode.rml',
                                      parser=report_participation_watermark_parser,
                                      header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

