# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from datetime import datetime
from report import report_sxw
from osv.orm import browse_record
import netsvc
import pooler
import cStringIO
import base64
from setuptools import find_packages
from base_rml2pdf_extra_tags.trml2pdf import customized_report_sxw

class fake_obj(object):
    def __init__(self, d):
        self.d = {}
        for k,v  in d.iteritems():
            if isinstance(v, dict):
                o = fake_obj(v)
                self.d[k] = o
            else:
                self.d[k] = v

    def __getattr__(self, name):
        if name == 'd':
            return super(fake_obj, self).__getattr__(name)
        try:
            return self.d[name]
        except KeyError:
            return super(fake_obj, self).__getattr__(name)

class parser_global_report(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context=None):
        super(parser_global_report, self).__init__(cr, uid, name, context=context)

        self.localcontext.update({
            'get_plain_results' : self._get_plain_results,
            'get_question_by_main_category': self.get_question_by_main_category,
            'get_eval': self.get_eval,
            'xrange': self.xrange,
        })

    def _get_plain_results(self, global_evaluation):
        values = {}
        for evaluation in global_evaluation.evaluation_ids:
            for line in evaluation.line_ids:
                if line.question_id.type == 'plain':
                    values.setdefault(line.question_id, []).append(line.response_plain)

        res = [(k, "\n".join(v)) for k, v in values.items()]
        return res

    def get_question_by_main_category(self, questions):
        # group question by main category
        main_categories_list = []
        main_categories_dict = {}

        def get_main_category(category):
            if category.parent_id:
                return get_main_category(category.parent_id)
            return category

        for q in questions:
            q_main_categ = get_main_category(q.category_id)

            if q_main_categ not in main_categories_list:
                main_categories_list.append(q_main_categ)

            main_categories_dict.setdefault(q_main_categ, []).append(q)

        return [ (x, main_categories_dict[x]) for x in main_categories_list ]

    def get_eval(self, objects):
        objs = []
        for obj in objects:
            if isinstance(obj, browse_record) \
                and obj._name == 'training.evaluation.questionnaire.preview.wizard':
                # create fake object to simulate training.evaluation.global
                new_obj = fake_obj({
                    'id': obj.eval_quizz_id.id,
                    'name': obj.eval_quizz_id.name,
                    'seance_id': {
                        'name': self.localcontext['translate']('TEST SEANCE'),
                        'date': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                        'contact_names': self.localcontext['translate']('LECTURER 1'),
                        'course_id': {
                            'name': self.localcontext['translate']('TEST COURSE'),
                            'lang_id': {
                                'code': obj.lang_id.code,
                            },
                        },
                    },
                    'questionnaire_id': obj.eval_quizz_id,
                })
                objs.append(new_obj)
                continue
            # default case, simply report the same value
            objs.append(obj)
        return objs

    def xrange(self, number):
        return range(number)

class report_and_store_checkboxes_position(report_sxw.report_sxw):
    def __init__(self, *args, **kwargs):
        super(report_and_store_checkboxes_position, self).__init__(*args, **kwargs)
        # FIXME cr is not defined here
        # self.training_scan_result = pooler.get_pool(cr.dbname).get('training.scan.result')

    def create_source_pdf(self, cr, uid, ids, data, report_xml, context=None):
        if context is None:
            context={}
        pool = pooler.get_pool(cr.dbname)
        attach = report_xml.attachment
        if attach:
            objs = self.getObjects(cr, uid, ids, context)
            results = []
            for obj in objs:
                aname = eval(attach, {'object':obj, 'time':time})
                result = False
                if report_xml.attachment_use and aname and context.get('attachment_use', True) and 'preview' not in context:
                    aids = pool.get('ir.attachment').search(cr, uid, [('datas_fname','=',aname+'.pdf'),('res_model','=',self.table),('res_id','=',obj.id)])
                    if aids:
                        brow_rec = pool.get('ir.attachment').browse(cr, uid, aids[0], context=context)
                        if not brow_rec.datas:
                            continue
                        d = base64.decodestring(brow_rec.datas)
                        results.append((d,'pdf'))
                        continue
                result = self.create_single_pdf(cr, uid, [obj.id], data, report_xml, context)
                if not result:
                    return False
                try:
                    if aname:
                        name = aname+'.'+result[1]
                        pool.get('ir.attachment').create(cr, uid, {
                            'name': aname,
                            'datas': base64.encodestring(result[0]),
                            'datas_fname': name,
                            'res_model': self.table,
                            'res_id': obj.id,
                            }, context=context
                        )
                        cr.commit()
                except Exception,e:
                     import traceback, sys
                     tb_s = reduce(lambda x, y: x+y, traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback))
                     netsvc.Logger().notifyChannel('report', netsvc.LOG_ERROR,str(e))
                results.append(result)
            if results:
                if results[0][1]=='pdf':
                    install_pack = []
                    for package in find_packages():
                        install_pack.append(package)
                    if 'report.pyPdf' not in install_pack:
                        netsvc.Logger().notifyChannel('report', netsvc.LOG_ERROR, "pyPdf package must be install")
                    from report.pyPdf import PdfFileWriter, PdfFileReader
                    output = PdfFileWriter()
                    for r in results:
                        reader = PdfFileReader(cStringIO.StringIO(r[0]))
                        for page in range(reader.getNumPages()):
                            output.addPage(reader.getPage(page))
                    s = cStringIO.StringIO()
                    output.write(s)
                    return s.getvalue(), results[0][1]
        return self.create_single_pdf(cr, uid, ids, data, report_xml, context)

    def create_single_pdf(self, cr, uid, ids, data, report_xml, context=None):
        if context is None:
            context = {}
        # TODO regroup in context['report_misc_data']
        pool = pooler.get_pool(cr.dbname)
        context['checkboxes_position'] = [] # add a reference in context since it can't be modified during next call
        context['checkboxes_context'] = [] # add a reference in context since it can't be modified during next call
        context['participation_id'] = [] # add a reference in context since it can't be modified during next call
        global_evaluation_id = False
        pdf, report_type = super(report_and_store_checkboxes_position, self).create_single_pdf(cr, uid, ids, data, report_xml, context)
        if 'preview' not in context:
            # only store checkboxes position for real print
            if context['participation_id']:
                global_evaluation_id = int(context['participation_id'][0])
            training_scan_result = pool.get('position.storage')
            training_scan_result.save_checkboxes_position(cr, uid, 'training.evaluation.global', global_evaluation_id, context['checkboxes_position'], context['checkboxes_context'])

        return pdf, report_type


class report_evaluation(customized_report_sxw, report_and_store_checkboxes_position):
    _use_extra_flowable = ['qrBarCode', 'CheckBox']
    def create(self, cr, uid, ids, data, context=None):
        return super(report_evaluation, self).create(cr, uid, ids, data, context=context)

    def getObjects(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        if 'preview' in context:
            pool = pooler.get_pool(cr.dbname)
            return pool.get('training.evaluation.questionnaire.preview.wizard').browse(cr, uid, ids, context=context)
        return super(report_evaluation, self).getObjects(cr, uid, ids, context=context)

report_evaluation('report.training.evaluation.empty.report',
                      'training.evaluation.global',
                      'addons/training_extra_report/report/empty_evaluation.rml',
                      parser=parser_global_report,
                      header=False)

report_evaluation('report.training.evaluation.empty.preview.report',
                      'training.evaluation.questionnaire',
                      'addons/training_extra_report/report/empty_evaluation.rml',
                      parser=parser_global_report,
                      header=False)
