import pooler
from report import report_sxw
from tools.translate import _
import tools

class session_volume(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(session_volume, self).__init__(cr, uid, name, context)
        self.localcontext.update( {
            'get_sessions' : self._get_sessions,
            'get_sessions_count' : self._get_sessions_count,
            'get_sessions_count_all' : self._get_sessions_count_all,
            'state2str' : self._state2str,

            'get_subscriptions_count' : self._get_subscriptions_count,
            'get_subscriptions' : self._get_subscriptions,
            'get_subscriptions_state' : self._get_subscriptions_state,
            'get_subscriptions_count_all' : self._get_subscriptions_count_all,

            'get_offers' : self._get_offers,

        })

    def _get_subscriptions_state(self):
        return ['draft', 'confirmed', 'cancelled', 'done']

    def _get_subscriptions_search(self, state, date_start, date_stop):
        proxy = pooler.get_pool(self.cr.dbname).get('training.subscription.line')
        return proxy.search(self.cr, self.uid, [('session_id.date', '>=', date_start), ('session_id.date', '<=', date_stop), ('state', '=', state)])

    def _get_subscriptions(self, state, date_start, date_stop):
        proxy = pooler.get_pool(self.cr.dbname).get('training.subscription.line')
        return proxy.browse(self.cr, self.uid, self._get_subscriptions_search(state, date_start, date_stop))

    def _get_subscriptions_count(self, state, date_start, date_stop):
        return len(self._get_subscriptions_search(state, date_start, date_stop))

    def _get_subscriptions_count_all(self, date_start, date_stop):
        proxy = pooler.get_pool(self.cr.dbname).get('training.subscription.line')
        return  proxy.search_count(self.cr, self.uid, [('session_id.date', '>=', date_start), ('session_id.date', '<=', date_stop)])

    def _get_offers(self, date_start, date_stop):
        pool = pooler.get_pool(self.cr.dbname)
        proxy_sl = pool.get('training.subscription.line')

        sl_ids = proxy_sl.search(self.cr, self.uid, [('session_id.date', '>=', date_start), ('session_id.date', '<=', date_stop)])

        offer_ids = set([ val['offer_id'][0] for val in proxy_sl.read(self.cr, self.uid, sl_ids, ['offer_id']) ])
        proxy_offer = pool.get('training.offer')
        values = []
        for offer_id in offer_ids:
            sl_count = proxy_sl.search_count(self.cr, self.uid, [('session_id.date', '>=', date_start), ('session_id.date', '<=', date_stop), ('session_id.offer_id', '=', offer_id)])
            data = proxy_offer.browse(self.cr, self.uid, offer_id)
            values.append((sl_count, proxy_offer.browse(self.cr, self.uid, offer_id)))
        values.sort(reverse=True)
        return values

    def _get_sessions_search(self, state, date_start, date_stop):
        ids = []
        proxy = pooler.get_pool(self.cr.dbname).get('training.session')
        if state == 'duplicated':
            self.cr.execute('SELECT s.id '\
                            'FROM training_session s '\
                            'INNER JOIN training_group G ON s.id = G.session_id '\
                            'WHERE s.date BETWEEN %s AND %s '\
                            'GROUP BY s.id, s.name, s.date '\
                            'HAVING COUNT(G.id) > 1 '\
                            'ORDER BY id ',
                            (date_start, date_stop,))
            ids = [x[0] for x in self.cr.fetchall()]
        else:
            STATES = {
                'done' : ['done', 'closed'],
                'opened' : ['opened', 'opened_confirmed', 'closed_confirmed'],
                'inprogress' : ['inprogress'],
                'cancelled' : ['cancelled'],
            }

            ids = proxy.search(self.cr, self.uid, [('date', '>=', date_start),('date', '<=', date_stop),('state', 'in', STATES[state])])
        return ids


    def _get_sessions(self, state, date_start, date_stop):
        proxy = pooler.get_pool(self.cr.dbname).get('training.session')
        return proxy.browse(self.cr, self.uid, self._get_sessions_search(state, date_start, date_stop))

    def _get_sessions_count(self, state, date_start, date_stop):
        return len(self._get_sessions_search(state, date_start, date_stop))

    def _get_sessions_count_all(self, date_start, date_stop):
        value = 0
        for state in ['duplicated', 'done', 'opened', 'inprogress', 'cancelled']:
            value += self._get_sessions_count(state, date_start, date_stop)
        return value

    def _state2str(self, model, state):
        STATES = {
            'session' : {
                'draft' : _('Draft'),
                'opened' : _('Opened'),
                'opened_confirmed' : _('Confirmed'),
                'closed_confirmed' : _('Closed Subscriptions'),
                'inprogress' : _('In Progress'),
                'closed' : _('Closed'),
                'cancelled' : _('Cancelled'),
            },
            'subscription.line' : {
                'draft' : _('Draft'),
                'confirmed' : _('Confirmed'),
                'cancelled' : _('Cancelled'),
                'done' : _('Done'),
            }
        }

        return STATES[model].get(state, '')

report_sxw.report_sxw('report.training.session.volume',
                      'training.session',
                      'addons/training_extra_report/report/session_volume.rml',
                      parser=session_volume, header=True)

report_sxw.report_sxw('report.training.session.volume.theoretical',
                      'training.session',
                      'addons/training_extra_report/report/session_volume_theoretical.rml',
                      parser=session_volume, header=True)

report_sxw.report_sxw('report.training.session.report',
                      'training.session',
                      'addons/training_extra_report/report/session_report.rml',
                      parser=session_volume, header=True),

report_sxw.report_sxw('report.training.count.subscriptions.offer',
                      'training.subscription.line',
                      'addons/training_extra_report/report/count_subscriptions_offer.rml',
                      parser=session_volume, header=True)

report_sxw.report_sxw('report.training.subscriptions.partner',
                      'training.session',
                      'addons/training_extra_report/report/subscriptions_partner.rml',
                      parser=session_volume, header=True)
