# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-TODAY Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
import time
import tools
from tools.translate import _
from datetime import datetime as dt
from dateutil.relativedelta import relativedelta

class training_report_session_volume(osv.osv_memory):
    _name = 'training.report.session.volume'
    _columns = {
    'date_start' : fields.date('Start of period', required=True),
    'date_stop' : fields.date('End of period', required=True),
    'year' : fields.integer('Year', required=True),
    'kind' : fields.selection([
                  ('product_lines', 'Statistics by Product Lines'),
                  ('theoretical_product_lines', 'Theoretical Statistics by Product Lines'),
                  ('sessions', 'Sessions Volume by State'),
                  ('count_subscriptions_offer', 'Subscription Volume by State'),
                  #('subscriptions_partner', 'Partner Amount of Subscriptions'),
              ], 'Kind', required=True)
    }

    _defaults = {
        'date_start': lambda self, cr, uid, context: self.first_day_of_previous_month(cr, uid, context).strftime('%Y-%m-%d'),
        'date_stop': lambda self, cr, uid, context: self.last_day_of_previous_month(cr, uid, context).strftime('%Y-%m-%d'),
        'year': lambda *a: -1,
        'kind': lambda *a: 'product_lines',
    }


    def first_day_of_previous_month(self, cr, uid, context=None):
        return dt.strptime((dt.today() - relativedelta(months=1)).strftime('%Y-%m-01'), '%Y-%m-%d')

    def last_day_of_previous_month(self, cr, uid, context=None):
        return self.first_day_of_previous_month(cr, uid, context) + relativedelta(months=1, seconds=-1)

    def previous_date(self, cr, uid, date, year, context=None):
        return dt.strptime(date, '%Y-%m-%d') + relativedelta(years=year)

    def previous_period(self, cr, uid, date_start, date_stop, year, context=None):
        return (self.previous_date(cr, uid, date_start, year,context), self.previous_date(cr, uid, date_stop, year, context))

    def compute_theoretical_product_lines(self, cr, uid, ids, date_start, date_stop, kind, context=None):
        session_pool = self.pool.get('training.session')
        subl_pool = self.pool.get('training.subscription.line')
        prodline_pool = self.pool.get('training.course_category')
        offer_pool = self.pool.get('training.offer')
        trcv_pool = self.pool.get('training.report.costs.view')

        cache_seance = {}
        cache_session = {}
        product_line = {} # dict containing cost + revenue per product lines

        query_base = "SELECT * FROM training_report_costs_view "
        query = "WHERE seance_date BETWEEN %s AND %s AND offer_kind = %s"
        query_args = (date_start, date_stop, kind)

        if kind == 'intra':
            cr.execute("SELECT DISTINCT(session_id) FROM training_report_intra_session_seance_view " + query, query_args)
        else:
            cr.execute("SELECT DISTINCT(session_id) FROM training_report_costs_view " + query, query_args)
        session_ids = [ x[0] for x in cr.fetchall() ]
        trcv_pool.docache_session_populate(cr, uid, cache_session, session_ids)

        if kind == 'intra':
            cr.execute("SELECT DISTINCT(seance_id) FROM training_report_intra_session_seance_view " + query, query_args)
        else:
            cr.execute("SELECT DISTINCT(seance_id) FROM training_report_costs_view " + query, query_args)
        seance_ids = [ x[0] for x in cr.fetchall() ]
        trcv_pool.docache_seance_populate(cr, uid, cache_seance, seance_ids)

        if kind != 'intra':
            cr.execute(query_base + query, query_args)
            for theor_cost in cr.dictfetchall():
                c_session_id = theor_cost['session_id']
                c_seance_id = theor_cost['seance_id']

                if c_session_id not in cache_seance[c_seance_id]['sessions_ratio']:
                    # seance already cached, but we still need to update the session ratio
                    session_total_duration = cache_session[theor_cost['session_id']]['duration']
                    cache_seance[c_seance_id]['sessions_ratio'][c_session_id] = cache_seance[c_seance_id]['duration'] / session_total_duration

                # all things are cached now
                c_seance = cache_seance[theor_cost['seance_id']]
                c_prodline_id = theor_cost['product_line_id']
                if c_prodline_id not in product_line:
                    plo = prodline_pool.browse(cr, uid, c_prodline_id)
                    product_line[c_prodline_id] = {
                        'id': c_prodline_id,
                        'name': plo.name,
                        'costs': 0.0,
                        'revenues': 0.0,
                        'subscriptions': set(),
                    }
                pl = product_line[c_prodline_id]
                pl['costs'] += c_seance['trainer_cost'] + c_seance['proc_cost']
                pl['revenues'] += c_seance['sessions_ratio'][theor_cost['session_id']] * trcv_pool.get_sale_price(cr, uid, theor_cost, context=context)
                pl['subscriptions'].add(theor_cost['id'])
        else: # kind == 'intra'
            cr.execute("SELECT * FROM training_report_intra_session_seance_view " + query, query_args)
            for theor_intra in cr.dictfetchall():
                c_session_id = theor_intra['session_id']
                c_seance_id = theor_intra['seance_id']

                if c_session_id not in cache_seance[c_seance_id]['sessions_ratio']:
                    # seance already cached, but we still need to update the session ratio
                    session_total_duration = cache_session[theor_intra['session_id']]['duration']
                    cache_seance[c_seance_id]['sessions_ratio'][c_session_id] = cache_seance[c_seance_id]['duration'] / session_total_duration

                c_seance = cache_seance[theor_intra['seance_id']]
                c_session = cache_session[theor_intra['session_id']]

                c_session_ratio = cache_seance[c_seance_id]['sessions_ratio'][c_session_id]

                c_prodline_id = theor_intra['product_line_id']
                if c_prodline_id not in product_line:
                    plo = prodline_pool.browse(cr, uid, c_prodline_id)
                    product_line[c_prodline_id] = {
                        'id': c_prodline_id,
                        'name': plo.name,
                        'costs': 0.0,
                        'revenues': 0.0,
                        'subscriptions': set(),
                    }
                pl = product_line[c_prodline_id]
                pl['costs'] += c_session_ratio * theor_intra['session_cost']
                pl['revenues'] += c_session_ratio * theor_intra['session_revenue']
                for x in xrange(1, theor_intra['number_of_participants'] + 1):
                    pl['subscriptions'].add(x)

        res = []
        total = { 'revenues': 0.0, 'costs': 0.0 }
        for p, v in product_line.iteritems():
            res.append(v)
            v['subscriptions'] = len(v['subscriptions'])
            v['margin'] = (v['revenues'] / (v['costs'] or 1.0)) * 100
            v['balance'] = (v['revenues'] - v['costs'])
            total['revenues'] += v['revenues']
            total['costs'] += v['costs']

        total['balance'] = total['revenues'] - total['costs']
        total['margin'] = (total['revenues'] / (total['costs'] or 1.0)) * 100

        return {
            'product_lines': res,
            'total': total,
            'date_start': date_start,
            'date_stop': date_stop,
            'kind': kind,
        }

#    def compute_product_lines_exam(self, cr, uid, ids, date_start, date_stop, context=None):
#        if context is None:
#            context = {}
#        tsl_proxy = self.pool.get('training.subscription.line')
#        tsl_ids = tsl_proxy.search(cr, uid,
#                                   [
#                                       ('state', 'not in', ('draft', 'cancelled')),
#                                       ('date', '>=', "%s 00:00:00" % (date_start)),
#                                       ('date', '<=', "%s 23:59:59" % (date_stop)),
#                                       ('kind', '=', 'exam'),
#                                   ],
#                                   context=context)
#
#        product_lines = {}
#        for sl in tsl_proxy.browse(cr, uid, tsl_ids, context=context):
#            if not (sl.course_id and sl.course_id.category_id):
#                continue
#            product_line_id = sl.course_id.category_id
#            product_lines.setdefaults(product_line_id, [])
#            if sl.invoice_line_id and sl.invoice_id:
#                product_lines[product_line_id].append(sl)
#        return True

    def compute_product_lines(self, cr, uid, ids, date_start, date_stop, kind, context=None):
        res = []
        tsl_proxy = self.pool.get('training.subscription.line')
        tsl_ids = tsl_proxy.search(cr, uid,
                                   [
                                       ('state', 'not in', ('draft', 'cancelled')),
                                       ('date', '>=', "%s 00:00:00" % (date_start)),
                                       ('date', '<=', "%s 23:59:59" % (date_stop)),
                                       ('kind', '=', kind),
                                   ],
                                   context=context)
        product_lines = {}
        for sl in tsl_proxy.browse(cr, uid, tsl_ids, context=context):
            session_id = sl.session_id
            invoice_id = sl.invoice_id
            product_line_id = sl.session_id.offer_id.product_line_id
            product_lines.setdefault(product_line_id,
                                     {
                                         'invoices' : set(),
                                         'sessions' : {},
                                         'subscription_count' : 0,
                                         'exams' : {},
                                     })
            product_lines[product_line_id]['subscription_count'] += 1

            if invoice_id and sl.invoice_line_id:
                product_lines[product_line_id]['invoices'].add(invoice_id)

            if sl.session_id not in product_lines[product_line_id]['sessions']:

                po_pool = self.pool.get('purchase.order')
                sl_sess_seance_ids = [ seance.id for seance in session_id.seance_ids ]
                sl_sess_po = po_pool.search(cr, uid, [('seance_id','in',sl_sess_seance_ids)], context=context)
                value = sum(po.invoice_ids and sum([ inv.amount_untaxed for inv in po.invoice_ids]) or po.amount_untaxed
                            for po in po_pool.browse(cr, uid, sl_sess_po, context=context))
                #value = sum(po.procurement_id.purchase_id.amount_total
                #            for seance in session_id.seance_ids
                #            for po in seance.purchase_line_ids
                #            if po.procurement_id and po.procurement_id.purchase_id)

                value += sum(request.purchase_order_id.amount_untaxed
                             for request in session_id.request_ids
                             if request.purchase_order_id and request.state in ('accepted', 'done'))

                product_lines[product_line_id]['sessions'][sl.session_id] = value

            if sl.kind == 'exam' and sl.course_id and sl.course_id.category_id:
                product_lines[product_line_id]['exams'].setdefault(sl.course_id.category_id, {'subscription_count' : 0,
                                                                                              'price' : 0.0,
                                                                                             })
                product_line = product_lines[product_line_id]['exams'][sl.course_id.category_id]
                product_line['subscription_count'] += 1
                product_line['price'] += sl.price

        for pl, data in product_lines.iteritems():
            subscription_count = data['subscription_count']
            pl_input = sum(invoice_id.amount_untaxed for invoice_id in data['invoices'])
            pl_output = sum( data['sessions'].values() or [0.0])
            pl_balance = pl_input - pl_output
            pl_margin = (pl_input / (pl_output or 1.0)) * 100
            pl_average_price = subscription_count > 0 and (pl_input / subscription_count) or 0
            pl_yield = subscription_count > 0 and (pl_balance / subscription_count) or 0

            if ((-0.00001 < pl_input < 0.00001) or (-0.00001 < pl_output < 0.00001)):
                pl_margin = _('N/A')

            values = {
                'id' : pl.id,
                'name' : pl.name,
                'input' : pl_input,
                'output' : pl_output,
                'balance' : pl_balance,
                'subscription_count' : subscription_count,
                'margin' : pl_margin,
                'average_price' : pl_average_price,
                'yield' : pl_yield,
                'exams' : [(category.name, vals['subscription_count'], vals['price']) for category, vals in data['exams'].iteritems()]
            }

            res.append(values)

        values = {
            'subscription' : sum(line['subscription_count'] for line in res),
            'input' : sum(line['input'] for line in res),
            'output' : sum(line['output'] for line in res),
        }

        values['balance'] = values['input'] - values['output']
        if (-0.00001 < values['input'] < 0.00001) or (-0.00001 < values['output'] < 0.00001):
            values['margin'] = _('N/A')
        else:
            values['margin'] = (values['input'] / (values['output'] or 1.0)) * 100.0
        values['average_price'] = values['subscription'] > 0 and (values['input'] / values['subscription']) or 0
        values['yield'] = values['subscription'] > 0 and (values['balance'] / values['subscription']) or 0

        return {
            'product_lines' : res,
            'total' : values,
            'date_start' : date_start,
            'date_stop' : date_stop,
            'kind' : kind,
        }

    def compute_sessions(self, cr, uid, ids, date_start, date_stop, context):
        return {
            'lines' : [],
            'date_start' : date_start,
            'date_stop' : date_stop,
        }

    def compute_subscrtiption_partner(self, cr, uid, ids, date_start, date_stop, context):
        cr.execute("""
                   SELECT res.name, coalesce(sum(tsl.price), 0.0) as total_price
                   FROM training_subscription_line tsl,
                        res_partner res,
                        training_session s
                   WHERE res.id = tsl.partner_id
                     AND tsl.session_id = s.id
                   AND tsl.state IN ('confirmed', 'done')
                   AND s.date BETWEEN %s AND %s
                   GROUP BY tsl.partner_id,res.name
                   ORDER BY total_price DESC
                   """, (date_start, date_stop))

        return {
            'lines' : [{'name' : name, 'price' : price} for name, price in cr.fetchall()],
            'date_start' : date_start,
            'date_stop' : date_stop,
        }

    def compute_count_subscriptions_offer(self, cr, uid, ids, date_start, date_stop, context):
        return {
            'lines' : [],
            'date_start' : date_start,
            'date_stop' : date_stop,
        }

    def _select(self, cr, uid, data, context=None):
        functions = {
            'sessions' : self.compute_sessions,
            'count_subscriptions_offer' : self.compute_count_subscriptions_offer,
            'subscriptions_partner' : self.compute_subscrtiption_partner,
        }

        date_start = data['form']['date_start']
        date_stop = data['form']['date_stop']

        old_date_start, old_date_stop = self.previous_period(cr, uid, date_start, date_stop, data['form']['year'], context)

        old_date_start = old_date_start.strftime('%Y-%m-%d')
        old_date_stop = old_date_stop.strftime('%Y-%m-%d')

        if data['form']['kind'] == 'product_lines':
            for kind in ('standard', 'exam', 'intra'):
                data['form']['selected_period_%s' % kind] = self.compute_product_lines(cr, uid, data, date_start, date_stop, kind, context)
                data['form']['old_period_%s' % kind] = self.compute_product_lines(cr, uid, data, old_date_start, old_date_stop, kind, context)
        elif data['form']['kind'] == 'theoretical_product_lines':
            for kind in ('standard', 'exam', 'intra'):
                data['form']['selected_period_%s' % kind] = self.compute_theoretical_product_lines(cr, uid, data, date_start, date_stop, kind, context)
        else:
            function = functions[data['form']['kind']]
            data['form']['selected_period'] = function(cr, uid, data, date_start, date_stop, context)
            data['form']['old_period'] = function(cr, uid, data, old_date_start, old_date_stop, context)

        return data['form']

    def action_print(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        datas = {'form':{}, 'ids': context.get('active_ids', [])}
        res = self.read(cr, uid, ids, ['date_start', 'date_stop', 'year', 'kind'], context)
        res = res and res[0] or {}
        datas['form'].update(res)

        report_name = False
        if res.get('kind',False) == 'product_lines':
            report_name = 'training.session.volume'
        elif res.get('kind',False) == 'theoretical_product_lines':
            report_name = 'training.session.volume.theoretical'
        elif res.get('kind',False) == 'sessions':
            report_name = 'training.session.report'
        elif res.get('kind',False) == 'count_subscriptions_offer':
            report_name = 'training.count.subscriptions.offer'
        elif res.get('kind',False) == 'subscriptions_partner':
            report_name = 'training.subscriptions.partner'
        if not report_name:
            return {}

        new_datas = self._select(cr, uid, datas, context)
        datas['form'].update(new_datas)
        return {
            'type': 'ir.actions.report.xml',
            'report_name': report_name,
            'datas': datas,
        }

training_report_session_volume()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
