# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from report import report_sxw
import netsvc
import pooler
from report.pyPdf import PdfFileReader, PdfFileWriter
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

class training_offer_intra_detail_report(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(training_offer_intra_detail_report, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'has_one_session' : self._has_one_session,
            'get_seances' : self._get_seances,
        })

    def _has_one_session(self, offer_id):
        ids = self.pool.get('training.session').search(self.cr, self.uid, [('offer_id', '=', offer_id)])
        return ids and True or False

    def _get_seances(self, offer_id):
        proxy = self.pool.get('training.session')
        ids = proxy.search(self.cr, self.uid, [('offer_id', '=', offer_id)])
        if ids:
            res = proxy.browse(self.cr, self.uid, ids[0]).seance_ids
            return res
        return []


class report_sxw_offer_intra_detail(report_sxw.report_sxw):
    def create_single_pdf(self, cr, uid, ids, data, report_xml, context=None):
        if context is None:
            context = {}

        offer_pool = pooler.get_pool(cr.dbname).get('training.offer')
        sale_order_report = netsvc.LocalService('report.sale.order')

        def merge_data(dest, source_data):
            source = PdfFileReader(StringIO(source_data))
            for pn in range(source.getNumPages()):
                dest.addPage(source.getPage(pn))

        result = PdfFileWriter()
        for offer in offer_pool.browse(cr, uid, ids, context=context):
            id = offer.id
            (_data, _type) = super(report_sxw_offer_intra_detail, self).create_single_pdf(cr, uid, [id], data, report_xml, context=context)
            if _data:
                merge_data(result, _data)

            if offer.sale_order_id:
                report_context = context.copy()
                (_so_data, _so_type) = sale_order_report.create(cr, uid, [offer.sale_order_id.id], {}, context=report_context)
                if _so_data:
                    merge_data(result, _so_data)
        result_data = StringIO()
        result.write(result_data)
        return (result_data.getvalue(), 'pdf')

report_sxw_offer_intra_detail('report.training.offer.intra.detail.report',
                      'training.offer',
                      'addons/training_intra/report/detail_offer_intra.rml',
                      parser=training_offer_intra_detail_report,
                      header=True)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

