# -*- encoding: utf-8 -*-
############################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    Copyright (C) 2008-2009 AJM Technologies S.A. (<http://www.ajm.lu). All Rights Reserved
#    Copyright (C) 2010-2011 Thamini S.à.R.L (<http://www.thamini.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################################

from osv import osv
from osv import fields
from tools.translate import _
import decimal_precision as dp

class sale_order(osv.osv):
    _inherit = 'sale.order'

    _columns = {
        'create_date' : fields.datetime('Create Date', readonly=True),
    }

sale_order()

class training_session(osv.osv):
    _inherit = 'training.session'

    _columns = {
        'offer_partner_id' : fields.related('offer_id', 'partner_id', type='many2one', relation='res.partner', readonly=True, string='Partner', select=1),
        'offer_sale_order_id' : fields.related('offer_id', 'sale_order_id', type='many2one', relation='sale.order', readonly=True, string='Sale Order', select=1),
        'offer_sale_order_state' : fields.related('offer_id', 'sale_order_id', 'state', string="Sale Order's State", type='selection', readonly=True,
                                                  selection=[
                                                      ('draft','Quotation'),
                                                      ('waiting_date','Waiting Schedule'),
                                                      ('manual','Manual In Progress'),
                                                      ('progress','In Progress'),
                                                      ('shipping_except','Shipping Exception'),
                                                      ('invoice_except','Invoice Exception'),
                                                      ('done','Done'),
                                                      ('cancel','Cancel'),
                                                  ]
                                                 ),
    }

    def on_change_offer(self, cr, uid, ids, offer_id, context=None):
        if not offer_id:
            return False
        offer = self.pool.get('training.offer').browse(cr, uid, offer_id, context=context)
        return {
            'value' : {
                'kind' : offer.kind,
                'name' : offer.name,
                'location_id' : (offer.kind == 'intra' and offer.location_id) and offer.location_id.id or False,
                'delivery_location_id': (offer.kind == 'intra' and offer.location_id) and offer.location_id.id or False,
                'catalog_id' : False,
            }
        }

training_session()

class training_seance(osv.osv):
    _inherit = 'training.seance'

    def _is_intra_search(self, cr, uid, obj, name, args, context=None):
        if not len(args):
            return []
        cr.execute("""
                   SELECT DISTINCT(tssr.seance_id)
                     FROM training_session_seance_rel tssr,
                          training_session ts,
                          training_offer offer
                   WHERE tssr.session_id = ts.id
                     AND offer.id = ts.offer_id
                     AND offer.kind = 'intra'
                   """
                  )
        ids = [x[0] for x in cr.fetchall()]
        if ids:
            return [('id', 'in', ids)]
        return []

    def _is_intra_compute(self, cr, uid, ids, fields, args, context=None):
        res = dict.fromkeys(ids, 0)
        for obj in self.browse(cr, uid, ids, context=context):
            res[obj.id] = all(session.kind == 'intra' for session in obj.session_ids)
        return res

    _columns = {
        'is_intra' : fields.function(_is_intra_compute, method=True, fnct_search=_is_intra_search, type='boolean', string='Is Intra'),
        'partner_id' : fields.related('session_ids', 'offer_id', 'partner_id', type='many2one', relation='res.partner', readonly=True, string='Partner', select=1),
        'sale_order_id' : fields.related('session_ids', 'offer_id', 'sale_order_id', type='many2one', relation='sale.order', readonly=True, string='Sale Order'),
        'sale_order_state' : fields.related('session_ids', 'offer_id', 'sale_order_id', 'state', string="Sale Order's State", type='selection', readonly=True,
                                            selection=[
                                                ('draft','Quotation'),
                                                ('waiting_date','Waiting Schedule'),
                                                ('manual','Manual In Progress'),
                                                ('progress','In Progress'),
                                                ('shipping_except','Shipping Exception'),
                                                ('invoice_except','Invoice Exception'),
                                                ('done','Done'),
                                                ('cancel','Cancel'),
                                            ],
                                           ),
    }

training_seance()

class training_seance_purchase_line(osv.osv):
    _inherit = 'training.seance.purchase_line'

    def _get_analytic_account_id(self, cr, uid, ids, context=None):
        if not ids:
            return False
        po_line = self.browse(cr, uid, ids[0], context=context)
        if po_line.seance_id.is_intra:
            offer_record = po_line.seance_id.session_ids[0].offer_id
            if offer_record and offer_record.analytic_account_id:
                return offer_record.analytic_account_id.id
            else:
                return super(training_seance_purchase_line, self)._get_analytic_account_id(cr, uid, ids, context=context)
        else:
            return super(training_seance_purchase_line, self)._get_analytic_account_id(cr, uid, ids, context=context)

training_seance_purchase_line()

class training_intra_budget_tpl(osv.osv):
    _name = 'training.intra.budget.tpl'

    def _amount_all(self, cr, uid, ids, fieldnames, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, context=context):
            values = {'costs' : 0.0, 'revenues' : 0.0, 'balance' : 0.0}
            for line in obj.line_ids:
                if line.type == 'cost':
                    values['costs'] += line.price
                else:
                    values['revenues'] += line.price
                values['balance'] = values['revenues'] - values['costs']
            res[obj.id] = values
        return res

    _columns = {
        'name' : fields.char('Name', size=64, required=True),
        'line_ids' : fields.one2many('training.intra.budget.tpl.line', 'template_id', 'Budget Intra Template Lines'),
        'costs' : fields.function(_amount_all, method=True, multi="compute", string="Costs", type="float", store=True,
                                  readonly=True, digits_compute=dp.get_precision('Account')),
        'revenues' : fields.function(_amount_all, method=True, multi="compute", string="Revenues", type="float", store=True, readonly=True,
                                      digits_compute=dp.get_precision('Account')),
        'balance' : fields.function(_amount_all, method=True, multi="compute", string="Balance", type="float", store=True, readonly=True,
                                      digits_compute=dp.get_precision('Account')),
    }

training_intra_budget_tpl()

class training_intra_budget_tpl_line(osv.osv):
    _name = 'training.intra.budget.tpl.line'

    def _price_compute(self, cr, uid, ids, fieldnames, args, context=None):
        res = dict.fromkeys(ids, 0.0)
        for obj in self.browse(cr, uid, ids, context=context):
            res[obj.id] = obj.product_qty * obj.unit_price
        return res

    _columns = {
        'template_id' : fields.many2one('training.intra.budget.tpl', 'Template'),
        'course_id' : fields.many2one('training.course', 'Course'),
        'product_id' : fields.many2one('product.product', 'Product', required=True),
        'product_qty' : fields.float('Quantity', required=True),
        'product_uom' : fields.many2one('product.uom', 'Product UoM', required=True),
        'unit_price' : fields.float('Unit Price', required=True,  digits_compute=dp.get_precision('Account')),
        'price' : fields.function(_price_compute, method=True, store=True, string='Subtotal',
                                   digits_compute=dp.get_precision('Account'), type='float'),
        'type' : fields.selection([('cost', 'Cost'),('revenue', 'Revenue')], 'Type', required=True),
    }

    _defaults = {
        'product_qty' : lambda *a: 1.0,
        'type': lambda *a: 'cost',
    }

    def on_change_product(self, cr, uid, ids, kind, product_id):
        if not product_id:
            return {
                'value' : {
                    'unit_price' : 0.0,
                    'product_uom' : 0
                }
            }
        product = self.pool.get('product.product').browse(cr, uid, product_id)

        return {
            'value' : {
                'unit_price' : product.list_price,
                'product_uom' : product.uom_id.id
            }
        }

    def _check_product_qty(self, cr, uid, ids, context=None):
        return self.browse(cr, uid, ids[0], context=context).product_qty >= 0

    _constraints = [
        (_check_product_qty, "The quantity can not be negative", ['product_qty']),
    ]

training_intra_budget_tpl_line()

class training_offer(osv.osv):
    _inherit = 'training.offer'

    def can_be_planned_search(self, cr, uid, context=None):
        domain = ['|', ('state', '=', 'validated'), '&', ('kind', '=', 'intra'), ('state', '=', 'draft')]
        return self.search(cr, uid, domain, context=context)
        #return self.search(cr, uid, [('state', '=', 'validated')], context=context)

    def can_be_planned_compute(self, cr, uid, ids, context=None):
        return dict((obj.id, obj.state == 'validated') for obj in self.browse(cr, uid, ids, context=context))

    def _amount_all(self, cr, uid, ids, fieldnames, args, context=None):
        res = {}

        for obj in self.browse(cr, uid, ids, context=context):
            values = {'costs' : 0.0, 'revenues' : 0.0, 'balance' : 0.0}

            for line in obj.budget_intra_line_ids:
                if line.type == 'cost':
                    values['costs'] += line.price
                else:
                    values['revenues'] += line.price

                values['balance'] = values['revenues'] - values['costs']

            res[obj.id] = values

        return res

    def _get_offer_from_budget_lines(self, cr, uid, ids, context=None):
        budget_line_obj = self.pool.get('training.intra.budget.line')
        result = set(( line.offer_id.id for line in budget_line_obj.browse(cr, uid, ids, context=context) ))
        return list(result)


    def _get_intra_session_date(self, cr, uid, ids, fieldname, args, context=None):
        result = dict.fromkeys(ids, False)
        cr.execute("""SELECT toff.id AS offer_id,
                             MIN(ts.date) AS session_date
                      FROM training_offer toff
                      LEFT JOIN training_session ts ON (ts.offer_id = toff.id)
                      WHERE toff.kind = 'intra' AND ts.state != 'cancelled'
                      GROUP BY toff.id
                    """)
        for line in cr.dictfetchall():
            result[line['offer_id']] = line['session_date']
        return result

    def _search_intra_session_date(self, cr, uid, obj, fieldname, args, context=None):
        search_cond = []
        search_args = []
        for arg in args:
            if len(arg) == 3 and arg[0] == fieldname:
                if arg[1] in ('ilike', 'not ilike', '>', '>=', '<', '<='):
                    search_cond += [ "ts.date %s %%s" % (arg[1]) ]
                    search_args += [arg[2]]
        cr.execute("""SELECT toff.id AS offer_id,
                             MIN(ts.date) AS session_date
                      FROM training_offer toff
                      LEFT JOIN training_session ts ON (ts.offer_id = toff.id)
                      WHERE toff.kind = 'intra' AND ts.state != 'cancelled'
                      AND %s
                      GROUP BY toff.id
                    """ % (' AND '.join(search_cond)), tuple(search_args))
        result = [ line['offer_id'] for line in cr.dictfetchall() ]
        return [('id', 'in', result)]

    def _is_intra_started(self, cr, uid, ids, fname, args, context=None):
        res = dict.fromkeys(ids, False)

        is_started_ids = self.search(cr, uid, [('intra_session_started','=',True)])
        res.update(dict([(x,True) for x in is_started_ids]))

        return res

    def _is_intra_started_search(self, cr, uid, obj, name, args, context=None):
        # get condition
        cond = True
        for a in args:
            if len(a) == 3 and a[0] == 'intra_session_started':
                cond = a[2]
        # get 'intra' offer which have count(inprogress_or_closed_session) > 0
        cr.execute("""SELECT
                        toff.id as id,
                        SUM(CASE WHEN ts.state IN ('inprogress','closed') THEN 1 ELSE 0 END) as count_inprogress
                    FROM
                        training_offer toff
                    LEFT JOIN
                        training_session ts ON ts.offer_id = toff.id
                    WHERE
                        toff.kind = 'intra'
                    GROUP BY
                        toff.id,toff.name
                    HAVING
                        SUM(CASE WHEN ts.state IN ('inprogress','closed') THEN 1 ELSE 0 END) > 0
                    ORDER BY
                        toff.id""")
        result_ids = [ x['id'] for x in cr.dictfetchall() ]
        return [('id',(cond and 'in' or 'not in'), result_ids)]

    def copy_data(self, cr, uid, id, default=None, context=None):
        res = super(training_offer, self).copy_data(cr, uid, id, default=default, context=context)
        if not res:
            return res

        to_delete = [ 'sale_order_id', 'sale_order_create_date', 'sale_order_state' ]
        for key in to_delete:
            if key in res:
                del res[key]

        return res

    _columns = {
        'intra_session_started': fields.function(_is_intra_started, fnct_search=_is_intra_started_search, type='boolean',
                                string='Is Intra Session Started', method=True, readonly=True),
        'intra_session_date': fields.function(_get_intra_session_date, fnct_search=_search_intra_session_date, type='datetime',
                                        string='Intra Session Date', method=True, readonly=True),
        'costs' : fields.function(_amount_all, method=True, multi="compute", string="Costs", digits_compute=dp.get_precision('Account'),
                                  type="float",readonly=True,
                                  store={
                                      'training.intra.budget.line' : (_get_offer_from_budget_lines, None, 10),
                                  },
                                  ),
        'revenues' : fields.function(_amount_all, method=True, multi="compute", string="Revenues",
                                     digits_compute=dp.get_precision('Account'), type="float",
                                     store={
                                         'training.intra.budget.line' : (_get_offer_from_budget_lines, None, 10),
                                     },
                                     readonly=True),
        'balance' : fields.function(_amount_all, method=True, multi="compute", string="Balance",
                                    digits_compute=dp.get_precision('Account'), type="float",
                                    store={
                                        'training.intra.budget.line' : (_get_offer_from_budget_lines, None, 10),
                                    },
                                    readonly=True),
        'responsible' : fields.many2one('res.users', 'Project Manager', select=1),
        'partner_id' : fields.many2one('res.partner', 'Partner', select=1),
        'address_id' : fields.many2one('res.partner.address', 'Address'),
        'job_id' : fields.many2one('res.partner.job', 'Contact'),
        'job_email' : fields.char('Email', size=64),
        'analytic_account_id': fields.many2one('account.analytic.account', 'Analytic Account'),
        'sale_order_id' : fields.many2one('sale.order', 'Sale Order', readonly=True),
        'sale_order_create_date' : fields.related('sale_order_id', 'create_date', type='datetime', readonly=True, string="Sale Order's Create Date"),
        'sale_order_state' : fields.related('sale_order_id', 'state', string="Sale Order's State", type='selection', readonly=True,
                                            selection=[
                                                ('draft','Quotation'),
                                                ('waiting_date','Waiting Schedule'),
                                                ('manual','Manual In Progress'),
                                                ('progress','In Progress'),
                                                ('shipping_except','Shipping Exception'),
                                                ('invoice_except','Invoice Exception'),
                                                ('done','Done'),
                                                ('cancel','Cancel'),
                                            ],
                                           ),
        'budget_intra_template_id' : fields.many2one('training.intra.budget.tpl', 'Budget Intra Template'),
        'budget_intra_line_ids' : fields.one2many('training.intra.budget.line', 'offer_id', 'Budget Intra Lines'),
        'attachment_id' : fields.many2one('ir.attachment', 'Support of Course', domain="[('res_model', '=', 'training.course'),('type', '=', 'course_material')]"),
        'attachment_price' : fields.related('attachment_id', 'price', type='float', string="Support's Price", readonly=True,
                                            digits_compute=dp.get_precision('Account')),
        'note_intra' : fields.text('Note'), # note for internal usage
        'remark_intra' : fields.text('Remark(s)'), # node for client usage
        'coordinator' : fields.many2one('res.users', 'Coordinator'),
        'number_of_sessions' : fields.integer('Number of Sessions'),
        'number_of_participants' : fields.integer('Number of Participants', help='The number of participants of all sessions'),
        'max_participants_seance' : fields.integer('Maximum of Participants per Seance'),
        'approved_by' : fields.many2one('res.users', 'Approved By'),
        'lecturer_id' : fields.many2one('res.partner.job', 'Lecturer'),
        'location_id' : fields.many2one('training.location', 'Location'),
        'state' : fields.selection([('draft', 'Draft'),
                                    ('validated', 'Validated'),
                                    ('deprecated', 'Deprecated'),
                                    ('refused', 'Refused'),
                                    ('done', 'Done'),
                                   ],
                                   'State', required=True, readonly=True, help="The status of the course"),
    }

    _defaults = {
        'coordinator' : lambda obj, cr, uid, context: uid,
        'number_of_sessions' : lambda *a: 1,
        'number_of_participants' : lambda *a: 12,
    }

    def action_add_costs(self, cr, uid, ids, context=None):
        if not ids:
            return False
        this = self.browse(cr, uid, ids[0], context=context)
        proxy = self.pool.get('training.intra.budget.line')
        for course in this.course_ids:
            for proc_line in course.course_id.purchase_line_ids:
                values = {
                    'offer_id' : this.id,
                    'product_id' : proc_line.product_id.id,
                    'description' : proc_line.attachment_id and proc_line.attachment_id.name or proc_line.product_id.name,
                    'product_qty' : this.number_of_participants or 1,
                    'product_uom' : proc_line.product_uom.id,
                    'unit_price' : proc_line.product_price,
                    'course_id' : proc_line.course_id.id,
                }
                proxy.create(cr, uid, values, context=context)

            for budget_line in course.course_id.budget_intra_line_ids:
                values = {
                    'offer_id' : this.id,
                    'product_id' : budget_line.product_id.id,
                    'description' : budget_line.product_id.name,
                    'product_qty' : budget_line.product_qty,
                    'product_uom' : budget_line.product_uom.id,
                    'unit_price' : budget_line.unit_price,
                    'type' : budget_line.type,
                    'course_id' : course.course_id.id,
                }
                proxy.create(cr, uid, values, context=context)
        return True

    def action_compute_values(self, cr, uid, ids, context=None):
        return True

    def on_change_job(self, cr, uid, ids, job_id, context=None):
        email = ''
        if job_id:
            email = self.pool.get('res.partner.job').browse(cr, uid, job_id, context=context).email or ''
        return {'value' : {'job_email' : email}}

    def on_change_partner(self, cr, uid, ids, partner_id, context=None):
        if not partner_id:
            return {'value' : {'address_id' : 0}}
        partner = self.pool.get('res.partner').browse(cr, uid, partner_id, context=context)
        if partner.address:
            return {'value' : {'address_id' : partner.address[0].id}}

        return {'value' : {}}

    def on_change_budget_intra_template(self, cr, uid, ids, tpl_id, context=None):
        if not tpl_id:
            return False
        tpl_line_proxy = self.pool.get('training.intra.budget.tpl.line')
        line_ids = tpl_line_proxy.search(cr, uid, [('template_id', '=', tpl_id)], context=context)
        new_line_values = []
        for line in tpl_line_proxy.browse(cr, uid, line_ids, context=context):
            values = {
                'product_id' : line.product_id.id,
                'product_uom' : line.product_uom.id,
                'product_qty' : line.product_qty,
                'unit_price' : line.unit_price,
                'type' : line.type,
                'description': line.product_id.name
            }
            new_line_values.append(values)
        return {
            'value' : {
                'budget_intra_line_ids': new_line_values
            }
        }

    def action_create_quotation(self, cr, uid, ids, context=None):
        if not ids:
            return False
        proxy_adist = self.pool.get('account.analytic.plan.instance')
        proxy_adistline = self.pool.get('account.analytic.plan.instance.line')
        proxy = self.pool.get('sale.order')
        proxy_line = self.pool.get('sale.order.line')
        journal_proxy = self.pool.get('account.journal')
        this = self.browse(cr, uid, ids[0], context=context)

        if not this.partner_id:
            raise osv.except_osv(_('Warning'),
                                 _('The partner for the intra offer is empty'))
        if not this.address_id:
            raise osv.except_osv(_('Warning'),
                                 _('The address for the intra offer is empty'))

        if not this.state == 'validated':
            raise osv.except_osv(_('Error'),
                                 _('You need to validated the intra offer before creating the Sale Order'))

        values = {
            'partner_id' : this.partner_id.id,
            'pricelist_id' : this.partner_id.property_product_pricelist.id,
            'fiscal_position': this.partner_id.property_account_position and this.partner_id.property_account_position.id or False,
            'payment_term': this.partner_id.property_payment_term and this.partner_id.property_payment_term.id or False,
            'partner_order_id' : this.address_id.id,
            'partner_invoice_id' : this.address_id.id,
            'partner_shipping_id' : this.address_id.id,
            'order_policy': 'manual',
        }
        so_values = values
        adist_id = None

        if this.analytic_account_id:
            values['project_id'] = this.analytic_account_id.id
        else:
            if not len(this.course_ids):
                raise osv.except_osv(_('Warning'),
                                     _('The list of courses is empty !'))
            total_duration = 0.0
            for course_rel in this.course_ids:
                if course_rel.course_id.duration == 0.0:
                    raise osv.except_osv(_('Error'),
                                         _("The following course has not a valid duration \"%s\" (%d)") % (course_rel.course_id.name, course_rel.course_id.id))
                total_duration += course_rel.course_id.duration
            journal_sales_srch = journal_proxy.search(cr, uid, [('type','=','sale')])
            journal_sales = journal_proxy.browse(cr, uid, journal_sales_srch, context=context)[0]

            # Create 'analytic distribution instance'
            adist_id = proxy_adist.create(cr, uid, {
                'journal_id': journal_sales.analytic_journal_id.id,
            })

            for course_rel in this.course_ids:
                aaid = course_rel.course_id.analytic_account_id
                if not isinstance(aaid, (int, long, bool)):
                    aaid = aaid.id

                res = proxy_adistline.create(cr, uid, {
                    'plan_id': adist_id,
                    'analytic_account_id': aaid,
                    'rate': course_rel.course_id.duration * 100.0 / total_duration
                })

        order_id = proxy.create(cr, uid, values, context=context)

        for line in this.budget_intra_line_ids:
            if line.type == 'revenue':
                values = {
                    'order_id' : order_id,
                    'product_id': line.product_id.id,
                    'product_uom' : line.product_uom.id,
                    'product_uom_qty' : line.product_qty * (line.number_of_seances or 1.0),
                    'price_unit' : line.unit_price,
                    'name' : line.description,
                }
                if not this.analytic_account_id:
                    values['analytics_id'] = adist_id

                # get all common value from sale's order line product_id_change function
                onchange_values = proxy_line.product_id_change(cr, uid, [],
                                            so_values['pricelist_id'], values['product_id'],
                                            partner_id=so_values['partner_id'], flag=True,
                                            update_tax=True, fiscal_position=so_values['fiscal_position'])
#                import pprint
#                pprint.pprint(onchange_values)
                new_values = onchange_values.get('value', {})
                if 'tax_id' in new_values:
                    new_values['tax_id'] = [(6, 0, new_values['tax_id'])]
                new_values.update(values)

                proxy_line.create(cr, uid, new_values, context=context)

        return this.write({'sale_order_id' : order_id})

        #return {
        #    'view_type': 'form',
        #    "view_mode": 'form',
        #    'res_model': 'sale.order',
        #    'view_id': self.pool.get('ir.ui.view').search(cr,uid,[('name','=','sale.order.form')]),
        #    'type': 'ir.actions.act_window',
        #    'target': 'current',
        #    'res_id' : order_id,
        #}


training_offer()

class training_intra_budget_line(osv.osv):
    _name = 'training.intra.budget.line'

    def on_change_product(self, cr, uid, ids, kind, product_id):
        if not (product_id and kind):
            return {'value' : {'unit_price' : 0.0,
                               'product_uom' : 0}
                   }

        product = self.pool.get('product.product').browse(cr, uid, product_id)

        return {
            'value' : {
                'unit_price' : kind == 'revenue' and product.list_price or product.standard_price,
                'product_uom' : product.uom_id.id,
                'description' : product.name,
            }
        }

    def _price_compute(self, cr, uid, ids, fieldnames, args, context=None):
        res = dict.fromkeys(ids, 0.0)
        for obj in self.browse(cr, uid, ids, context=context):
            res[obj.id] = obj.product_qty * obj.unit_price * (obj.number_of_seances or 1.0)

        return res

    _columns = {
        'offer_id' : fields.many2one('training.offer', 'Offer', required=True, select=True),
        'description' : fields.char('Description', size=128, required=True),
        'product_id' : fields.many2one('product.product', 'Product', select=1),
        'product_qty' : fields.float('Quantity', required=True),
        'product_uom' : fields.many2one('product.uom', 'Product UoM', required=True),
        'unit_price' : fields.float('Unit Price', required=True, digits_compute=dp.get_precision('Account')),
        'price' : fields.function(_price_compute, method=True, string='Price', store=True, digits_compute=dp.get_precision('Account'), type='float'),
        'number_of_seances' : fields.integer('Recurrence', required=True),
        'type' : fields.selection([('cost', 'Cost'),('revenue', 'Revenue')], 'Type', required=True, select=1),
        'course_id' : fields.many2one('training.course', 'Course', select=1),
    }

    _defaults = {
        'product_qty' : lambda *a: 1.0,
        'number_of_seances' : lambda *a: 1,
        'type': lambda *a: 'cost',
    }

    def _check_product_qty(self, cr, uid, ids, context=None):
        return self.browse(cr, uid, ids[0], context=context).product_qty >= 0

    _constraints = [
        (_check_product_qty, "The quantity can not be negative", ['product_qty']),
    ]

    _order = "type asc"

training_intra_budget_line()


class training_course(osv.osv):
    _inherit = 'training.course'

    _columns = {
        'budget_intra_line_ids' : fields.one2many('training.intra.budget.tpl.line', 'course_id', 'Budget Intra Template Lines'),
    }

training_course()

class training_participation_stakeholder(osv.osv):
    _inherit = 'training.participation.stakeholder'

    def _pol_get_analytic_account(self, session_record, seance_record, context=None):
        if session_record.offer_id.kind == 'intra' \
                and session_record.offer_id.analytic_account_id:
                return session_record.offer_id.analytic_account_id.id
        # fallback to default behaviour
        return super(training_participation_stakeholder, self)._pol_get_analytic_account(session_record, seance_record, context=context)

training_participation_stakeholder()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
