# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from report import report_sxw

class account_analytic_balance_year(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):

        super(account_analytic_balance_year, self).__init__(cr, uid, name, context)
        self.localcontext.update( {
            'lines_g': self._lines_g,
            'move_sum_debit': self._move_sum_debit,
            'move_sum_credit': self._move_sum_credit,
            'sum_debit': self._sum_debit,
            'sum_credit': self._sum_credit,
            'sum_balance': self._sum_balance,
            'move_sum_balance': self._move_sum_balance,
            'date_set':self._get_data,
            'year_get':self._get_year,
        })

    def _lines_g(self, account_id, date1, date2):
        account_analytic_obj = self.pool.get('account.analytic.account')
        year = [int(date1[0:4]) - 1, int(date1[0:4])]
        ids = account_analytic_obj.search(self.cr, self.uid, [('parent_id', 'child_of', [account_id])])
        self.cr.execute("SELECT aa.name AS name, aa.code AS code, \
                    sum(aal.amount) AS balance, sum(aal.unit_amount) AS quantity \
                FROM account_analytic_line AS aal, account_account AS aa \
                WHERE (aal.general_account_id=aa.id) \
                    AND (aal.account_id in (" + ','.join(map(str, ids)) + "))\
                     AND to_char(date, 'MM-DD')>=%s AND to_char(date, 'MM-DD')<=%sAND aa.active \
                    And date_part('year',date) in ("+ ','.join(map(str, year)) +") \
                GROUP BY aal.general_account_id, aa.name, aa.code, aal.code ,date_part('year',date)\
                ORDER BY aal.code", (date1[5:10], date2[5:10]))
        result = self.cr.dictfetchall()

        for res in result:
            if res.get('balance') > 0:
                res.update({
                    'debit' : res.get('balance',0),
                    'credit' : 0.0
                })
            elif res.get('balance') < 0:
                res.update({
                    'debit' : 0.0,
                    'credit' : -res.get('balance',0)
                })
            else:
                res.update({
                    'balance' : 0.0,
                    'debit' : 0.0,
                    'credit' : -res.get('balance',0)
                })
        return result

    def _get_year(self,date1,date2):
        return [str(int(date1[0:4])-1), str(int(date1[0:4]))]

    def _get_data(self, account_id, date1, date2):
        year = [str(int(date1[0:4])-1), str(int(date1[0:4]))]
        debit = self._move_sum_debit(account_id, date1, date2)
        credit = self._move_sum_credit(account_id, date1, date2)
        amount = self._move_sum_balance(account_id, date1, date2)
        return {
                'debit' : debit,
                'credit' : credit,
                'amount' : amount,
                'year' : year
                }

    def _move_sum_debit(self, account_id, date1, date2):
        account_analytic_obj = self.pool.get('account.analytic.account')
        year = [int(date1[0:4])-1, int(date1[0:4])]
        ids = account_analytic_obj.search(self.cr, self.uid, [('parent_id', 'child_of', [account_id])])
        self.cr.execute("SELECT sum(amount)\
            FROM account_analytic_line \
            WHERE account_id in ("+ ','.join(map(str, ids)) +") \
             AND to_char(date, 'MM-DD')>=%s AND to_char(date, 'MM-DD')<=%s AND amount>0 \
                and date_part('year',date) in ("+ ','.join(map(str, year)) +") \
                group by date_part('year',date)",(date1[5:10], date2[5:10]))

        return self.cr.dictfetchall()

    def _move_sum_credit(self, account_id, date1, date2):
        account_analytic_obj = self.pool.get('account.analytic.account')
        year = [int(date1[0:4])-1, int(date1[0:4])]
        ids = account_analytic_obj.search(self.cr, self.uid, [('parent_id', 'child_of', [account_id])])
        self.cr.execute("SELECT -sum(amount) as sum \
            FROM account_analytic_line \
            WHERE account_id in ("+ ','.join(map(str, ids)) +") \
             AND to_char(date, 'MM-DD')>=%s AND to_char(date, 'MM-DD')<=%s AND amount>0 \
                and date_part('year',date) in ("+ ','.join(map(str, year)) +") \
                group by date_part('year',date)",(date1[5:10], date2[5:10]))
        return self.cr.dictfetchall()

    def _move_sum_balance(self, account_id, date1, date2):
        result = []
        debit = self._move_sum_debit(account_id, date1, date2)
        credit = self._move_sum_credit(account_id, date1, date2)
        for index in range(len(debit)):
            result.append(debit[index]['sum'] - credit[index]['sum'])
        return result

    def _move_sum_quantity(self, account_id, date1, date2):
        account_analytic_obj = self.pool.get('account.analytic.account')
        year = [int(date1[0:4])-1, int(date1[0:4])]
        ids = account_analytic_obj.search(self.cr, self.uid, [('parent_id', 'child_of', [account_id])])
        self.cr.execute("SELECT sum(unit_amount) as sum,date_part('year',date) as year \
                FROM account_analytic_line \
                WHERE account_id in ("+ ','.join(map(str, ids)) +") \
                    AND to_char(date, 'MM-DD')>=%s AND to_char(date, 'MM-DD')<=%s AND amount<0 \
                    and date_part('year',date) in ("+ ','.join(map(str, year)) +") \
                    group by date_part('year',date)",(date1[5:10], date2[5:10]))

        return self.cr.dictfetchall() or []

    def _sum_debit(self, accounts, date1, date2):
        account_analytic_obj = self.pool.get('account.analytic.account')
        year = [int(date1[0:4])-1, int(date1[0:4])]
        ids = map(lambda x: x.id, accounts)
        if not len(ids):
            return 0.0

        ids2 = account_analytic_obj.search(self.cr, self.uid, [('parent_id', 'child_of', ids)])
        self.cr.execute("SELECT sum(amount) as sum \
            FROM account_analytic_line \
            WHERE account_id in ("+ ','.join(map(str, ids2)) + " )  \
                AND to_char(date, 'MM-DD')>=%s AND to_char(date, 'MM-DD')<=%s AND amount>0 \
                 and date_part('year',date) in ("+ ','.join(map(str, year)) + " ) \
                 group by date_part('year',date) ",
            (date1[5:10], date2[5:10]))

        return self.cr.dictfetchall()

    def _sum_credit(self, accounts, date1, date2):
        account_analytic_obj = self.pool.get('account.analytic.account')
        year = [int(date1[0:4])-1, int(date1[0:4])]
        ids = map(lambda x: x.id, accounts)
        if not len(ids):
            return 0.0
        ids2 = account_analytic_obj.search(self.cr, self.uid, [('parent_id', 'child_of', ids)])
        self.cr.execute("SELECT -sum(amount) as sum \
                FROM account_analytic_line \
               WHERE account_id in ("+ ','.join(map(str, ids2)) +") \
                    AND to_char(date, 'MM-DD')>=%s AND to_char(date, 'MM-DD')<=%s AND amount>0 \
                     and date_part('year',date) in ("+ ','.join(map(str, year)) +") \
                     group by date_part('year',date) ",
                (date1[5:10], date2[5:10]))
        return self.cr.dictfetchall()

    def _sum_balance(self, accounts, date1, date2):
        debit = self._sum_debit(accounts, date1, date2) or []
        credit = self._sum_credit(accounts, date1, date2) or []
        result = []

        for index in range(len(debit)):
            result.append(debit[index]['sum'] - credit[index]['sum'])
        return result

report_sxw.report_sxw('report.account.analytic.account.balance.year',
        'account.analytic.account','addons/training_report/report/analytic_balance_year.rml',
        parser=account_analytic_balance_year, header=False)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
