import pooler
from report import report_sxw

class course_profitability(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(course_profitability, self).__init__(cr, uid, name, context)
        self.localcontext.update( {
            'lines_g': self._lines_g,
            'subscription_number': self._total_subscriptions,
            'cost_per_subscription':self._cost_per_subscription,
        })
        
    def _lines_g(self, account_id, account_fixed_cost, date1, date2):
        result = []
        account_analytic_obj = self.pool.get('account.analytic.account')
        ids = account_analytic_obj.search(self.cr, self.uid,
                [('parent_id', 'child_of', [account_id])])

        for acc_id in account_analytic_obj.browse(self.cr, self.uid, ids):
            res = {}
            res = {
                   'code' : acc_id.code or '',
                   'complete_name' : acc_id.complete_name,
                   'subscription_number': self._get_subscriptions(acc_id.id, date1, date2), 
                   'variable_costs': self._total_variable_costs(acc_id.id, account_fixed_cost, date1, date2),
                   'fixed_costs': acc_id.credit,
                   'revenues': acc_id.debit, 
                   'net_margin': self._net_margin(acc_id.id, account_fixed_cost), 
                   'margin_subscription': self._margin_subscription(acc_id.id, account_fixed_cost, date1, date2)
                   }
            result.append(res)
        return result

    def _total_subscriptions(self, account_id, date1, date2):
        res = {}
        self.cr.execute("select count(l.subscription_id) from training_subscription_line l \
                    where to_char(l.create_date,'YYYY-MM-DD') >= %s and to_char(l.create_date,'YYYY-MM-DD') <= %s",
                    (date1,date2))
        res = self.cr.dictfetchall()[0]
        return res
    
    def _cost_per_subscription(self, account_id, account_fixed_cost, date1, date2):
        subscription_nbr = self._total_subscriptions(account_id, date1, date2)
        if subscription_nbr['count']:
            cost_per_sub = account_fixed_cost / subscription_nbr['count']
        else:
            cost_per_sub = 1.0
        return cost_per_sub
    
    def _get_subscriptions(self, account_id, date1, date2):
        self.cr.execute("select count(l.subscription_id), c.analytic_account_id \
                        from training_subscription_line l, training_session s, training_offer o, training_course c \
                        where l.session_id = s.id and s.offer_id = o.id and \
                        c.id in (select course_id from training_course_offer_rel where offer_id=s.offer_id) and \
                        c.analytic_account_id = %s and \
                        to_char(l.create_date,'YYYY-MM-DD') >= %s and to_char(l.create_date,'YYYY-MM-DD') <= %s\
                        group by c.analytic_account_id",(account_id, date1, date2))
        ans = self.cr.dictfetchall()
        if ans:
            return ans[0]['count']
        else:
            return 0
        
    def _total_variable_costs(self, account_id, account_fixed_cost, date1, date2):
        sub_nbr = self._get_subscriptions(account_id, date1, date2)
        cost = self._cost_per_subscription(account_id, account_fixed_cost, date1, date2)
        var_cost = sub_nbr * cost
        return var_cost
    
    def _get_revenues(self, account_id):
        account_obj = self.pool.get('account.analytic.account')
        debit = account_obj.browse(self.cr, self.uid, account_id).debit
        return debit
    
    def _get_fixed_cost(self, account_id):
        account_obj = self.pool.get('account.analytic.account')
        credit = account_obj.browse(self.cr, self.uid, account_id).credit
        return credit
    
    def _net_margin(self, account_id, account_fixed_cost):
        debit = self._get_revenues(account_id)
        credit = self._get_fixed_cost(account_id)
        net_margin = debit - credit - account_fixed_cost
        return net_margin
    
    def _margin_subscription(self, account_id, account_fixed_cost, date1, date2):
        margin = self._net_margin(account_id, account_fixed_cost)
        sub_no = self._get_subscriptions(account_id, date1, date2)
        if sub_no:
            margin_subscription = margin/sub_no
        else:
            margin_subscription = 1.0
        return margin_subscription
        
report_sxw.report_sxw('report.training.course.profitability',
        'training.course','addons/training_report/report/course_profitability.rml',
        parser=course_profitability, header=False)
