# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
import time
from tools.sql import drop_view_if_exists
import xlwt
import base64
try:
    import cStringIO as StringIO
except ImportError:
    import StringIO

from training.training import training_offer_kind_compute
from training.training import training_course_kind_compute

# http://code.activestate.com/recipes/476197-first-last-day-of-the-month/#c1
def get_first_day(date, date_years=0, date_months=0):
    # date_years, date_months are "deltas" to apply to date
    year, month = date.year + date_years, datet.month + date_months
    avg, month = divmod(month - 1, 12)
    return date(year + avg, month + 1, 1)

def get_last_day(date):
    return get_first_day(date, 0, 1) + timedelta(-1)


class subscription_line(osv.osv):
    _inherit = 'training.subscription.line'
    _columns = {
        'product_id': fields.related('offer_id', 'product_id', string='Offer Product', type='many2one', relation='product.product', readonly=True, store=True),
    }

subscription_line()

REPORTS = {
    'sale.partner' : {
        'title' : 'Top Clients',
        'fields' : {
            'partner_id' : {
                'string' : 'Partner',
                'type' : 'many2one',
                'relation' : 'res.partner'
            },
            'turnover' : {
                'string' : 'Turnover',
                'type' : 'float',
            },
            'subscription_volume' : {
                'string' : 'Subscription Volume',
                'type' : 'integer',
            }
        },
        'header' : ['partner_id', 'turnover', 'subscription_volume'],
    },
    'sale.supplier': {
        'title': 'Top Suppliers',
        'fields': {
            'partner_id': {
                'string': 'Partner',
                'type': 'many2one',
                'relation': 'res.partner',
            },
            'turnover': {
                'string': 'Turnover',
                'type': 'float',
            },
            'request_count': {
                'string': 'Training Request Count',
                'type': 'integer',
            },
        },
        'header': ['partner_id', 'turnover', 'request_count'],
    },
    'subscription.offer': {
        'title' : 'Top Cours',
        'fields' : {
            'offer_id' : {
                'string' : 'Offer',
                'type' : 'many2one',
                'relation' : 'training.offer'
            },
            'turnover' : {
                'string' : 'Turnover',
                'type' : 'float',
            },
            'theoretical_turnover' : {
                'string' : 'Theoretical Turnover',
                'type' : 'float',
            },
            'subscription_volume' : {
                'string' : 'Subscription Volume',
                'type' : 'integer',
            },
            'session_nbr' : {
                'string' : 'Session Number',
                'type' : 'integer',
            },
            'subscription_rate' : {
                'string' : 'Subscription Rate',
                'type' : 'integer',
            }
        },
        'header' : ['offer_id', 'turnover', 'subscription_volume', 'session_nbr', 'subscription_rate'],
    }
}

class training_report_wizard(osv.osv):
    _name = 'training.report.wizard'

    _columns = {
        'date_start' : fields.date('Date Start', required=True),
        'date_stop' : fields.date('Date Stop', required=True),
        'state' : fields.selection([('init', 'Init'),('result', 'Result')], 'State', required=True, readonly=True),
        'report' : fields.selection([(report_name, report['title']) for report_name, report in REPORTS.iteritems()], 'Report', required=True),
        'filename' : fields.char('Filename', size=128, readonly=True),
        'file_data' : fields.binary('File', readonly=True),
    }

    _defaults = {
        'date_start': lambda *a: time.strftime('%Y-01-01'),
        'date_stop': lambda *a: time.strftime('%Y-12-31'),
        'report' : lambda *a: 'sale.partner',
        'state' : lambda *a: 'init',
    }

    def action_close(self, cr, uid, ids, context=None):
        return {'type' : 'ir.actions.act_window_close'}

    def action_generate_report(self, cr, uid, ids, context=None):
        if not ids:
            return False
        def execute_query(group_column, column_name):
            cr.execute("""
                SELECT l.%(gc)s as %(cn)s,
                       sum(a.price_subtotal) as turnover,
                       sum(l.price) as theoretical_turnover,
                       count(l.id) as subscription_volume
                  FROM training_subscription_line l
             LEFT JOIN training_session s ON s.id = l.session_id
       LEFT OUTER JOIN (SELECT il.id, il.price_subtotal
                          FROM account_invoice_line il
                     LEFT JOIN account_invoice i ON i.id = il.invoice_id
                         WHERE i.state IN ('open', 'paid')) a ON l.invoice_line_id = a.id
                 WHERE l.state in ('confirmed', 'done')
                   AND s.date BETWEEN %%s AND %%s
              GROUP BY l.%(gc)s
                       """ % {'gc' : group_column, 'cn' : column_name}
                       , (this.date_start, this.date_stop))

            return cr.fetchall()

        def execute_query_invoice(group_column, column_name):
            cr.execute("""
                SELECT ai.%(gc)s as %(cn)s,
                       sum(CASE WHEN ai.type = 'out_refund' THEN -ai.amount_untaxed ELSE ai.amount_untaxed END) as turnover,
                       sum(invoice_summary.subscription_count) AS subscription_volume
                  FROM account_invoice ai
             LEFT JOIN (select ai.id AS invoice_id, count(tsl.id) AS subscription_count
                        FROM account_invoice ai
                        LEFT JOIN account_invoice_line ail ON (ail.invoice_id = ai.id)
                        LEFT JOIN training_subscription_line tsl ON (tsl.invoice_line_id = ail.id)
                        GROUP BY ai.id) AS invoice_summary ON (invoice_summary.invoice_id = ai.id)
                 WHERE ai.type IN ('out_invoice','out_refund')
                   AND ai.state IN ('open','paid')
                   AND ai.date_invoice BETWEEN %%s AND %%s
              GROUP BY ai.%(gc)s
              ORDER BY turnover DESC, subscription_volume DESC
            """ % {'gc': group_column, 'cn': column_name}
            , (this.date_start, this.date_stop))
            return cr.fetchall()


        def execute_query_invoice_supplier(group_column, column_name):

            cr.execute("""
                SELECT ai.%(gc)s as %(cn)s,
                       sum(CASE WHEN ai.type = 'in_refund' THEN -ai.amount_untaxed ELSE ai.amount_untaxed END) as turnover,
                       COALESCE(sum(invoice_summary.request_count),0) AS request_count
                  FROM account_invoice ai
             LEFT JOIN (SELECT ai.id, count(tpsr.id) AS request_count FROM account_invoice ai, purchase_order po
                        LEFT JOIN purchase_invoice_rel pi ON (pi.purchase_id = po.id)
                        LEFT JOIN training_participation_stakeholder_request tpsr ON (tpsr.purchase_order_id = po.id)
                        GROUP BY ai.id) AS invoice_summary ON (ai.id = invoice_summary.id)
                 WHERE ai.type IN ('in_invoice','in_refund')
                   AND ai.state IN ('open','paid')
                   AND ai.date_invoice BETWEEN %%s AND %%s
              GROUP BY ai.%(gc)s
              ORDER BY turnover DESC, request_count DESC
            """ % {'gc': group_column, 'cn': column_name}
            , (this.date_start, this.date_stop))



            return cr.fetchall()

        def execute_query_offer(group_column, column_name):
            cr.execute("""
                            """ % {'gc': group_column, 'cn': column_name}
            , (this.date_start, this.date_stop))
            return cr.fetchall()

        def stream_write_row(row_id, values):
            row = stream.row(row_id)
            for idx, value in enumerate(values):
                if value is None:
                    continue
                xlwt.Row.set_cell_text(row, idx, value)
            return row_id + 1

        def write_to_xls(row_id, iterable):
            row = stream.row(row_id)

            for idx, column_name in enumerate(REPORTS[this.report]['header']):
                value = iterable[idx]
                if value is None:
                    continue

                field = REPORTS[this.report]['fields'][column_name]

                if field['type'] == 'many2one':
                    value = self.pool.get(field['relation']).browse(cr, uid, value, context=context)[field.get('name', 'name')]
                    xlwt.Row.set_cell_text(row, idx, value)
                elif field['type'] in ('float', 'integer'):
                    xlwt.Row.set_cell_number(row, idx, value)
                elif field['type'] in ('date', 'datetime'):
                    xlwt.Row.set_cell_date(row, idx, value)
                elif field['type'] == 'boolean':
                    xlwt.ROw.set_cell_boolean(row, idx, value)

            return row_id + 1

        # Use the current wizard
        this = self.browse(cr, uid, ids[0], context=context)

        # Create the temporary file
        import tempfile
        parent_directory = tempfile.mkdtemp(prefix='openerp_', suffix='_report')

        strIO = StringIO.StringIO()

        wb = xlwt.Workbook(encoding='utf-8')
        stream = wb.add_sheet(REPORTS[this.report]['title'])
        stream.portrait = 0 # Landscape

        row_id = 0
        row_id = stream_write_row(row_id, ['Report', REPORTS[this.report]['title']])
        row_id = stream_write_row(row_id, ['Date Start', this.date_start])
        row_id = stream_write_row(row_id, ['Date Stop', this.date_stop])

        row_id = stream_write_row(row_id, [REPORTS[this.report]['fields'][header]['string'] for header in REPORTS[this.report]['header']])
        length_headers = len(REPORTS[this.report]['header'])

        if this.report == 'sale.partner':
            for temp in execute_query_invoice('partner_id', 'partner_id'):
                row_id = write_to_xls(row_id, temp)
        elif this.report == 'sale.supplier':
            for temp in execute_query_invoice_supplier('partner_id', 'partner_id'):
                row_id = write_to_xls(row_id, temp)
        elif this.report == 'sale.user':
            for temp in execute_query('create_uid', 'user_id'):
                row_id = write_to_xls(row_id, temp)
        elif this.report == 'sale.offer':
            cr.execute("""
                SELECT l.offer_id as offer_id,
                       (select product_line_id from training_offer where id = l.offer_id) as product_line_id,
                       sum(a.price_subtotal) as turnover,
                       sum(l.price) as theoretical_turnover,
                       count(l.id) as subscription_volume
                  FROM training_subscription_line l
             LEFT JOIN training_session s ON s.id = l.session_id
       LEFT OUTER JOIN (SELECT il.id, il.price_subtotal
                          FROM account_invoice_line il
                     LEFT JOIN account_invoice i ON i.id = il.invoice_id
                         WHERE i.state IN ('open', 'paid')) a ON l.invoice_line_id = a.id
                 WHERE l.state in ('confirmed', 'done')
                   AND s.date BETWEEN %s AND %s
              GROUP BY l.offer_id
                       """, (this.date_start, this.date_stop))

            for temp in cr.fetchall():
                row_id = write_to_xls(row_id, temp)
        elif this.report == 'sale.product':
            for temp in execute_query('product_id', 'product_id'):
                row_id = write_to_xls(row_id, temp)
        elif this.report == 'subscription.offer':
            cr.execute("""
                    SELECT toff.id AS offer_id,
                           COALESCE(CASE WHEN toff.kind = 'intra' THEN sum(ai.amount_untaxed) ELSE sum(ail.price_subtotal) END, 0) AS turnover,
                           COUNT(tsl.id) AS subscription_volume,
                           COUNT(DISTINCT(ts.id)) AS session_nbr,
                           COUNT(tsl.id) * 1.0 / COUNT(DISTINCT(ts.id)) AS subscrition_rate
                      FROM training_offer toff
                 LEFT JOIN training_session ts ON (ts.offer_id = toff.id AND ts.state != 'cancelled')
                 LEFT JOIN training_subscription_line tsl ON (tsl.session_id = ts.id AND tsl.state IN ('confirmed', 'done'))
                 LEFT JOIN (SELECT il.id, il.price_subtotal
                              FROM account_invoice_line il
                         LEFT JOIN account_invoice i ON i.id = il.invoice_id
                             WHERE i.state IN ('open', 'paid')) ail ON tsl.invoice_line_id = ail.id
                 LEFT JOIN sale_order_invoice_rel soir ON (soir.order_id = toff.sale_order_id)
                 LEFT JOIN account_invoice ai ON (soir.invoice_id = ai.id)
                     WHERE ts.date BETWEEN %s AND %s
                  GROUP BY toff.id, toff.kind
                  ORDER BY turnover DESC
                       """, (this.date_start, this.date_stop))
            for temp in cr.fetchall():
                row_id = write_to_xls(row_id, temp)

        wb.save(strIO)

        values = {
            'state' : 'result',
            'filename' : '%s.xls' % REPORTS[this.report]['title'],
            'file_data' : strIO.getvalue().encode('base64'),
        }

        strIO.close()
        return self.write(cr, uid, ids, values, context=context)

training_report_wizard()

class training_report_intra_session_seance_view(osv.osv):
    _name = 'training.report.intra.session.seance.view'
    _table = 'training_report_intra_session_seance_view'
    _auto = False

    def init(self, cr):
        drop_view_if_exists(cr, 'training_report_intra_session_seance_view')
        cr.execute("""CREATE OR REPLACE VIEW training_report_intra_session_seance_view AS (

        SELECT  tsea.id AS seance_id,
                tsea.duration AS seance_duration,
                tsea.date AS seance_date,
                ts.date AS session_date,
                CASE WHEN toff.number_of_participants IS NULL OR toff.number_of_participants = 0.0 THEN 12 ELSE toff.number_of_participants END AS number_of_participants,
                SUM(CASE WHEN tibl.type = 'cost' THEN (tibl.unit_price * tibl.product_qty) ELSE 0.0 END) AS session_cost,
                SUM(CASE WHEN tibl.type = 'revenue' THEN (tibl.unit_price * tibl.product_qty) ELSE 0.0 END) AS session_revenue,
                tsea.course_id,ts.id AS session_id,
                toff.id AS offer_id,
                toff.kind AS offer_kind,
                toff.product_line_id AS product_line_id,
                CASE WHEN toff.analytic_account_id IS NOT NULL THEN toff.analytic_account_id ELSE tc.analytic_account_id END AS computed_analytic_id
        FROM training_seance tsea
        LEFT JOIN training_session_seance_rel tssr ON (tssr.seance_id = tsea.id)
        LEFT JOIN training_session ts ON (tssr.session_id = ts.id)
        LEFT JOIN training_offer toff ON (ts.offer_id = toff.id)
        LEFT JOIN training_course tc ON (tsea.course_id = tc.id)
        LEFT JOIN training_intra_budget_line tibl ON (tibl.offer_id = toff.id)
        WHERE toff.kind = 'intra'
        GROUP BY tsea.id, tsea.duration, tsea.date, ts.date, tsea.course_id, ts.id, toff.id, toff.kind, toff.analytic_account_id, tc.analytic_account_id, toff.number_of_participants,toff.product_line_id
        ORDER BY tsea.id
        )""")

    _columns = {
        'seance_id': fields.many2one('training.seance', 'Seance', readonly=True,),
        'session_id': fields.many2one('training.session', 'Session', readonly=True,),
        'offer_id': fields.many2one('training.offer', 'Offer', readonly=True,),
        'offer_kind': fields.char('Offer Kind', size=16, readonly=True,),
        'analytic_account_id': fields.many2one('account.analytic.account', 'Analytic Account', readonly=True,),
    }

training_report_intra_session_seance_view()

class training_report_session_stats_view(osv.osv):
    _name = 'training.report.session.stats.view'
    _table = 'training_report_session_stats_view'
    _auto = False

    _columns = {
        'session_id': fields.many2one('training.session', 'Session'),
        'subscription_count': fields.integer('Subscription Count'),
        'group_count': fields.integer('Group Count'),
        'global_seances_duration': fields.integer('Global Seances Duration'),
        'seances_duration': fields.integer('Seances Duration'),
        'participant_count': fields.integer('Participant Count'),
    }

    def init(self, cr):
        drop_view_if_exists(cr, 'training_report_session_stats_view')
        cr.execute("""CREATE OR REPLACE VIEW training_report_session_stats_view AS (

            SELECT
                ts.id AS session_id,
                count(tsl.id) AS subscription_count,
                ts.min_limit_reached AS min_limit_reached,
                CASE WHEN count(distinct(tg.id)) = 0 THEN 1 ELSE count(distinct(tg.id)) END AS group_count,
                COALESCE(session_seance_count.duration, 1) AS global_seances_duration,
                COALESCE(session_seance_count.duration, 1) / (CASE WHEN count(distinct(tg.id)) = 0 THEN 1 ELSE count(distinct(tg.id)) END) AS seances_duration,
                count(distinct(tsl.contact_id))
            FROM
                training_session ts
            LEFT JOIN
                training_subscription_line tsl ON (tsl.session_id = ts.id AND tsl.state in ('confirmed','done'))
            LEFT JOIN
                training_group tg ON (tg.session_id = ts.id)
            LEFT JOIN
                (select ts.id AS id, sum(tsea.duration) AS duration FROM training_session ts LEFT JOIN training_session_seance_rel tssr ON (tssr.session_id = ts.id) LEFT JOIN training_seance tsea ON (tssr.seance_id = tsea.id) GROUP BY ts.id) AS session_seance_count ON (session_seance_count.id = ts.id)
            GROUP BY
                ts.id, ts.min_limit_reached, session_seance_count.duration

        )""")
training_report_session_stats_view()

class training_report_seance_stats_view(osv.osv):
    _name = 'training.report.seance.stats.view'
    _table = 'training_report_seance_stats_view'
    _auto = False

    _columns = {
        'seance_id': fields.many2one('training.seance', 'Seance'),
        'kind': fields.char('Kind', size=32),
        'duration': fields.float('Duration'),
        'course_id': fields.many2one('training.course', 'Course'),
        'group_name': fields.char('Group Name', size=64),
        'participant_count': fields.integer('Participant Count'),
        'product_id': fields.many2one('product.product', 'Product'),
        'product_price': fields.float('Product Price'),
        'one_trainer_cost': fields.float('Uniq Trainer Cost'),
        'trainer_cost': fields.float('Trainer Cost'),
    }

    def init(self, cr):
        drop_view_if_exists(cr, 'training_report_seance_purchase_stats_view')
        drop_view_if_exists(cr, 'training_report_seance_stats_view')
        cr.execute("""CREATE OR REPLACE VIEW training_report_seance_stats_view AS (

                    select
                    	tsea.id AS seance_id,
                    	tsea.kind,
                            tsea.duration,
                            tsea.course_id,
                    	tg.name AS group_name,
                    	COALESCE(CASE WHEN (CASE WHEN tsea.manual THEN tsea.participant_count_manual ELSE count(tp.id) END ) <= 0 THEN 1 ELSE (CASE WHEN tsea.manual THEN tsea.participant_count_manual ELSE count(tp.id) END) END, 0) AS participant_count,
                    	CASE WHEN tsea.kind = 'exam' THEN pp.id ELSE tct.product_id END AS product_id,
                    	CASE WHEN tsea.kind = 'exam' THEN pt.standard_price ELSE pt2.standard_price END AS product_price,
                            COALESCE(tsea.duration * (CASE WHEN tsea.kind = 'exam' THEN pt.standard_price ELSE pt2.standard_price END) / (CASE WHEN (CASE WHEN tsea.manual THEN tsea.participant_count_manual ELSE count(tp.id) END ) <= 0 THEN 1 ELSE (CASE WHEN tsea.manual THEN tsea.participant_count_manual ELSE count(tp.id) END) END), 0.0) AS one_trainer_cost,

                    	COALESCE((tsea.duration * (CASE WHEN tsea.kind = 'exam' THEN pt.standard_price ELSE pt2.standard_price END) / (CASE WHEN (CASE WHEN tsea.manual THEN tsea.participant_count_manual ELSE count(tp.id) END ) <= 0 THEN 1 ELSE (CASE WHEN tsea.manual THEN tsea.participant_count_manual ELSE count(tp.id) END) END)) * (CASE WHEN tsea.kind = 'exam' AND tsea.course_id IS NULL THEN 2.0 ELSE 1.0 END), 0.0) AS trainer_cost


                    FROM
                    	training_seance tsea
                    LEFT JOIN
                    	training_participation tp ON (tp.group_id = tsea.group_id AND tp.seance_id = tsea.id)
                    INNER JOIN
                    	product_product pp ON (pp.default_code = 'R50H')
                    LEFT JOIN
                    	training_course tc ON (tsea.kind != 'exam' AND tsea.course_id = tc.id)
                    LEFT JOIN
                    	training_course_type tct ON (tc.course_type_id = tct.id)
                    LEFT JOIN
                    	training_group tg ON (tsea.group_id = tg.id)
                    LEFT JOIN
                            product_product pp2 ON (tct.product_id = pp2.id)
                    LEFT JOIN
                            product_template pt ON (pp.product_tmpl_id = pt.id)
                    LEFT JOIN
                            product_template pt2 ON (pp2.product_tmpl_id = pt2.id)
                    GROUP BY
                    	tsea.id, tsea.kind, tsea.duration, tsea.course_id, tg.name, tsea.manual, tsea.participant_count_manual, pp.id, tct.product_id, pt.standard_price, pt2.standard_price

                            )""")

training_report_seance_stats_view()

class training_report_seance_purchase_stats_view(osv.osv):
    _name = 'training.report.seance.purchase.stats.view'
    _table = 'training_report_seance_purchase_stats_view'
    _auto = False

    def init(self, cr):
        drop_view_if_exists(cr, 'training_report_seance_purchase_stats_view')
        cr.execute("""CREATE OR REPLACE VIEW training_report_seance_purchase_stats_view AS (
        SELECT tspl.id,
               tspl.seance_id,
               tspl.product_id,
               (CASE WHEN pp.procurement_method = 'fix' THEN 1 ELSE tpssv.participant_count END * tspl.product_qty) AS total_qty,
               (CASE WHEN pp.procurement_price = 'from_attachment' THEN tspl.attachment_id ELSE NULL END) AS attachment_id,
               pt.standard_price,
               COALESCE(sum(dpc.price), 0.0) AS attachment_price,
               (CASE WHEN pp.procurement_price = 'from_attachment' THEN COALESCE(sum(dpc.price),0.0) ELSE pt.standard_price END) AS price,
               pp.procurement_method,
               pp.procurement_price
        FROM training_seance_purchase_line tspl LEFT JOIN product_product pp ON (tspl.product_id = pp.id) LEFT JOIN training_report_seance_stats_view tpssv ON (tpssv.seance_id = tspl.seance_id) LEFT JOIN product_template pt ON (pp.product_tmpl_id = pt.id) LEFT JOIN document_price_component dpc ON (dpc.attach_id = tspl.attachment_id) GROUP BY tspl.id, tspl.seance_id,tspl.product_id,pp.procurement_method,pp.procurement_price,tpssv.participant_count,tspl.product_qty,pt.standard_price,tspl.attachment_id
        )""")

    _columns = {
        'seance_id': fields.many2one('training.seance', 'Seance', readonly=True),
        'product_id': fields.many2one('product.product', 'Product', readonly=True),
        'total_qty': fields.integer('Total Product Quantity', readonly=True),
        'attachment_id': fields.many2one('ir.attachment', 'Attachment', readonly=True),
        'standard_price': fields.float('Product Standard Price', readonly=True),
        'attachment_price': fields.float('Attachment Price', readonly=True),
        'price': fields.float('Purchase Line Price', readonly=True),
        'procurement_method': fields.char('Procurement Method', size=16, readonly=True),
        'procurement_price': fields.char('Procurement Price', size=16, readonly=True),
    }

training_report_seance_purchase_stats_view()

class training_report_costs_view(osv.osv):
    _name = 'training.report.costs.view'
    _table = 'training_report_costs_view'
    _auto = False

    _columns = {
        'session_id': fields.many2one('training.session', 'Session', readonly=True),
        'session_date': fields.datetime('Session Date', readonly=True),
        'seance_id': fields.many2one('training.seance', 'Seance', readonly=True),
        'seance_kind': fields.selection(training_course_kind_compute, 'Seance Kind', readonly=True),
        'seance_date': fields.datetime('Seance Date', readonly=True),
        'offer_kind': fields.selection(training_offer_kind_compute, 'Offer Kind', readonly=True),
        'offer_id': fields.many2one('training.offer', 'Offer', readonly=True),
        'computed_course_id': fields.many2one('training.course', 'Course', readonly=True),
        'computed_analytic_id': fields.many2one('account.analytic.account', 'Analytic Account', readonly=True),
        'product_line_id': fields.many2one('training.course_category', 'Product Line', readonly=True),
    }

    def init(self, cr):
        drop_view_if_exists(cr, 'training_report_costs_view')
        cr.execute("""CREATE OR REPLACE VIEW training_report_costs_view AS (
            SELECT tsl.id AS id,
                   tsl.price AS subl_price,
                   ts.id AS session_id,
                   ts.date AS session_date,
                   tsea.id AS seance_id,
                   tsea.kind AS seance_kind,
                   tsea.date as seance_date,
                   tsea.group_id AS seance_group_id,
                   toff.kind AS offer_kind,
                   toff.id AS offer_id,
                   CASE WHEN tc_seance.id IS NULL THEN tc_subl.id ELSE tc_seance.id END AS computed_course_id,
                   CASE WHEN tc_seance.analytic_account_id IS NULL THEN tc_subl.analytic_account_id ELSE tc_seance.analytic_account_id END AS computed_analytic_id,
                   CASE WHEN toff.kind = 'exam' THEN tc_subl.category_id ELSE toff.product_line_id END AS product_line_id
            FROM training_subscription_line tsl
            LEFT JOIN training_session ts ON tsl.session_id = ts.id
            LEFT JOIN training_session_seance_rel tssr ON tssr.session_id = ts.id
            LEFT JOIN training_seance tsea ON tssr.seance_id = tsea.id
            LEFT JOIN training_course tc_seance ON tsea.course_id = tc_seance.id
            LEFT JOIN training_course tc_subl ON tsl.course_id = tc_subl.id
            LEFT JOIN training_offer toff ON ts.offer_id = toff.id
            LEFT JOIN sale_order so ON toff.sale_order_id = so.id
            WHERE tsl.state in ('confirmed','done')
              AND ts.min_limit_reached = True
            ORDER BY tsl.id,
                     ts.id,
                     tsea.id,
                     computed_course_id,
                     computed_analytic_id
        ) """)

    def docache_session_populate(self, cr, uid, cache_session, session_ids):
        if not session_ids:
            return
        cr.execute("SELECT * FROM training_report_session_stats_view WHERE session_id IN (%s)" % (','.join(map(str, session_ids)),))
        for sess_stats in cr.dictfetchall():
            sess_stats['duration'] = sess_stats['seances_duration']
            cache_session[sess_stats['session_id']] = sess_stats
        return True

    def docache_seance_populate(self, cr, uid, cache_seance, seance_ids):
        if not seance_ids:
            return
        query_seance_ids = ','.join(map(str, seance_ids))
        # pre-compute procurement price
        seance_procs_cache = {}
        for seance_id in seance_ids:
            seance_procs_cache[seance_id] = {
                            'proc_fix': 0.0,
                            'proc_sub': 0.0,
                        }
        cr.execute("SELECT * FROM training_report_seance_purchase_stats_view trspsv WHERE seance_id IN (%s)" % (query_seance_ids))
        for purch_stat in cr.dictfetchall():
            price = purch_stat['price']

            if purch_stat['procurement_method'] == 'fix':
                seance_procs_cache[purch_stat['seance_id']]['proc_fix'] += price * float(purch_stat['total_qty'])
            else:
                seance_procs_cache[purch_stat['seance_id']]['proc_sub'] += price

        cr.execute("SELECT * FROM training_report_seance_stats_view WHERE seance_id IN (%s)" % (query_seance_ids))
        for seance_stats in cr.dictfetchall():
            seance_stats['group'] = seance_stats['group_name']
            seance_stats['participants'] = seance_stats['participant_count']
            seance_stats['proc_cost'] = seance_procs_cache[seance_stats['seance_id']]['proc_sub'] + (seance_procs_cache[seance_stats['seance_id']]['proc_fix'] / (seance_stats['participant_count'] or 1.0))
            if seance_stats['kind'] == 'exam':
                seance_stats['proc_cost'] += 1.0 # add 1 euro per participant questionnaire
            seance_stats['sessions_ratio'] = {}

            cache_seance[seance_stats['seance_id']] = seance_stats
        return True

    def docache_session(self, cache_session, session):
        if session.id not in cache_session:
            cache_session[session.id] = {
                'participant_count': session.participant_count,
                'subscription_count': session.confirmed_subscriptions,
                'duration': (sum([ seance.duration for seance in session.seance_ids]) or 1.0) / (float(len(session.group_ids)) or 1.0)
            }
        return True

    def docache_seance(self, cr, uid, cache_seance, seance, context=None):
        if seance.id not in cache_seance:
            if seance.manual:
                pcount = seance.participant_count_manual
            else:
                pcount = len([ x for x in seance.participant_ids if x.group_id.id == seance.group_id.id ])
            pcount = pcount or 1.0 # make sure pcount is not 0, overwise we will get ZeroDivisionError
            # calc trainer costs
            if seance.kind == 'exam':
                p_id = self.pool.get('product.product').search(cr, uid, [('default_code','=','R50H')])[0]
                trainer_product = self.pool.get('product.product').browse(cr, uid, p_id, context=context)
            else:
                trainer_product = seance.course_id.course_type_id.product_id

            trainer_cost = seance.duration * trainer_product.standard_price / pcount

            if seance.kind == 'exam' and not seance.course_id:
                # double trainer cost as we except at least 2 trainer per generic 'exam seance'
                trainer_cost = trainer_cost * 2.0

            # calc procurement costs
            proc_cost = 0.0
            proc_fix_sum = 0.0
            for proc in seance.purchase_line_ids:
                if proc.fix == 'fix':
                    proc_fix_sum += proc.product_price * proc.product_qty
                else:
                    proc_cost += proc.product_price * proc.product_qty
            # we divide fix costs on all participants
            proc_cost += proc_fix_sum / pcount

            if seance.kind == 'exam':
                # considerer questionnaire cost 1€ per participant
                proc_cost += 1.0

            cache_seance[seance.id] = {
                'duration': seance.duration,
                'trainer_cost': trainer_cost,
                'proc_cost': proc_cost,
                'participants': pcount,
                'group': seance.group_id.name,
                'sessions_ratio': {},
            }
        return True

#        print("          group class      : %s" % (cache_seance[seance.id]['group']))
#        print("          trainer cost     : %s" % (cache_seance[seance.id]['trainer_cost']))
#        print("          procurement cost : %s" % (cache_seance[seance.id]['proc_cost']))
#        print("          participant cnt  : %s" % (cache_seance[seance.id]['participants']))

    def get_sale_price(self, cr, uid, theor_cost, context=None):
        offer_pool = self.pool.get('training.offer')
        subl_pool = self.pool.get('training.subscription.line')
        price = 0.0
        if theor_cost.get('offer_kind',False) == 'intra':
            offer_data = offer_pool.browse(cr, uid, theor_cost['offer_id'], context=context)
            participant_count = offer_data.number_of_participants or 12
            price = (offer_data.sale_order_id and offer_data.sale_order_id.amount_untaxed or 0.0) / participant_count
        else:
            #price = subl_pool.read(cr, uid, theor_cost['id'], ['price'])['price']
            price = theor_cost.get('subl_price')
        return price

training_report_costs_view()

class training_report_confirmed_subscription_count(osv.osv):
    _name = 'training.report.confirmed.subscription.count'
    _table = 'training_report_confirmed_subscription_count'
    _auto = False
    _order = 'date'

    def init(self, cr):
        drop_view_if_exists(cr, 'training_report_confirmed_subscription_count')
        cr.execute("""CREATE OR REPLACE VIEW training_report_confirmed_subscription_count AS (
        SELECT  to_number(to_char(ts.date, 'YYYYMM01')||CASE WHEN toff.kind = 'standard' THEN '1' WHEN toff.kind = 'exam' THEN '2' WHEN toff.kind = 'intra' THEN '3' ELSE '0' END, '999999999') AS id,
                to_char(ts.date, 'YYYY-MM-01') AS date,
                toff.kind AS session_kind,
                sum(CASE WHEN tsl.state IN ('confirmed','done') THEN 1 ELSE 0 END) AS subscription_count,
                sum(CASE WHEN tsl.state = 'cancelled' THEN 1 ELSE 0 END) AS cancelled_subscription_count,
                sum(CASE WHEN tsl.state = 'draft' THEN 1 ELSE 0 END) AS draft_subscription_count
        FROM training_subscription_line tsl
        LEFT JOIN training_session ts ON (tsl.session_id = ts.id)
        LEFT JOIN training_offer toff ON (ts.offer_id = toff.id)
        GROUP BY to_number(to_char(ts.date, 'YYYYMM01')||CASE WHEN toff.kind = 'standard' THEN '1' WHEN toff.kind = 'exam' THEN '2' WHEN toff.kind = 'intra' THEN '3' ELSE '0' END, '999999999'),
                 to_char(ts.date, 'YYYY-MM-01'),
                 toff.kind
        ORDER BY to_char(ts.date, 'YYYY-MM-01')
        )""")

    _columns = {
        'date': fields.date('Date', readonly=True),
        'session_kind': fields.selection([('exam', 'Examen'),('intra','Intra'),('standard','Standard')], 'Session Kind', readonly=True),
        'subscription_count': fields.integer('Subscriptions', readonly=True),
        'cancelled_subscription_count': fields.integer('Cancelled Subscriptions', readonly=True),
        'draft_subscription_count': fields.integer('Draft Subscriptions', readonly=True),
    }

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}
        if context.get('date_filter', '') == 'current_year':
            args.extend([('date','>=',time.strftime('%Y-01-01')),('date','<=',time.strftime('%Y-12-31'))])
#        if context.get('date_filter', '') == 'current_month':
#            month_start = time.strftime('%Y-%m-%d')
#            month_start_dt = datetime.datetime(map(int, month_start.split('-')))
#            month_end = get_last_day(month_start_dt, d_month=1).strftime('%Y-%m-%d')
#            args.extend
        return super(training_report_confirmed_subscription_count, self).search(cr, uid, args, offset=offset, limit=limit, order=order, context=context, count=count)

training_report_confirmed_subscription_count()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
