# -*- encoding: utf-8 -*-
############################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    Copyright (C) 2008-2009 AJM Technologies S.A. (<http://www.ajm.lu). All Rights Reserved
#    Copyright (C) 2010-2011 Thamini S.à.R.L (<http://www.thamini.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
############################################################################################

from osv import osv, fields
import netsvc
import time

class res_partner(osv.osv):
    _inherit = 'res.partner'

    _columns = {
        'notify_location' : fields.many2one('res.partner.job', 'Notification for Location',
                                        domain="[('name', '=', active_id),('state', '=', 'current')]"),
    }

res_partner()

class training_location(osv.osv):
    _name = 'training.location'
    _description = 'Location'

    _columns = {
        'name' : fields.char('Room', size=64, select=True, required=True),
        'partner_id' : fields.many2one('res.partner', 'Partner', required=True, select=1),
        'address_id' : fields.many2one('res.partner.address', 'Address', required=True,
                                      domain="[('partner_id','=',partner_id)]"),
        'seats' : fields.integer('Seats',help='Total Seats'),
    }

    _defaults = {
        'seats' : lambda *a: 0,
    }

    def name_get(self, cr, uid, ids, context=None):
        res = []
        for obj in self.browse(cr, uid, list(set(ids)), context=context):
            res.append((obj.id, "%s (%s)" % (obj.name, obj.partner_id.name,)))
        return res

    def name_search(self, cr, uid, name, args=None, operator='ilike', context=None, limit=80):
        search_ids = self.search(cr, uid, ['|',('name',operator,name),('partner_id',operator,name)], context=context)
        return self.name_get(cr, uid, search_ids, context=context)

    def on_change_partner(self, cr, uid, ids, partner_id, context=None):
        if not partner_id:
            return False

        values = {
            'address_id' : 0,
        }

        proxy = self.pool.get('res.partner.address')
        ids = proxy.search(cr, uid, [('partner_id', '=', partner_id),('type', '=', 'default')])

        if not ids:
            ids = proxy.search(cr, uid, [('partner_id', '=', partner_id)])

        if ids:
            values['address_id'] = ids[0]

        return {'value' : values}

    def _check_address(self, cr, uid, ids, context=None):
        location_proxy = self.pool.get('training.location')

        location = location_proxy.browse(cr, uid, ids[0], context=context)

        address_proxy = self.pool.get('res.partner.address')

        return address_proxy.search_count(cr, uid, [('partner_id', '=', location.address_id.id)]) > 0

    def _check_seats(self, cr, uid, ids, context=None):
        return self.browse(cr, uid, ids, context=context)[0].seats > 0

    _constraints = [
        #(_check_address, "Can you check the address ?", ['partner_id', 'address_id']),
        (_check_seats, "This room has not seats !", ['seats']),
    ]

training_location()

class training_seance(osv.osv):
    _inherit = 'training.seance'

    _columns = {
        'location_id' : fields.many2one('training.location', 'Training Location', select=1,help='The location where the training will take place'),
        'location_note': fields.text('Training Location Note', help='Give some extra informations about the training location'),
        'delivery_location_id': fields.many2one('training.location', 'Training Delivery Location', select=1, help='The location where training procurements (ex: support of course) will be delivered'),
        'delivery_location_note': fields.text('Training Delivery Location Note', help='Give some extra informations about the training delivery location'),
        'reserved' : fields.boolean('Reserved',help='Location is reserved '),
        'seats' : fields.related('location_id', 'seats', type='integer', string='Maximum Seats',
                                 readonly=True,help='Maximum seats available in location')
    }

    def _default_get_location_id(self, cr, uid, context=None):
        # FIXME: get default for a core_config option
        cr.execute("SELECT min(id) from training_location")
        location = cr.fetchone()
        if not location or (len(location) > 0 and not location[0]):
            location = False
        return location and len(location) and location[0] or False

    _defaults = {
        'reserved' : lambda *a: 0,
        'location_id': _default_get_location_id,
        'delivery_location_id': _default_get_location_id,
    }

    # training.seance
    #def copy(self, cr, uid, seance_id, defaults=None, context=None):
    #    defaults['location_id'] = 0
    #    defaults['reserved'] = 0
    #    return super(training_seance, self).copy(cr, uid, seance_id, defaults, context)


training_seance()

class training_session(osv.osv):
    _inherit = 'training.session'
    _columns = {
        'location_id' : fields.many2one('training.location', 'Location', select=1,help='The location where the training will take place'),
        'location_note': fields.text('Training Location Note', help='Give some extra informations about the training location'),
        'delivery_location_id': fields.many2one('training.location', 'Training Delivery Location', select=1, help='The location where training procurements (ex: support of course) will be delivered'),
        'delivery_location_note': fields.text('Training Delivery Location Note', help='Give some extra informations about the training delivery location'),
        'seats' : fields.related('location_id', 'seats', type='integer', string='Maximum Seats',
                                 readonly=True,help='Maximum seats available in location')
    }

    #def copy(self, cr, uid, session_id, defaults=None, context=None):
    #    defaults['location_id'] = 0
    #    return super(training_session, self).copy(cr, uid, session_id, defaults, context)


    def _create_seance(self, cr, uid, session, context=None):
        seance_ids = super(training_session, self)._create_seance(cr, uid, session, context=context)
        seance_vals = {}
        if session.location_id:
            seance_vals.update({
                         'location_id' : session.location_id.id,
                         'delivery_location_id' : session.location_id.id
                         }) # will be overwritten if a the "delivery location" is really filled up
        if session.delivery_location_id:
            seance_vals.update({'delivery_location_id' : session.delivery_location_id.id})
        if session.location_note:
            seance_vals.update({'location_note' : session.location_note})
        if session.delivery_location_note:
            seance_vals.update({'delivery_location_note' : session.delivery_location_note})
        if seance_vals:
            self.pool.get('training.seance').write(cr, uid, seance_ids, seance_vals, context=context)
        return seance_ids

training_session()

class training_participation_stakeholder(osv.osv):
    _inherit = 'training.participation.stakeholder'
    _columns = {
        'seance_location_id': fields.related('seance_id', 'location_id', type='many2one', relation='training.location', string='Location', readonly=True),
    }

training_participation_stakeholder()

class purchase_order_line_seance_room(osv.osv):
    _inherit = 'purchase.order.line'

    _columns = {
        'location_id': fields.related('seance_id', 'location_id', type='many2one', relation='training.location', string='Training Location', readonly=True),
        'delivery_location_id': fields.related('seance_id', 'delivery_location_id', type='many2one', relation='training.location', string='Delivery Location', readonly=True),
    }

purchase_order_line_seance_room()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
